package
{
	import org.flixel.*;

	/**
	 * The chat bubble is supposed to be bound to
	 * anything / anyone, with the ability to automatically
	 * update its location and height to show everything the
	 * character has to say
	 */
	public class ChatBubble extends FlxSprite
	{
		[Embed(source="../data/fonts/DigitalStrip2BB_reg.TTF",fontFamily="Dialog")] protected var junk:String; 
		/**
		 * Point at which the arrow of the bubble points to
		 */
		private var _focus:FlxObject;
		/**
		 * FlxText representing the text of the bubble
		 */
		private var _text:FlxText;
		private var _name:String;
		private var _blah:String;
		/**
		 * Timer for determining fade time
		 */
		private var _fadeTimer:Number;
		
		/**
		 * 	0: right
		 *  1: left
		 **/
		private var _side:int;
		
		public var _character:Character;
		public var _portrait:FlxSprite;
		
		[Embed(source="../data/chatbubble.png")] private var ImgBubble:Class;
		[Embed(source="../data/brad_portrait.png")] private var ImgPortrait:Class;
		
		public function ChatBubble(character:Character)
		{
			_fadeTimer = 3;
			_character = character;
			_text = new FlxText(0, 0, 200, "");
			_text.size = 16;
			_text.alignment = "left";
			_text.color = 0x000000;
			_text.font = "Dialog"; 
			
			//super(0, 0, ImgBubble);
			super(0, 0);
			loadGraphic(ImgBubble,true,true,290,300);
			//loadGraphic(ImgBubble, true);
			//addAnimation("right", [0]);
			//addAnimation("left", [1]);
			
			//width = 225;
			//height = 200;
			
			_portrait = new FlxSprite(x+5, y+5);
			_portrait.loadGraphic(ImgPortrait, true, true, 80, 100);
			_portrait.addAnimation("normal", [0]);
			_portrait.play("normal");
			
			_portrait.x = x + 5;
			_portrait.y = y + 8;
			
			_text.x = _portrait.x + _portrait.width + 5;
			_text.y = y + 8;
			
			_side = 0;
			
			solid = false;
			fixed = true;
			moves = false;
			Hide();
		}
		
		public function bindCharacter(character:Character):void
		{
			_character = character;	
			update();
		}
		
		override public function update():void
		{
			super.update();
			_text.update();
			_portrait.update();
			
			x = _character.x + _character.width;
			y = _character.y + _character.height/3 - height;
			
			_portrait.x = x + 5;
			_portrait.y = y + 8;
			
			_text.x = _portrait.x + _portrait.width + 5;
			_text.y = y + 8;
			
			
			if(_blah != null)
			{
				autoFlipBubble();
				adjustBubble();
			}
			
		}
		
		override public function render():void 
		{
			super.render();
			_text.render();
			_portrait.render();
		}
		
		public function setSpeech(Text:String, Duration:Number = 0):void 
		{
			_blah = Text;
			_fadeTimer = Duration;
		}
			
		
		/**
		 *	Auto-adjusting the size of the bubble to fit the text
		 **/ 
		public function adjustBubble():void
		{
			_text.text = _character._name+":\n"+_blah;
			
			// Reset the text and the bubble every time
			scale = new FlxPoint(1, 1);
			if(_side == 0)
			{
				x = _character.x + _character.width;
				_portrait.x = x + 5;
				_text.x = _portrait.x + _portrait.width + 5;
			}
			else
			{
				x = _character.x - width;
				_portrait.x = x + width - _portrait.width - 5;
				_text.x = x + 8;
			}
			y = _character.y + _character.height/3 - height;
			_text.y = y + 8;
			
			var bubbleTextHeight:int = height - 10;
			
			if(_text.height + 16 > bubbleTextHeight) {
				// Rescale the height of the speech text
				var heightScale:Number = (_text.height + 16) / bubbleTextHeight;
				scale = new FlxPoint(1, heightScale);
				
				// Scaling doesn't change the height of _bubbleSprite
				var newHeight:Number = heightScale * height;

				// Adjust the y location of the scaled bubble
				var offset_y:int = (newHeight - height) / 2;
				y -= offset_y;
				
				// Adjust the y location of the text
				_text.y -= 2 * offset_y;
				_portrait.y = _text.y;
			}
		
		}
		
		/**
		 * 	Flips left and right
		 **/
		public function autoFlipBubble():void
		{
			var gameWidth:int = FlxG.width;
			
			// character is on the right half
			// of the screen
			// bubble should be on the left
			if(_character.x + _character.width/2 > gameWidth/2)
			{
				_side = 1;
				x = _character.x - width;
				//play("left");
				facing = LEFT;
				_portrait.facing = RIGHT;
			} 
			else
			{
				_side = 0;
				x = _character.x + _character.width;
				//play("right");
				facing = RIGHT;
				_portrait.facing = LEFT;
			}
		}
		
		public function Show():void
		{
			visible = true;
			_text.visible = true;
			_portrait.visible = true;
		}
		
		public function Hide():void
		{
			visible = false;
			_text.visible = false;
			_portrait.visible = false;
		}
		
	}
}