package {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class ExplosionSet extends FlxGroup {
		private var _explosionAreaWidth:Number;
		private var _explosionAreaHeight:Number;

		/*
		 * Duration of all explosions
		 */
		private var _explodeTimer:Number;
		/*
		 * Timer before next explosion
		 */
		private var _explodeSubTimer:Number;
		/*
		 * Time between explosions
		 */
		private var _explodeSubTime:Number;

		private var _explosionPool:ObjectPool;

		public function ExplosionSet(size:int = 0) {
			super();
			x = 0;
			y = 0;
			
			if(size == 0) {
				_explosionPool = new ObjectPool(Explosion, 15);
			} else {
				_explosionPool = new ObjectPool(Explosion, 0);
				var e:Explosion;
				for (var i : int = 0; i < 15; i++) {
					e = new Explosion(size);
					_explosionPool.add(e);
				}
			}
			add(_explosionPool);
			
			_explosionAreaWidth = 0;
			_explosionAreaHeight = 0;
			_explodeTimer = 0;
			_explodeSubTimer = 0;
			_explodeSubTime = 0.1;
		}

		override public function update():void {
			if(_explodeTimer > 0) {
				_explodeTimer -= FlxG.elapsed;
				
				if(_explodeSubTimer > 0) {
					_explodeSubTimer -= FlxG.elapsed;
				} else {
					var e:Explosion;
					e = Explosion(_explosionPool.getNextAvail());
					
					e.explode(Math.random() * _explosionAreaWidth + x, Math.random() * _explosionAreaHeight + y);
					
					_explodeSubTimer = Math.random() * _explodeSubTime;
				}
			}
			
			super.update();
		}

		public function explode(X:Number, Y:Number, Width:Number, Height:Number, Duration:Number):void {
			x = X;
			y = Y;
			_explosionAreaWidth = Width;
			_explosionAreaHeight = Height;
			
			_explodeTimer = Duration;
			_explodeSubTimer = 0;
		}

		public function explodeSingle(X:Number, Y:Number):void {
			var e:Explosion;
			e = Explosion(_explosionPool.getNextAvail());
			e.explode(X, Y);
		}

		public function get exploding():Boolean {
			return (_explodeTimer > 0);
		}
		
		public function set explodeSubTime(value:Number):void {
			_explodeSubTime = value;
		}
	}
}
