package {
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxState;

	/**
	 * @author Yori Kvitchko
	 */
	public class FadeState extends FlxState {
		/*
		 * Timer
		 */
		private var _fadeTimer:Number;
		/*
		 * Duration
		 */
		private var _fadeDuration:Number;
		/*
		 * In or Out
		 */
		private var _fadeIn:Boolean;
		/*
		 * Action to perform after fade finished
		 */
		private var _fadeAction:Function;
		/*
		 * Whether or not everything else is paused while fading
		 */
		private var _fadePause:Boolean;
		/*
		 * Helper sprite representing the fade mask
		 */
		private var _fadeMask:FlxSprite;

		override public function create():void {
			// Initialize the fade screen and start with a 0.5 second fade in
			_fadeMask = new FlxSprite(0, 0);
			_fadeMask.scrollFactor.x = _fadeMask.scrollFactor.y = 0;
			_fadeMask.createGraphic(FlxG.width, FlxG.height, 0xff000000);
			fadeIn(0.5, false);
		}

		override public function update():void {
			// Fade Logic
			if(_fadeDuration > 0) {
				if(_fadeTimer < 0) {
					// Done fading
					_fadeDuration = 0;
					if(_fadeAction != null)
						_fadeAction();
				} else {
					// Fade
					_fadeTimer -= FlxG.elapsed;
					if(_fadeIn) {
						_fadeMask.alpha = _fadeTimer / _fadeDuration;
					} else {
						_fadeMask.alpha = 1 - _fadeTimer / _fadeDuration;
					}
				
					// Don't update anything while fading if fadePause is on
					if(_fadePause)
						return;
				}
			}
			
			super.update();
		}

		/*
		 * Fade in over a period of (duration) seconds and then optionally
		 * execute the (action) function
		 */
		public function fadeIn(duration:Number, pause:Boolean, action:Function = null):void {
			_fadeTimer = _fadeDuration = duration;
			_fadeIn = true;
			_fadePause = pause;
			_fadeAction = action;
		}

		/*
		 * Fade out over a period of (duration) seconds and then optionally
		 * execute the (action) function
		 */
		public function fadeOut(duration:Number, pause:Boolean, action:Function = null):void {
			_fadeTimer = _fadeDuration = duration;
			_fadeIn = false;
			_fadePause = pause;
			_fadeAction = action;
		}

		/*
		 * Apply the actual fading during post processing
		 */
		override public function postProcess():void {
			if(_fadeDuration > 0) {
				_fadeMask.render();
			}
		}
		
		public function get fading() : Boolean {
			return (_fadeTimer > 0);
		}
	}
}
