package {
	import org.flixel.FlxButton;
	import org.flixel.FlxEmitter;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	/**
	 * @author Yori Kvitchko
	 */
	public class InternalTwoShotView extends FlxGroup {
		// CONSTANTS
		public static const CAPTAIN:uint = 0;
		public static const VIKI:uint = 1;
		public static const FRED:uint = 2;
		public static const DROID:uint = 3;
		public static const TV:uint = 4;
		public static const BRAD:uint = 5;
		public static const HEATHER:uint = 6;

		
		public static var CHAR_LOOKUP:Object;

		/*
		 * Off Screen
		 */
		public static const SUB_LOC_OFF:uint = 0;
		/*
		 * Left sub station
		 */
		public static const SUB_LOC_FAR_LEFT:uint = 1;
		/*
		 * Left sub station
		 */
		public static const SUB_LOC_LEFT:uint = 2;
		/*
		 * Center sub station
		 */
		public static const SUB_LOC_CENTER:uint = 3;
		/*
		 * Right sub station
		 */
		public static const SUB_LOC_RIGHT:uint = 4;
		/*
		 * Right sub station
		 */
		public static const SUB_LOC_FAR_RIGHT:uint = 5;

		/*
		 * Height of each individual letterbox bar
		 */
		public static const LETTERBOX_HEIGHT:uint = 150;
		/*
		 * Character position where the bottom of their portrait aligns with the top of the letterbox
		 */
		private static const CHAR_Y_POS:Number = 640 - LETTERBOX_HEIGHT - 256;
		private static const CHAR_X_FAR_LEFT_POS:Number = -21;
		private static const CHAR_X_LEFT_POS:Number = 192; // prev: -21
		private static const CHAR_X_CENTER_POS:Number = 400;
		private static const CHAR_X_RIGHT_POS:Number = 620; // prev: 834
		private static const CHAR_X_FAR_RIGHT_POS:Number = 834;

		private static const CHAR_SUBLOC_LOOKUP:Array = [0, CHAR_X_FAR_LEFT_POS, CHAR_X_LEFT_POS, CHAR_X_CENTER_POS, CHAR_X_RIGHT_POS, CHAR_X_FAR_RIGHT_POS];

		private static const TEXT_X_FAR_LEFT_POS:Number = 185;
		private static const TEXT_X_LEFT_POS:Number = 410; // prev: 185
		private static const TEXT_X_CENTER_POS:Number = 570;
		private static const TEXT_X_RIGHT_POS:Number = 410; // prev: 613
		private static const TEXT_X_FAR_RIGHT_POS:Number = 613;

		private static const TEXT_SUBLOC_LOOKUP:Array = [0, TEXT_X_FAR_LEFT_POS, TEXT_X_LEFT_POS, TEXT_X_CENTER_POS, TEXT_X_RIGHT_POS, TEXT_X_FAR_RIGHT_POS];

		/*
		 * Scroll value for far left view
		 */
		private static const CAMERA_X_FAR_LEFT:Number = 0;
		/*
		 * Scroll value for left view
		 */
		private static const CAMERA_X_LEFT:Number = -213; // prev: 0
		/*
		 * Scroll value for centered view
		 */
		private static const CAMERA_X_CENTER:Number = -213;
		/*
		 * Scroll value for right view
		 */
		private static const CAMERA_X_RIGHT:Number = -213; // prev: -427
		/*
		 * Scroll value for far right view
		 */
		private static const CAMERA_X_FAR_RIGHT:Number = -427;

		private static const CAMERA_SUBLOC_LOOKUP:Array = [0, CAMERA_X_FAR_LEFT, CAMERA_X_LEFT, CAMERA_X_CENTER, CAMERA_X_RIGHT, CAMERA_X_FAR_RIGHT];

		// VARIABLES
		
		/*
		 * All characters, indexed according to constants at top
		 */
		public var characters:Array;
		/*
		 * Matching array for character miniatures
		 */
		private var _miniCharacters:Array;

		private var _conversation:TwoShotConversation;

		/*
		 * Current location being viewed ( 0 = intro background )
		 */
		private var _curLocation:uint;
		/*
		 * Camera focusing on left or right sublocation
		 */
		private var _cameraSubLocation:uint;
		/*
		 * Whether or not the camera is currently panning
		 */
		private var _panning:Boolean;
		/*
		 * Panning direction and ammount
		 */
		private var _panDelta:Number;
		/*
		 * Panning target
		 */
		private var _panningSubLocation:uint;
		/*
		 * Was a character transfer the reason for the location change
		 */
		private var _justTransfered:Boolean;

		/*
		 * Used for fade location change
		 */
		private var _newLocation:uint;
		/*
		 * Used for fade location change
		 */
		private var _newSubLocation:uint;

		private var _backgrounds:Array;

		/*
		 * HUD
		 */
		private var _hud:FlxGroup;
		private var _hudHighlight:FlxSprite;
		private var _hudSelector:FlxSprite;
		private var _navigationBtn:FlxButton;
		private var _weaponsBtn:FlxButton;
		private var _shieldsBtn:FlxButton;
		private var _deployBtn:FlxButton;
		
		private var _navigationStatus1:FlxText;
		private var _weaponsStatus1:FlxText;
		private var _shieldsStatus1:FlxText;
		private var _navigationStatus2:FlxText;
		private var _weaponsStatus2:FlxText;
		private var _shieldsStatus2:FlxText;
		
		private var _navPower:FlxSprite;
		private var _wepPower:FlxSprite;
		private var _shieldPower:FlxSprite;

		private var _titleText:FlxText;
		private var _titleTextBlackout:FlxSprite;
		private var _titleTextInterval:Interval;
		
		private var _levelDeny:Boolean;
		private var _internalTutorialOneComplete:Boolean;
		private var _internalTutorialTwoComplete:Boolean;

		// Cinematics info
		/*
		 * Currently in cinematic mode?
		 */
		public var _cinematicMode:Boolean;
		/*
		 * Used to determine if we fade or immediately change to the first room
		 */
		private var _firstCinematic:Boolean;
		/*
		 * Array of cinematics to process
		 */
		private var _cinematics:Array;
		/*
		 * Current cinematic
		 */
		private var _currentCinematic:Cinematic;
		/*
		 * Current action
		 */
		private var _currentAction:Action;
		/*
		 * Whether or not text has been put in a bubble for a talk action
		 */
		private var _actionTextSet:Boolean;
		/*
		 * Timer used for action durations
		 */
		private var _actionTimer:Number;
		/*
		 * Determines if the user gains control over characters after the cinematic is over
		 */
		public var _controlAfterCinematic:Boolean;

		// User control info
		private var _selectedChar:int;

		// Skip cinematic button
		private var _skipText:FlxText;
		private var _justSkipped:Boolean;

		/*
		 * Sprite representing the first letterbox
		 */
		private var _letterbox_1:FlxSprite;
		/*
		 * Sprite representing the second letterbox
		 */
		private var _letterbox_2:FlxSprite;

		// EMBEDDED RESOURCES
		
		// Characters
		[Embed(source="../data/characters/fred.png")] 
		private var ImgCharacterFred:Class;

		[Embed(source="../data/characters/viki.png")] 
		private var ImgCharacterViki:Class;

		[Embed(source="../data/characters/brad.png")] 
		private var ImgCharacterBrad:Class;

		[Embed(source="../data/characters/droid.png")] 
		private var ImgCharacterDroid:Class;

		[Embed(source="../data/characters/captain.png")] 
		private var ImgCharacterCaptain:Class;

		[Embed(source="../data/characters/heather.png")] 
		private var ImgCharacterHeather:Class;

		// Character Components
		[Embed(source="../data/characters/components/captain_badge.png")] 
		private var ImgCharacterCaptainBadge:Class;
		
		[Embed(source="../data/characters/components/viki_smoke.png")] 
		private var ImgCharacterVikiSmoke:Class;
		
		[Embed(source="../data/characters/components/fred_hat.png")] 
		private var ImgCharacterFredHat:Class;
		
		[Embed(source="../data/characters/components/bro_heart.png")] 
		private var ImgCharacterDroidHeart:Class;

		// Backgrounds
		[Embed(source="../data/bg_nav_blur.png")] 
		private var ImgBGNav:Class;

		[Embed(source="../data/bg_shield_blur.png")] 
		private var ImgBGShield:Class;

		[Embed(source="../data/bg_wep_blur.png")] 
		private var ImgBGWep:Class;

		// HUD Elements
		[Embed(source="../data/hud/hud_highlight.png")] 
		private var ImgHUDHighlight:Class;
		
		[Embed(source="../data/hud/hud_nav.png")] 
		private var ImgHUDNav:Class;

		[Embed(source="../data/hud/hud_shield.png")] 
		private var ImgHUDShield:Class;

		[Embed(source="../data/hud/hud_wep.png")] 
		private var ImgHUDWep:Class;

		[Embed(source="../data/hud/hud_nav_selected.png")] 
		private var ImgHUDNavSelected:Class;

		[Embed(source="../data/hud/hud_shield_selected.png")] 
		private var ImgHUDShieldSelected:Class;

		[Embed(source="../data/hud/hud_wep_selected.png")] 
		private var ImgHUDWepSelected:Class;

		[Embed(source="../data/hud/hud_wep_power.png")] 
		private var ImgHUDWepPower:Class;
		
		[Embed(source="../data/hud/hud_nav_power.png")] 
		private var ImgHUDNavPower:Class;
		
		[Embed(source="../data/hud/hud_shield_power.png")] 
		private var ImgHUDShieldPower:Class;

		[Embed(source="../data/cursor_internal.png")] 
		private var ImgCursor:Class;

		// Music
		[Embed(source='../data/sounds/ambient.mp3')]
		private const SndAmbient:Class;
		
		private var _stars:Array;
		private var _starEmitter:FlxEmitter;

		public function InternalTwoShotView() {
			super();
			
			// Initialize Character Status (scores)
			// 0 = alive
			// 1 = suicide
			// 2 = killed
			FlxG.levels[CAPTAIN] = 0;
			FlxG.levels[VIKI] = 0;
			FlxG.levels[FRED] = 0;
			FlxG.levels[DROID] = 0;
			
			// Init HUD
			_hud = new FlxGroup();
			
			// Background star field seen in window of nav room
			_stars = new Array(50);
			for (var i:int = 0;i < 50;i++) {
				_stars[i] = new StarSprite(320, 320, 1);
			}
			_starEmitter = new FlxEmitter(320, 320);
			_starEmitter.members = _stars;
			_starEmitter.minRotation = 0;
			_starEmitter.maxRotation = 0;
			_starEmitter.start(false, 0.1);
			add(_starEmitter);
			
			// Backgrounds
			var bg:FlxSprite;
			_backgrounds = new Array(4); // Correspond to locations with 0 as intro
			
			// Intro
			bg = new FlxSprite(0, 0);
			bg.createGraphic(1067, 640, 0xff1b325f);
			bg.visible = false;
			_backgrounds[0] = add(bg);
			
			// Nav
			bg = new FlxSprite(0, 0, ImgBGNav);
			bg.scrollFactor.x = 0.656;
			bg.visible = false;
			_backgrounds[Script.NAV] = add(bg);
			
			// Weapons
			bg = new FlxSprite(0, 0, ImgBGWep);
			bg.scrollFactor.x = 0.656;
			bg.visible = false;
			_backgrounds[Script.WEP] = add(bg);
			
			// Shields
			bg = new FlxSprite(0, 0, ImgBGShield);
			bg.scrollFactor.x = 0.656;
			bg.visible = false;
			_backgrounds[Script.SHIELD] = add(bg);
			
			// Characters (initialize positions)
			var c:TwoShotCharacter;
			var comp:TwoShotCharacterComponent;
			characters = new Array(5);
			_miniCharacters = new Array(5);
			var mini:FlxSprite;

			CHAR_LOOKUP = new Object();
			CHAR_LOOKUP["C"] = CAPTAIN;
			CHAR_LOOKUP["V"] = VIKI;
			CHAR_LOOKUP["F"] = FRED;
			CHAR_LOOKUP["H"] = HEATHER;
			CHAR_LOOKUP["B"] = BRAD;
			CHAR_LOOKUP["D"] = DROID;
			CHAR_LOOKUP["T"] = TV;

			// Captain
			c = new TwoShotCharacter("Captain", ImgCharacterCaptain, this);
			c.location = Script.NAV;
			c.sub_location = SUB_LOC_CENTER;
			characters[CAPTAIN] = add(c);
			// Captain's Badge
			comp = new TwoShotCharacterComponent(c);
			comp.loadGraphic(ImgCharacterCaptainBadge, true, false, 196, 256);
			comp.addAnimation("default", [0, 1, 2, 3], 4);
			comp.play("default");
			c.addComponent(comp);
			
			// Viki
			c = new TwoShotCharacter("Viki", ImgCharacterViki, this);
			c.location = Script.NAV;
			c.sub_location = SUB_LOC_LEFT;
			characters[VIKI] = add(c);
			// Viki's Smoke
			comp = new TwoShotCharacterComponent(c);
			comp.loadGraphic(ImgCharacterVikiSmoke, true, false, 256, 256);
			comp.addAnimation("default", [0, 1, 2, 3], 4);
			comp.play("default");
			c.addComponent(comp);
			
			// Fred
			c = new TwoShotCharacter("Fred", ImgCharacterFred, this);
			c.location = Script.NAV;
			c.sub_location = SUB_LOC_RIGHT;
			characters[FRED] = add(c);
			// Fred's Hat
			comp = new TwoShotCharacterComponent(c);
			comp.loadGraphic(ImgCharacterFredHat, true, false, 256, 249);
			comp.addAnimation("default", [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24], 4);
			comp.play("default");
			c.addComponent(comp);
			
			//			_char = new TwoShotCharacter("Heather", ImgCharacterHeather, this);
			//			_char.location = Script.NAV;
			//			_char.sub_location = SUB_LOC_LEFT;
			//			characters[HEATHER] = add(_char);
			//			
			//			_char = new TwoShotCharacter("Brad", ImgCharacterBrad, this);
			//			_char.location = Script.WEP;
			//			_char.sub_location = SUB_LOC_CENTER;
			//			characters[BRAD] = add(_char);

			// Droid
			c = new TwoShotCharacter("Droid", ImgCharacterDroid, this);
			c.location = Script.WEP;
			c.sub_location = SUB_LOC_LEFT;
			characters[DROID] = add(c);
			// Droid's Heart
			comp = new TwoShotCharacterComponent(c);
			comp.loadGraphic(ImgCharacterDroidHeart, true, false, 256, 256);
			comp.addAnimation("default", [1, 1, 1, 1, 1, 0, 1, 0], 4);
			comp.play("default");
			c.addComponent(comp);
			
			// TODO: Change to TV Graphic
			c = new TwoShotCharacter("TV", ImgCharacterDroid, this);
			c.location = Script.DEAD;
			c.sub_location = SUB_LOC_OFF;
			characters[TV] = add(c);
			
			// Chat Bubbles
			_conversation = new TwoShotConversation(TEXT_X_FAR_LEFT_POS, 375);
			_conversation.visible = false;
			add(_conversation);
			_actionTextSet = false;
			_actionTimer = 0;
			
			// Letterbox
			_letterbox_1 = new FlxSprite(0, 0);
			_letterbox_1.createGraphic(1068, LETTERBOX_HEIGHT, 0xff000000);
			_letterbox_2 = new FlxSprite(0, FlxG.height - LETTERBOX_HEIGHT);
			_letterbox_2.createGraphic(1068, LETTERBOX_HEIGHT, 0xff000000);
			add(_letterbox_1);
			add(_letterbox_2);
			
			// Blackout Text
			_titleTextBlackout = new FlxSprite(0, 0);
			_titleTextBlackout.createGraphic(640, 640, 0xff000000);
			_titleTextBlackout.scrollFactor.x = _titleTextBlackout.scrollFactor.y = 0;
			_titleTextBlackout.exists = false;
			add(_titleTextBlackout);
			
			// Title Text
			_titleText = new FlxText(0, 300, FlxG.width, "");
			_titleText.scrollFactor.x = _titleText.scrollFactor.y = 0;
			_titleText.alignment = "center";
			_titleText.size = 36;
			add(_titleText);
			
			// Fade out, text appears, fade in
			_titleTextInterval = new Interval([0.1, 3, 0.5]);

			
			// HUD
			_hud.exists = false;
			add(_hud);
			
			_hudHighlight = new FlxSprite(48, 20, ImgHUDHighlight);
			_hudHighlight.scrollFactor.x = _hudHighlight.scrollFactor.y = 0;
			_hudHighlight.exists = false;
			_hud.add(_hudHighlight);
			
			_hudSelector = new FlxSprite(-100, -100);
			_hudSelector.createGraphic(120, 48, 0xff606042);
			_hudSelector.scrollFactor.x = _hudSelector.scrollFactor.y = 0;
			_hud.add(_hudSelector);
			
			// User control
			_selectedChar = 0;
			
			//			// Buttons
			//			var t:FlxText;
			//
			//			// Weapons Room Button
			//			_weaponsBtn = new FlxButton(80, 50, onWep);
			//			_weaponsBtn.loadGraphic((new FlxSprite()).createGraphic(120, 50, 0xff5c3030), (new FlxSprite()).createGraphic(120, 50, 0xff8e4b4b));
			//			_weaponsBtn.scrollFactor.x = _weaponsBtn.scrollFactor.y = 0;
			//			t = new FlxText(0, 10, 120, "Weapons");
			//			t.color = 0xffffff;
			//			t.size = 20;
			//			t.alignment = "center";
			//			_weaponsBtn.loadText(t);
			//			_hud.add(_weaponsBtn);
			//			
			//			// Navigation Room Button
			//			_navigationBtn = new FlxButton(240, 50, onNav);
			//			_navigationBtn.loadGraphic((new FlxSprite()).createGraphic(160, 50, 0xff322967), (new FlxSprite()).createGraphic(160, 50, 0xff6c58da));
			//			_navigationBtn.scrollFactor.x = _navigationBtn.scrollFactor.y = 0;
			//			t = new FlxText(10, 10, 140, "Navigation");
			//			t.color = 0xffffff;
			//			t.size = 20;
			//			t.alignment = "center";
			//			_navigationBtn.loadText(t);
			//			_hud.add(_navigationBtn);
			//			
			//			// Shields Room Button
			//			_shieldsBtn = new FlxButton(440, 50, onShield);
			//			_shieldsBtn.loadGraphic((new FlxSprite()).createGraphic(120, 50, 0xff397890), (new FlxSprite()).createGraphic(120, 50, 0xff57b6d9));
			//			_shieldsBtn.scrollFactor.x = _shieldsBtn.scrollFactor.y = 0;
			//			t = new FlxText(10, 10, 100, "Shields");
			//			t.color = 0xffffff;
			//			t.size = 20;
			//			t.alignment = "center";
			//			_shieldsBtn.loadText(t);
			//			_hud.add(_shieldsBtn);

			var t:FlxText;
			
			// Weapons Room Status
			// image
			//			_weapons = new FlxSprite(80, 60);
			//			_weapons.loadGraphic(ImgHUDWep);
			//			_weapons.addAnimation("default", [0]);
			//			_weapons.addAnimation("selected", [1]);
			//			_weapons.play("default");
			//			_weapons.scrollFactor.x = _weapons.scrollFactor.y = 0;
			//			_hud.add(_weapons);
			// button
			_weaponsBtn = new FlxButton(64, 60, onWep);
			_weaponsBtn.loadGraphic((new FlxSprite()).loadGraphic(ImgHUDWep), (new FlxSprite()).loadGraphic(ImgHUDWepSelected));
			_weaponsBtn.scrollFactor.x = _weaponsBtn.scrollFactor.y = 0;
			_hud.add(_weaponsBtn);
			// text
			t = new FlxText(_weaponsBtn.x, _weaponsBtn.y - 30, _weaponsBtn.width + 32, "Weapons");
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.color = 0xffffff;
			t.size = 20;
			t.alignment = "center";
			_hud.add(t);
			
			// Navigation Room Status
			// image
			//			_navigation = new FlxSprite(252, 60);
			//			_navigation.loadGraphic(ImgHUDNav);
			//			_navigation.addAnimation("default", [0]);
			//			_navigation.addAnimation("selected", [1]);
			//			_navigation.play("default");
			//			_navigation.scrollFactor.x = _navigation.scrollFactor.y = 0;
			//			_hud.add(_navigation);
			// button
			_navigationBtn = new FlxButton(244, 60, onNav);
			_navigationBtn.loadGraphic((new FlxSprite()).loadGraphic(ImgHUDNav), (new FlxSprite()).loadGraphic(ImgHUDNavSelected));
			_navigationBtn.scrollFactor.x = _navigationBtn.scrollFactor.y = 0;
			_hud.add(_navigationBtn);
			// text
			t = new FlxText(_navigationBtn.x, _navigationBtn.y - 30, _navigationBtn.width + 32, "Navigation");
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.color = 0xffffff;
			t.size = 20;
			t.alignment = "center";
			_hud.add(t);
			
			// Shields Room Status
			// image
			//			_shields = new FlxSprite(440, 60);
			//			_shields.loadGraphic(ImgHUDShield);
			//			_shields.addAnimation("default", [0]);
			//			_shields.addAnimation("selected", [1]);
			//			_shields.play("default");
			//			_shields.scrollFactor.x = _shields.scrollFactor.y = 0;
			//			_hud.add(_shields);
			// button
			_shieldsBtn = new FlxButton(424, 60, onShield);
			_shieldsBtn.loadGraphic((new FlxSprite()).loadGraphic(ImgHUDShield), (new FlxSprite()).loadGraphic(ImgHUDShieldSelected));
			_shieldsBtn.scrollFactor.x = _shieldsBtn.scrollFactor.y = 0;
			_hud.add(_shieldsBtn);
			// text
			t = new FlxText(_shieldsBtn.x, _shieldsBtn.y - 30, _shieldsBtn.width + 32, "Shields");
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.color = 0xffffff;
			t.size = 20;
			t.alignment = "center";
			_hud.add(t);
			
			// Upgrade Status
			
			// Wep
			// 1
			_weaponsStatus1 = new FlxText(_weaponsBtn.x + 0, _weaponsBtn.y + _weaponsBtn.height, _weaponsBtn.width + 32, "");
			_weaponsStatus1.scrollFactor.x = _weaponsStatus1.scrollFactor.y = 0;
			_weaponsStatus1.color = 0xffe4e4;//0x8e4b4b;
			_weaponsStatus1.size = 12;
			_weaponsStatus1.alignment = "center";
			_weaponsStatus1.text = "AUTO-AIM 5000";
			// 2
			_weaponsStatus2 = new FlxText(_weaponsBtn.x + 1, _weaponsBtn.y + _weaponsBtn.height + 1, _weaponsBtn.width + 32, "");
			_weaponsStatus2.scrollFactor.x = _weaponsStatus2.scrollFactor.y = 0;
			_weaponsStatus2.color = 0xffffff;
			_weaponsStatus2.size = 12;
			_weaponsStatus2.alignment = "center";
			_weaponsStatus2.text = "AUTO-AIM 5000";
			// add
//			_hud.add(_weaponsStatus2);
			_hud.add(_weaponsStatus1);
			
			// Nav
			// 1			
			_navigationStatus1 = new FlxText(_navigationBtn.x + 0, _navigationBtn.y + _navigationBtn.height, _navigationBtn.width + 32, "");
			_navigationStatus1.scrollFactor.x = _navigationStatus1.scrollFactor.y = 0;
			_navigationStatus1.color = 0xded8ff;//0x6c58da;
			_navigationStatus1.size = 12;
			_navigationStatus1.alignment = "center";
			_navigationStatus1.text = "TELEPORT-O-MATIC";
			// 2
			_navigationStatus2 = new FlxText(_navigationBtn.x + 1, _navigationBtn.y + _navigationBtn.height + 1, _navigationBtn.width + 32, "");
			_navigationStatus2.scrollFactor.x = _navigationStatus2.scrollFactor.y = 0;
			_navigationStatus2.color = 0xffffff;
			_navigationStatus2.size = 12;
			_navigationStatus2.alignment = "center";
			_navigationStatus2.text = "TELEPORT-O-MATIC";
			// add
//			_hud.add(_navigationStatus2);
			_hud.add(_navigationStatus1);
			
			// Shield
			// 1
			_shieldsStatus1 = new FlxText(_shieldsBtn.x + 0, _shieldsBtn.y + _shieldsBtn.height, _shieldsBtn.width + 32, "");
			_shieldsStatus1.scrollFactor.x = _shieldsStatus1.scrollFactor.y = 0;
			_shieldsStatus1.color = 0xdcf6ff;//0x57b6d9;
			_shieldsStatus1.size = 12;
			_shieldsStatus1.alignment = "center";
			_shieldsStatus1.text = "ZAPPO REFLECTO";
			// 2
			_shieldsStatus2 = new FlxText(_shieldsBtn.x + 1, _shieldsBtn.y + _shieldsBtn.height + 1, _shieldsBtn.width + 32, "");
			_shieldsStatus2.scrollFactor.x = _shieldsStatus2.scrollFactor.y = 0;
			_shieldsStatus2.color = 0xffffff;
			_shieldsStatus2.size = 12;
			_shieldsStatus2.alignment = "center";
			_shieldsStatus2.text = "ZAPPO REFLECTO";
			// add
//			_hud.add(_shieldsStatus2);
			_hud.add(_shieldsStatus1);
			
			// Power Meter
			//wep
			_wepPower = new FlxSprite(_weaponsBtn.x + _weaponsBtn.width, _weaponsBtn.y);
			_wepPower.scrollFactor.x = _wepPower.scrollFactor.y = 0;
			_wepPower.loadGraphic(ImgHUDWepPower, true, false, 32, 48);
			_wepPower.addAnimation("0", [0]);
			_wepPower.addAnimation("1", [1]);
			_wepPower.addAnimation("2", [2]);
			_wepPower.play("0");
			_hud.add(_wepPower);
			//nav
			_navPower = new FlxSprite(_navigationBtn.x + _navigationBtn.width, _navigationBtn.y);
			_navPower.scrollFactor.x = _navPower.scrollFactor.y = 0;
			_navPower.loadGraphic(ImgHUDNavPower, true, false, 32, 48);
			_navPower.addAnimation("0", [0]);
			_navPower.addAnimation("1", [1]);
			_navPower.addAnimation("2", [2]);
			_navPower.play("0");
			_hud.add(_navPower);
			//shield
			_shieldPower = new FlxSprite(_shieldsBtn.x + _shieldsBtn.width, _shieldsBtn.y);
			_shieldPower.scrollFactor.x = _shieldPower.scrollFactor.y = 0;
			_shieldPower.loadGraphic(ImgHUDShieldPower, true, false, 32, 48);
			_shieldPower.addAnimation("0", [0]);
			_shieldPower.addAnimation("1", [1]);
			_shieldPower.addAnimation("2", [2]);
			_shieldPower.play("0");
			_hud.add(_shieldPower);
			
			// Deploy Button
			if(FlxG.state is PlayState) {
				_deployBtn = new FlxButton(220, 515, onDeploy);
				_deployBtn.loadGraphic((new FlxSprite()).createGraphic(200, 50, 0xff172e7c), (new FlxSprite()).createGraphic(200, 50, 0xff2141aa));
				_deployBtn.scrollFactor.x = _deployBtn.scrollFactor.y = 0;
				t = new FlxText(0, 10, _deployBtn.width, "Warp In!");
				t.scrollFactor.x = t.scrollFactor.y = 0;
				t.color = 0xffffff;
				t.size = 20;
				t.alignment = "center";
				_deployBtn.loadText(t);
				_hud.add(_deployBtn);
			}
			
			// Mini Characters
			mini = new FlxSprite(-100, -100);
			mini.createGraphic(8, 16, 0xff2f2fb6);
			mini.scrollFactor.x = mini.scrollFactor.y = 0;
			_miniCharacters[CAPTAIN] = _hud.add(mini);
						
			mini = new FlxSprite(-100, -100);
			mini.createGraphic(8, 16, 0xff9468d1);
			mini.scrollFactor.x = mini.scrollFactor.y = 0;
			_miniCharacters[VIKI] = _hud.add(mini);
						
			mini = new FlxSprite(-100, -100);
			mini.createGraphic(8, 16, 0xff0076c1);
			mini.scrollFactor.x = mini.scrollFactor.y = 0;
			_miniCharacters[FRED] = _hud.add(mini);
			
			mini = new FlxSprite(-100, -100);
			mini.createGraphic(8, 16, 0xfff9ee00);
			mini.scrollFactor.x = mini.scrollFactor.y = 0;
			_miniCharacters[DROID] = _hud.add(mini);
			
			mini = new FlxSprite(-100, -100);
			mini.createGraphic(8, 16, 0xff000000);
			mini.scrollFactor.x = mini.scrollFactor.y = 0;
			_miniCharacters[TV] = _hud.add(mini);
			
			updateHUD();
			
			// Skip Text
			_skipText = new FlxText(FlxG.height / 2 - 130, FlxG.height - 30, 260, "( [Space] Skip Cinematic )");
			_skipText.alignment = "center";
			_skipText.size = 16;
			_skipText.scrollFactor.x = 0;
			_skipText.scrollFactor.y = 0;
			add(_skipText);
			_justSkipped = false;
			_justTransfered = false;
			
			// Tutorial buttons
			var bp:BlueprintButtons;
			bp = new BlueprintButtons();
			bp.scrollFactor.x = bp.scrollFactor.y = 0;
			bp.reset(20, 590);
			_hud.add(bp);
			
			// Initialize View (default to intro background)
			changeLocation(0, SUB_LOC_LEFT);
			_panning = false;
			_panDelta = 10;
			_levelDeny = false;
			_internalTutorialOneComplete = false;
			_internalTutorialTwoComplete = false;
		}

		// ************************ CINEMATIC PROCESSING ***************************
		
		/*
		 * Load up cinematics
		 */
		public function queueupCinematics(cinematics:Array):void {
			_cinematics = cinematics;
		}

		/*
		 * Play cinematics
		 */
		public function playCinematics(control:Boolean = false):void {
			_cinematicMode = true;
			_firstCinematic = true;
			_controlAfterCinematic = control;
			FlxG.mouse.hide();
		}

		/*
		 * Main cinematic logic function
		 */
		private function processCinematics():void {
//			var c:TwoShotCharacter;
			_skipText.text = "( Click to Advance Text )";
			_titleTextBlackout.exists = true;
			
			if(_currentCinematic) {
				// Process Cinematic
				processCinematic();
			} else if(_cinematics.length > 0) {
				// Get next Cinematic
				_currentCinematic = _cinematics.shift();
				// Title Action
				_currentAction = new Action("", "TITLE");
				_skipText.visible = false;
				// Determine who the focus is
				var ch:String;
				// First to have an action
				ch = Action(_currentCinematic.actions[0]).character;
				_currentCinematic.focus = CHAR_LOOKUP[ch];
//				// If the first was not a helper check the rest to see if a helper is among them
//				if(ch != "V" && ch != "H" && ch != "D") {
//					for (var i:int = 1;i < _currentCinematic.actions.length;i++) {
//						ch = Action(_currentCinematic.actions[i]).character;
//						if(ch == "V" || ch == "H" || ch == "D") {
//							_currentCinematic.focus = CHAR_LOOKUP[ch];
//							break;
//						}
//					}
//				}

				if(TwoShotCharacter(characters[CHAR_LOOKUP[ch]]).location == Script.DEAD) {
					// If the character is dead, don't continue
//					_currentCinematic = null;
//					_currentAction = null;
				} else {
					// Move camera to focus
//					c = TwoShotCharacter(characters[_currentCinematic.focus]);
//					if(_curLocation != c.location) {
						// Always fade
//						if(_firstCinematic)
//							changeLocation(c.location, c.sub_location);
//						else
//							fadeLocation(c.location, c.sub_location);
//					} else if (_cameraSubLocation != c.sub_location) {
//						panCamera(c.sub_location);
//					}
					
					_firstCinematic = false;
				}
			} else {
				// Done processing all cinematics
				_cinematicMode = false;
				FlxG.mouse.show();
				_skipText.visible = false;
				fadeLocation(Script.NAV, SUB_LOC_CENTER);
			}
		}

		/*
		 * Process a single cinematic
		 */
		private function processCinematic():void {
			if(_currentAction) {
				// Process next action
				processAction();
			} else if(_currentCinematic.actions.length > 0) {
				// Start processing next action
				_currentAction = _currentCinematic.actions.shift();
			} else {
				// Done processing this cinematic
				_currentCinematic = null;
				_conversation.clear();
			}
		}

		/*
		 * Process a single action
		 */
		private function processAction():void {
			var c:TwoShotCharacter;
			
			//trace(_currentAction);
			if(_currentAction.type == "speak") {
				if(_currentCinematic.focus == HEATHER && CHAR_LOOKUP[_currentAction.character] == VIKI) {
					panCamera(TwoShotCharacter(characters[VIKI]).sub_location);
					_conversation.clear();
					_currentCinematic.focus = VIKI;
				} else if(_currentCinematic.focus == VIKI && CHAR_LOOKUP[_currentAction.character] == HEATHER) {
					panCamera(TwoShotCharacter(characters[HEATHER]).sub_location);
					_conversation.clear();
					_currentCinematic.focus = HEATHER;
				}
				
				if(!_actionTextSet) {
					var f:uint; // facing
					if((_conversation.x + _conversation.origin.x) > TwoShotCharacter(characters[CHAR_LOOKUP[_currentAction.character]]).x) {
						f = FlxSprite.LEFT;
					} else {
						f = FlxSprite.RIGHT;
					}
					_conversation.pushText(_currentAction.content, f);
					_actionTextSet = true;
				} else if(_actionTextSet && FlxG.mouse.justPressed() && _conversation.queueLength() == 0) {
					_actionTextSet = false;
					_currentAction = null;
				}
			} else if (_currentAction.type == "DIE") {
				// Scoring
				var cnt:Array = getPositionCount();
				FlxG.levels[CHAR_LOOKUP[_currentAction.character]] = cnt[_curLocation] + 1;
				
				_conversation.clear();
				killCharacter(CHAR_LOOKUP[_currentAction.character]);
			} else if (_currentAction.type == "FOCUS") {
				if(!PlayState(FlxG.state).fading && !_panning) {
					c = TwoShotCharacter(characters[CHAR_LOOKUP[_currentAction.character]]);
					if(c.location != _curLocation) {
						fadeLocation(c.location, c.sub_location);
					} else if(TEXT_SUBLOC_LOOKUP[c.sub_location] != _conversation.x) {
						_conversation.clear();
						panCamera(c.sub_location);
					} else {
						_currentAction = null;
					}
				}
			} else if (_currentAction.type == "TITLE") {
				if(_titleTextInterval.justLooped || (_titleTextInterval.currentInterval != 0 && FlxG.mouse.justPressed())) {
					_titleText.text = "";
					_titleTextBlackout.alpha = 0;
					_titleTextBlackout.exists = false;
					_skipText.visible = true;
					_currentAction = null;
					_titleTextInterval.reset();
				} else {
					_titleTextInterval.elapse(FlxG.elapsed);
					
					if (_titleTextInterval.justTicked) {
						if(_titleTextInterval.currentInterval == 0) {
							_titleTextBlackout.exists = true;
							if(_titleTextBlackout.alpha != 1)
								_titleTextBlackout.alpha = 0;
							_titleText.text = _currentCinematic.title;
							_titleText.alpha = 0;
						} else if(_titleTextInterval.currentInterval == 1) {
							c = TwoShotCharacter(characters[_currentCinematic.focus]);
							changeLocation(c.location, c.sub_location);
							_titleTextBlackout.alpha = 1;
							_titleText.text = _currentCinematic.title;
							_titleText.alpha = 1;
						}
					}
					if(_titleTextInterval.currentInterval == 0 && _titleTextBlackout.alpha < 1) {
						_titleTextBlackout.alpha = _titleText.alpha = _titleTextInterval.percentTicked;
					}
					if(_titleTextInterval.currentInterval == 2) {
						_titleTextBlackout.alpha = _titleText.alpha = 1 - _titleTextInterval.percentTicked;
					}
				}
			} else if (_currentAction.type == "APPROACH") {
				if(TwoShotCharacter(characters[CHAR_LOOKUP[_currentAction.character]]).sub_location == SUB_LOC_LEFT) {
					slideCharacter(CHAR_LOOKUP[_currentAction.character], 50, 1);
				} else {
					slideCharacter(CHAR_LOOKUP[_currentAction.character], -50, 1);
				}
				_currentAction = null;
			} else if (_currentAction.type == "END") {
				PlayState(FlxG.state).fadeOut(2, true, PlayState(FlxG.state).loseGame);
			} else if (_currentAction.type == "WIN") {
				PlayState(FlxG.state).fadeOut(2, true, PlayState(FlxG.state).winGame);
			} else {
				_currentAction = null;
			}
		}

		// ************************ CINEMATIC PROCESSING ***************************

		// ************************ USER CONTROL ***********************************

		private function processUserControl():void {
			var c:TwoShotCharacter;
			var i:int;
			
			// User control tutorial
			PlayState.loadTutorial(1);
			
			// Character Stress tutorial
			if(FlxG.level > 1)
				PlayState.loadTutorial(8);
			
			// Panning
			if(!_selectedChar) {
				if(FlxG.mouse.screenX < 100) {
					FlxG.scroll.x += 3;
					if(FlxG.scroll.x > 0)
					FlxG.scroll.x = 0;
				}
				if(FlxG.mouse.screenX > (FlxG.width - 100)) {
					FlxG.scroll.x -= 3;
					if(FlxG.scroll.x < -427)
					FlxG.scroll.x = -427;
				}
			}
			
			// Mouse Over
			if(_navigationBtn.overlaps(FlxG.mouse.cursor)) {
				FlxG.mouse.cursor.play("select");
			}else if(_weaponsBtn.overlaps(FlxG.mouse.cursor)) {
				FlxG.mouse.cursor.play("select");
			} else if(_shieldsBtn.overlaps(FlxG.mouse.cursor)) {
				FlxG.mouse.cursor.play("select");
			} else {
				FlxG.mouse.cursor.play("default");
				
				for (i = 1;i < characters.length;i++) {
					c = TwoShotCharacter(characters[i]);
					if(c.visible && c.overlaps(FlxG.mouse.cursor)) {
						FlxG.mouse.cursor.play("select");
					}
				}
			}
			
			// Selection
			if(FlxG.mouse.justPressed()) {
				if(_selectedChar) {
					// Already selected
					if(_navigationBtn.overlaps(FlxG.mouse.cursor)) {
						// Move to Nav
						if(transferCharacter(_selectedChar, Script.NAV)) {
//							_selectedChar = 0;
							updateHUD();
						}
					} else if(_weaponsBtn.overlaps(FlxG.mouse.cursor)) {
						// Move to Wep
						if(transferCharacter(_selectedChar, Script.WEP)) {
//							_selectedChar = 0;
							updateHUD();
						}
					} else if(_shieldsBtn.overlaps(FlxG.mouse.cursor)) {
						// Move to Shield
						if(transferCharacter(_selectedChar, Script.SHIELD)) {
//							_selectedChar = 0;
							updateHUD();
						}
					} else {
						// Clear selection
						_conversation.clear();
						_selectedChar = 0;
					}
				} else {
					_conversation.clear();
				}
				
				// Ignore the captain, select anyone else you click on
				for (i = 1;i < characters.length;i++) {
					c = TwoShotCharacter(characters[i]);
					
					if(c.visible && c.overlaps(FlxG.mouse.cursor)) {
						panCamera(c.sub_location);
						if((_conversation.x + _conversation.origin.x) > c.x) {
							_conversation.pushText("Where do you want me, Sir?", FlxSprite.LEFT);
						} else {
							_conversation.pushText("Where do you want me, Sir?", FlxSprite.RIGHT);
						}
						_selectedChar = i;
					}
				}
			}

			// Turn highlight on / off
			_hudHighlight.exists = (_selectedChar > 0);
		}

		private function onNav():void {
			if(_selectedChar == 0 && _curLocation != Script.NAV && active && !_cinematicMode) {
				fadeLocation(Script.NAV, SUB_LOC_CENTER);
			} else {
				_selectedChar = 0;
			}
			FlxG.mouse.reset();
		}

		private function onWep():void {
			if(_selectedChar == 0 && _curLocation != Script.WEP && active && !_cinematicMode) {
				fadeLocation(Script.WEP, SUB_LOC_CENTER);
			} else {
				_selectedChar = 0;
			}
			FlxG.mouse.reset();
		}

		private function onShield():void {
			if(_selectedChar == 0 && _curLocation != Script.SHIELD && active && !_cinematicMode) {
				fadeLocation(Script.SHIELD, SUB_LOC_CENTER);
			} else {
				_selectedChar = 0;
			}
			FlxG.mouse.reset();
		}
		
		private function onDeploy():void {
			var pos:Array;
			
			// Level 1 force balanced
			if(FlxG.level == 1 && !_internalTutorialOneComplete) {
				pos = getPositionCount();
				
				if(pos[Script.NAV] > 1 || pos[Script.WEP] > 1 || pos[Script.SHIELD] > 1) {
					if(_curLocation != Script.NAV) {
						onNav();
						_levelDeny = true;
					} else {
						panCamera(SUB_LOC_CENTER);
						_conversation.pushText("We should really only have one person per station for this first battle.", FlxSprite.LEFT);
					}
					return;
				}
			}
			
			// Level 2 force a pair
			if(FlxG.level == 2 && !_internalTutorialTwoComplete) {
				pos = getPositionCount();
				
				if(pos[Script.NAV] != 2 && pos[Script.WEP] != 2 && pos[Script.SHIELD] != 2) {
					if(_curLocation != Script.NAV) {
						onNav();
						_levelDeny = true;
					} else {
						panCamera(SUB_LOC_CENTER);
						_conversation.pushText("We should really try out one of those special weapons, and it'll keep the crew from getting lonely.", FlxSprite.LEFT);
					}
					return;
				}
			}
			
			if(active && !_cinematicMode) {
				PlayState(FlxG.state).deploy();
				
				if(FlxG.level == 1)
					_internalTutorialOneComplete = true;
				if(FlxG.level == 2)
					_internalTutorialTwoComplete = true;
			}
			FlxG.mouse.reset();
		}

		private function updateHUD():void {
			var c:TwoShotCharacter;
			var mini:FlxSprite;
			
			// Character Minis (ignore captain)
			for (var i:int = 1;i < characters.length;i++) {
				c = TwoShotCharacter(characters[i]);
				mini = FlxSprite(_miniCharacters[i]);
				
				if(c.location == Script.NAV) {
					if(c.sub_location == SUB_LOC_LEFT) {
						mini.x = _navigationBtn.x + 32;
					} else if(c.sub_location == SUB_LOC_RIGHT) {
						mini.x = _navigationBtn.x + 80;
					}
					mini.y = _navigationBtn.y + 24;
				} else if(c.location == Script.WEP) {
					if(c.sub_location == SUB_LOC_LEFT) {
						mini.x = _weaponsBtn.x + 32;
					} else if(c.sub_location == SUB_LOC_RIGHT) {
						mini.x = _weaponsBtn.x + 80;
					}
					mini.y = _weaponsBtn.y + 24;
				} else if(c.location == Script.SHIELD) {
					if(c.sub_location == SUB_LOC_LEFT) {
						mini.x = _shieldsBtn.x + 32;
					} else if(c.sub_location == SUB_LOC_RIGHT) {
						mini.x = _shieldsBtn.x + 80;
					}
					mini.y = _shieldsBtn.y + 24;
				} else {
					mini.x = -100;
					mini.y = -100;
				}
			}
			
			// Status Text
			var cnt:Array = getPositionCount();
//			
//			statusChange(_weaponsStatus1, cnt[Script.WEP] > 0);
//			statusChange(_weaponsStatus2, cnt[Script.WEP] > 1);
//			statusChange(_navigationStatus1, cnt[Script.NAV] > 0);
//			statusChange(_navigationStatus2, cnt[Script.NAV] > 1);
//			statusChange(_shieldsStatus1, cnt[Script.SHIELD] > 0);
//			statusChange(_shieldsStatus2, cnt[Script.SHIELD] > 1);

			// Specials Text
			if(cnt[Script.WEP] > 1) {
				_weaponsStatus1.visible = _weaponsStatus2.visible  = true;
			} else {
				_weaponsStatus1.visible = _weaponsStatus2.visible  = false;
			}
			if(cnt[Script.NAV] > 1) {
				_navigationStatus1.visible = _navigationStatus2.visible = true;
			} else {
				_navigationStatus1.visible = _navigationStatus2.visible = false;
			}
			if(cnt[Script.SHIELD] > 1) {
				_shieldsStatus1.visible = _shieldsStatus2.visible = true;
			} else {
				_shieldsStatus1.visible = _shieldsStatus2.visible = false;
			}
			
			// Power Meters
			_wepPower.play(Number(cnt[Script.WEP]).toString());
			_navPower.play(Number(cnt[Script.NAV]).toString());
			_shieldPower.play(Number(cnt[Script.SHIELD]).toString());
		}

//		private function statusChange(st:FlxText, status:Boolean):void {
//			if(status) {
//				st.color = 0xff00ff00;
//			} else {
//				st.color = 0xffff0000;
//			}
//		}

		// ************************ USER CONTROL ***********************************

		/*
		 * Switch the currently viewed location to new_loc and place characters where appropriate
		 */
		public function changeLocation(new_loc:uint, camera_loc:uint):void {
			var c:TwoShotCharacter;
			_conversation.clear();
			
			// Remove characters currently in view and replace
			for (var i:int = 0;i < characters.length;i++) {
				c = TwoShotCharacter(characters[i]);
				
				if(c.location == _curLocation) {
					moveCharacter(i, c.sub_location, false);
				}
				if(new_loc && c.location == new_loc) {
					moveCharacter(i, c.sub_location, true);
				}
			}
			
			// Hide for the intro and appear for everything else
			if(new_loc)
				_conversation.visible = true;
			else
				_conversation.visible = false;
			
			// Remove old background and replace with new
			FlxSprite(_backgrounds[_curLocation]).visible = false;
			FlxSprite(_backgrounds[new_loc]).visible = true;
			
			// Set current location
			_curLocation = new_loc;
			
			// Highlight on hud
			if(_curLocation == Script.WEP) {
				_hudSelector.x = _weaponsBtn.x;
				_hudSelector.y = _weaponsBtn.y;
			} else if(_curLocation == Script.NAV) {
				_hudSelector.x = _navigationBtn.x;
				_hudSelector.y = _navigationBtn.y;
			} else if(_curLocation == Script.SHIELD) {
				_hudSelector.x = _shieldsBtn.x;
				_hudSelector.y = _shieldsBtn.y;
			}
			
			moveCamera(camera_loc);
		}

		/*
		 * Calls changeLocation after a fade
		 */
		public function fadeLocation(new_loc:uint, camera_loc:uint):void {
			_conversation.clear();
			_newLocation = new_loc;
			_newSubLocation = camera_loc;
			FadeState(FlxG.state).fadeOut(0.5, true, fadeLocationComplete);
		}

		public function fadeLocationComplete():void {
			changeLocation(_newLocation, _newSubLocation);
			FadeState(FlxG.state).fadeIn(0.5, false);
			
			// Report upgrade status
			if(_justTransfered) {
				var loc_count:Array = getPositionCount();
				
				if(_curLocation == Script.WEP) {
					if(loc_count[_curLocation] == 1) {
						panCamera(SUB_LOC_LEFT);
						_conversation.pushText("Weapons Power Station Activated\nWe should now be able to do @MORE DAMAGE@.", FlxSprite.LEFT);
					} else if(loc_count[_curLocation] == 2) {
						panCamera(SUB_LOC_RIGHT);
						_conversation.pushText("'The Curveball' AUTO-AIM 5000. Never miss a shot again [not a guarantee], @CURVE AROUND@ your enemies! Now activated in three sizes, large, extra large, and ludicrous sized.", FlxSprite.RIGHT);
						PlayState.loadTutorial(3);
					}
				}
				
				if(_curLocation == Script.NAV) {
					if(loc_count[_curLocation] == 1) {
						panCamera(SUB_LOC_LEFT);
						_conversation.pushText("Navigation Power Station Activated\nWe should now be able to @MOVE FASTER@.", FlxSprite.LEFT);
					} else if(loc_count[_curLocation] == 2) {
						panCamera(SUB_LOC_RIGHT);
						_conversation.pushText("TELEPORT-O-MATIC. It slices, it dices, it @TELEPORTS@, it turns us inside out! It's also now activated.", FlxSprite.RIGHT);
						PlayState.loadTutorial(4);
					}
				}
				
				if(_curLocation == Script.SHIELD) {
					if(loc_count[_curLocation] == 1) {
						panCamera(SUB_LOC_LEFT);
						_conversation.pushText("Shields Power Station Activated\nWe should now have @MORE SHIELDS@.", FlxSprite.LEFT);
						PlayState.loadTutorial(6);
					} else if(loc_count[_curLocation] == 2) {
						panCamera(SUB_LOC_RIGHT);
						_conversation.pushText("ZAPPO REFLECTO. Guaranteed to repel hundreds of different species of bullets. Requires having @AT LEAST ONE SHIELD CHARGE@. Consistent results not guaranteed.", FlxSprite.RIGHT);
						PlayState.loadTutorial(5);
					}
				}
				
				_justTransfered = false;
			}
			
			// First level only
			if(_levelDeny) {
				panCamera(SUB_LOC_CENTER);
				if(FlxG.level == 1) {
					_conversation.pushText("We should really only have one person per station for this first battle.", FlxSprite.LEFT);
				} else if (FlxG.level == 2) {
					_conversation.pushText("We should really try out one of those special weapons, and it'll keep the crew from getting lonely.", FlxSprite.LEFT);
				}
				_levelDeny = false;
			}
		}

		override public function update():void {
			// Clear justSkipped
			_justSkipped = false;
			
			// Trigger Cinematic Skip
			if(FlxG.keys.justPressed("SPACE") || (FlxG.mouse.justPressed() && _skipText.overlapsPoint(FlxG.mouse.x, FlxG.mouse.y))) {
				onSkip();
			}
			
			// Panning
			if(_panning) {
				if(FlxG.scroll.x > CAMERA_SUBLOC_LOOKUP[_panningSubLocation] && _panDelta > 0) {
					_panning = false;
					panCameraComplete();
				} else if(FlxG.scroll.x < CAMERA_SUBLOC_LOOKUP[_panningSubLocation] && _panDelta < 0) {
					_panning = false;
					panCameraComplete();
				} else {
					FlxG.scroll.x += _panDelta;
				}
			} else if(_cinematicMode) {
				_hud.exists = false;
				processCinematics();
			} else if(_controlAfterCinematic) {
				_hud.exists = true;
				processUserControl();
			}

			super.update();
		}

		/*
		 * Transfers a character to another room
		 */
		public function transferCharacter(char:uint, loc:uint):Boolean {
			var loc_count:Array = getPositionCount();
			var f:uint;
			var prev_loc:uint;
			var c:TwoShotCharacter;
			
			c = TwoShotCharacter(characters[char]);
			
			// Text Box Facing
			if((_conversation.x + _conversation.origin.x) > c.x) {
				f = FlxSprite.LEFT; 
			} else {
				f = FlxSprite.RIGHT; 
			}
			
			if(loc == c.location) {
				// Already here
				_conversation.pushText("I'm already here, Sir!", f);
				return false;
			} else if(loc_count[loc] >= 2) {
				// Full
				_conversation.pushText("There's no room for me there, Sir.", f);
				return false;
			} else {
				// Move to New Locations
				prev_loc = c.location;
				c.location = loc;
				if(loc_count[loc] == 0){
					moveCharacter(char, SUB_LOC_LEFT, false);
				} else {
					moveCharacter(char, SUB_LOC_RIGHT, false);
				}
				
				// If there were 2 people at my station and I was to the left, move the right guy to the left
				for (var i:int = 0;i < characters.length;i++) {
					c = TwoShotCharacter(characters[i]);
					if(loc_count[c.location] > 1 && c.location == prev_loc && c.sub_location == SUB_LOC_RIGHT)
							moveCharacter(i, SUB_LOC_LEFT);
				}
				
				fadeLocation(loc, SUB_LOC_CENTER);
				_justTransfered = true;
				
				return true;
			}
		}

		/*
		 * Takes a character and moves them to one of the three screen positions SUB_LOC_LEFT, SUB_LOC_RIGHT, SUB_LOC_CENTER, SUB_LOC_OFF
		 */
		public function moveCharacter(char:uint, pos:uint, vis:Boolean = true):void {
			var c:TwoShotCharacter;
			
			c = TwoShotCharacter(characters[char]);
			c.sub_location = pos;
			c.visible = vis;
			
			if (pos == SUB_LOC_OFF) {
				c.visible = false;
				c.x = -1000;
				c.y = -1000;
			} else {
				c.x = CHAR_SUBLOC_LOOKUP[pos]; 
				c.y = CHAR_Y_POS;
			}
		}

		/*
		 * Takes a character and plays the given animation
		 */
		public function animateCharacter(char:uint, anim:String):void {
			var c:TwoShotCharacter;
			
			c = TwoShotCharacter(characters[char]);
			c.play(anim);
		}

		/*
		 * Kills off a character
		 */
		public function killCharacter(char:uint):void {
			var c:TwoShotCharacter;
			
			c = TwoShotCharacter(characters[char]);
			c.location = Script.DEAD;
			moveCharacter(char, SUB_LOC_OFF, false);
			updateHUD();
			
			FlxG.flash.start(0xffff0000, 1, killCharacterComplete);
		}
		
		private function killCharacterComplete():void {
			if(_cinematicMode) {
				_currentAction = null;
				_conversation.clear();
			}
		}

		/*
		 * Activate character slide
		 */
		public function slideCharacter(char:uint, distance:Number, duration:Number):void {
			var c:TwoShotCharacter;
			
			c = TwoShotCharacter(characters[char]);
			c.slide(distance, duration);
		}

		/*
		 * Moves the camera to either the SUB_LOC_LEFT view or SUB_LOC_RIGHT view
		 */
		public function moveCamera(pos:uint):void {
			_cameraSubLocation = pos;
			FlxG.scroll.x = CAMERA_SUBLOC_LOOKUP[pos];
			_conversation.reset(TEXT_SUBLOC_LOOKUP[pos], _conversation.y);
		}

		public function panCamera(pos:uint):void {
			// Don't pan if we're already here
			if(FlxG.scroll.x != CAMERA_SUBLOC_LOOKUP[pos]) {
				if(FlxG.scroll.x < CAMERA_SUBLOC_LOOKUP[pos])
					_panDelta = 10;
				else
					_panDelta = -10;
				_panningSubLocation = pos;
				_panning = true;
			}
			
			// Move the conversation to the correct side immediately
			_conversation.reset(TEXT_SUBLOC_LOOKUP[pos], _conversation.y);
		}

		private function panCameraComplete():void {
			_panning = false;
			moveCamera(_panningSubLocation);
		}

		private function onSkip():void {
			_justSkipped = true;
		}

		/*
		 * Return a dictionary table of character names and their respective locations
		 */
		public function getPositionTable():Object {
			var c:TwoShotCharacter;
			var lookuptable:Object = new Object();
			
			// Ignore the captain
			for (var i:int = 1;i < characters.length;i++) {
				c = TwoShotCharacter(characters[i]);
				lookuptable[c.name] = c.location;
			}
			return lookuptable;	
		}

		public function getPositionCount():Array {
			var c:TwoShotCharacter;
			var arr:Array = [0, 0, 0, 0]; // All 4 locations empty

			// Ignore the captain
			for (var i:int = 1;i < characters.length;i++) {
				c = TwoShotCharacter(characters[i]);
				arr[c.location]++;
			}
			
			return arr;
		}

		public function set justSkipped(value:Boolean):void {
			_justSkipped = value;
		}

		public function get justSkipped():Boolean {
			return _justSkipped;
		}

		public function switchTo():void {
			var v:Boolean = FlxG.mouse.cursor.visible;
			FlxG.mouse.load(ImgCursor);
			FlxG.mouse.cursor.loadGraphic(ImgCursor, true, false, 24, 24);
			FlxG.mouse.cursor.addAnimation("default", [0]);
			FlxG.mouse.cursor.addAnimation("select", [1]);
			FlxG.mouse.cursor.visible = v;
			FlxG.followTarget = null;
			moveCamera(_cameraSubLocation); // Resets scroll.x
			FlxG.scroll.y = 0;
			FlxG.playMusic(SndAmbient);
			_titleTextBlackout.exists = true;
			_titleTextBlackout.alpha = 1;
			_conversation.clear();
		}
	}
}
