package {

	/**
	 * @author Yori Kvitchko
	 * 
	 * Class used for keeping track of and looping through a set of intervals
	 */
	public class Interval {
		private var _intervals:Array;
		private var _currentInterval:int;
		private var _numIntervals:int;
		private var _timer:Number;
		private var _justTicked:Boolean;
		private var _justLooped:Boolean;

		/*
		 * Intervals is an array of numbers which represents the length of a set of intervals 
		 */
		public function Interval(Intervals:Array) {
			_intervals = Intervals;
			_numIntervals = _intervals.length;
			
			reset();
		}

		public function reset():void {
			_currentInterval = 0;
			_timer = Number(_intervals[_currentInterval]);
			_justTicked = false;
			_justLooped = false;
		}

		public function elapse(elapsedTime:Number):void {
			_timer -= elapsedTime;
			
			if(_timer < 0) {
				// Next Interval
				_currentInterval++;
				// Loops
				if(_currentInterval >= _numIntervals) {
					_currentInterval = 0;
					_justLooped = true;
				}
				// Set new interval time
				_timer = Number(_intervals[_currentInterval]);
				
				// Set justTicked
				_justTicked = true;
			} else {
				_justTicked = false;
				_justLooped = false;
			}
		}

		/*
		 * Percent of the current interval completed
		 */
		public function get percentTicked():Number {
			return (Number(_intervals[_currentInterval]) - _timer) / Number(_intervals[_currentInterval]);
		}

		public function get currentInterval():int {
			return _currentInterval;
		}
		
		public function get numIntervals():int {
			return _numIntervals;
		}

		public function get justTicked():Boolean {
			return _justTicked;
		}
		
		public function get justLooped():Boolean {
			return _justLooped;
		}
	}
}
