package {
	import cinematics.LoseCinState;
	import cinematics.WinCinState;

	import org.flixel.FlxG;
	import org.flixel.FlxSprite;

	public class PlayState extends FadeState {
		/*
		 * Ship View
		 */
		public var sv:SpaceView;
		/*
		 * Internal View
		 */
		public var itsv:InternalTwoShotView;
		/*
		 * Script object
		 */
		private var _script:Script;
		/*
		 * Which tutorial is currently loaded if any
		 */
		public static var tutorial:FlxSprite;
		
		public static var viewedTutorials:Array;
		public static var tutorials:Array;
		
		/*
		 * There is no level 0
		 */
		private var _levelClasses:Array = [null, LevelOne, LevelTwo, LevelThree, LevelFour, LevelFive, LevelSix];

		// TUTORIALS
		
		/*
		 * Internal Navigation
		 */
		[Embed(source="../data/tutorials/internal_navigation.png")] 
		private var ImgTutInternalNav:Class;
		
		/*
		 * External Navigation
		 */
		[Embed(source="../data/tutorials/external_navigation.png")] 
		private var ImgTutExternalNav:Class;
		
		[Embed(source="../data/tutorials/spec_navigation.png")] 
		private var ImgTutSpecNav:Class;
		[Embed(source="../data/tutorials/spec_weapons.png")] 
		private var ImgTutSpecWep:Class;
		[Embed(source="../data/tutorials/spec_shields.png")] 
		private var ImgTutSpecShield:Class;
		[Embed(source="../data/tutorials/shields.png")] 
		private var ImgTutShields:Class;
		[Embed(source="../data/tutorials/ejected.png")] 
		private var ImgTutEjected:Class;
		[Embed(source="../data/tutorials/character_stress.png")] 
		private var ImgTutStress:Class;

		override public function create():void {
			// Tutorials 
			tutorial = null; // none loaded
			
			var t:FlxSprite;
			
			tutorials = new Array();
			tutorials[0] = null;
			viewedTutorials = new Array();
			viewedTutorials[0] = false;
			
			// Internal Nav (1)
			t = new FlxSprite(0, 0, ImgTutInternalNav);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[1] = t;
			viewedTutorials[1] = false;
			
			// Internal Nav (2)
			t = new FlxSprite(0, 0, ImgTutExternalNav);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[2] = t;
			viewedTutorials[2] = false;
			
			// Spec Wep (3)
			t = new FlxSprite(0, 0, ImgTutSpecWep);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[3] = t;
			viewedTutorials[3] = false;
			
			// Spec Nav (4)
			t = new FlxSprite(0, 0, ImgTutSpecNav);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[4] = t;
			viewedTutorials[4] = false;
			
			// Spec Shield (5)
			t = new FlxSprite(0, 0, ImgTutSpecShield);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[5] = t;
			viewedTutorials[5] = false;
			
			// Shields (6)
			t = new FlxSprite(0, 0, ImgTutShields);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[6] = t;
			viewedTutorials[6] = false;
			
			// Ejection (7)
			t = new FlxSprite(0, 0, ImgTutEjected);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[7] = t;
			viewedTutorials[7] = false;
			
			// Stress (8)
			t = new FlxSprite(0, 0, ImgTutStress);
			t.scrollFactor.x = t.scrollFactor.y = 0;
			t.exists = false;
			tutorials[8] = t;
			viewedTutorials[8] = false;
			
			// Internal and Space Views			
			sv = null;
			itsv = new InternalTwoShotView();
			itsv.changeLocation(Script.NAV, InternalTwoShotView.SUB_LOC_CENTER);
			add(itsv);
			
			// Cinematics / Script
			_script = new Script(itsv.getPositionTable());
			
			// Start at 0 (which doesn't exist)
			FlxG.level = 0;
			// Load level 1
			loadInteriorLevel(1);

			// Initialize Fade
			super.create();
			
			// Background Color
			bgColor = 0xff000216;
		}

		override public function update():void {
			if(tutorial && !fading) {
				if(FlxG.mouse.justPressed()) {
					tutorial = null;
				}
			} else {
				super.update();
			}
			
			// Cheating level skip
//			if(FlxG.keys.justPressed("ONE")) {
//				FlxG.level = 1;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("TWO")) {
//				FlxG.level = 2;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("THREE")) {
//				FlxG.level = 3;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("FOUR")) {
//				FlxG.level = 4;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("FIVE")) {
//				FlxG.level = 5;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("SIX")) {
//				FlxG.level = 6;
//				loadExteriorLevel();
//			}
//			if(FlxG.keys.justPressed("SEVEN")) {
//				FlxG.level = 7;
//				loadExteriorLevel();
//			}
			
			// Restart with M key
			if(FlxG.keys.justPressed("M"))
				FlxG.state = new MenuState();
		}

		/*
		 * Shows a single tutorial, ignores the call if it has been shown once already
		 */
		public static function loadTutorial(t:uint, force:Boolean = false):void {
			if(!viewedTutorials[t] || force) {
				tutorial = FlxSprite(tutorials[t]);
				tutorial.exists = true;
				viewedTutorials[t] = true;
			}
		}

		/*
		 * Load specified level
		 */
		private function loadInteriorLevel(level:int, escape:Boolean = false):void {
			// Don't load cinematics if the player lost the last level
			if(level > FlxG.level) {
				// Load cinematics from state machine here, but only if level increased
				itsv.queueupCinematics(_script.getCinematics(itsv.getPositionTable()));
				if(level == 1) {
					itsv.playCinematics(true);
				} else {
					itsv.playCinematics(true);
				}
			} else {
				// Defeat logic here
				itsv.queueupCinematics(_script.getHint(itsv.getPositionTable()));
				itsv.playCinematics(true);
				if(!escape) {
					// Only do defeat logic here which relies on a failed or non-escape
				}
			}
			
			FlxG.level = level;
			
			if(sv)
				sv.visible = sv.active = false;
			itsv.visible = itsv.active = true;
			itsv.switchTo();
		}

		private function loadExteriorLevel():void {
			// Get upgrade levels
			var level:int = FlxG.level;
			
			// Return to menu if all levels beaten
			if(level >= (_levelClasses.length)) {
				winGame();
				return;
			}
			
			FlxG.levels[level] = new _levelClasses[level](itsv.getPositionCount()); // Get upgrade values from isv instead

			if(sv) {
				sv = SpaceView(defaultGroup.replace(sv, FlxG.levels[level]));
			} else {
				sv = SpaceView(add(FlxG.levels[level]));
			}
			
			sv.visible = sv.active = true;
			itsv.visible = itsv.active = false;
			sv.switchTo();
		}

		/*
		 * Called when the player has won the level in space view
		 * Destroy old level here
		 */
		public function victory():void {
			fadeOut(0.5, true, victoryComplete);
		}

		private function victoryComplete():void {
			// Destroy old level
			SpaceView(FlxG.levels[FlxG.level]).destroy();
			FlxG.levels[FlxG.level] = null;
			// Load next level
			loadInteriorLevel(FlxG.level + 1);
			
			fadeIn(0.5, true);
		}

		/*
		 * Called when the player has lost the level in space view
		 * Destroy old level here
		 */
		public function defeat(escape:Boolean):void {
			if(escape)
				fadeOut(0.5, true, escapeComplete);
			else
				fadeOut(0.5, true, defeatComplete);
		}

		private function escapeComplete():void {
			// Destroy old level
			SpaceView(FlxG.levels[FlxG.level]).destroy();
			FlxG.levels[FlxG.level] = null;
			// Re-load the level
			loadInteriorLevel(FlxG.level, true);
			
			fadeIn(0.5, true);			
		}

		private function defeatComplete():void {
			// Destroy old level
			SpaceView(FlxG.levels[FlxG.level]).destroy();
			FlxG.levels[FlxG.level] = null;
			// Re-load the level
			loadInteriorLevel(FlxG.level);
			
			fadeIn(0.5, true);			
		}

		/*
		 * Called by internal ship view to indicate the player has chosen to deploy
		 */
		public function deploy():void {
			fadeOut(0.5, true, deployComplete);
		}

		private function deployComplete():void {
			loadExteriorLevel();
			fadeIn(0.5, true);
		}

		override public function render():void {
			super.render();
			if(tutorial) {
				tutorial.render();
			}
		}
		
		public function loseGame():void {
			FlxG.music.stop();
			FlxG.state = new LoseCinState();
		}
		
		public function winGame():void {
			FlxG.music.stop();
			FlxG.state = new WinCinState();
		}
	}
}
