package {
	import org.flixel.FlxG;

	import flash.display.Sprite;

	[SWF(width="640", height="640", backgroundColor="#000000")]

	/*
	 * The Script class is simply a helper class for
	 * loading scripts from files
	 */

	public class Script extends Sprite {

		// Constants 
		public static const DEAD : uint = 0;
		public static const NAV : uint = 1;
		public static const WEP : uint = 2;
		public static const SHIELD : uint = 3;
		public static const TEMP : uint = 4;

		[Embed(source="story/captain.txt",mimeType="application/octet-stream")] 
		private var  CaptainScript : Class;

		[Embed(source="story/viki.txt",mimeType="application/octet-stream")] 
		private var  VikiScript : Class;

		[Embed(source="story/fred.txt",mimeType="application/octet-stream")] 
		private var  FredScript : Class;

		[Embed(source="story/droid.txt",mimeType="application/octet-stream")] 
		private var  DroidScript : Class;

		[Embed(source="story/droid_fred.txt",mimeType="application/octet-stream")] 
		private var  DroidFredScript : Class;

		[Embed(source="story/viki_droid.txt",mimeType="application/octet-stream")] 
		private var  VikiDroidScript : Class;

		[Embed(source="story/fred_viki.txt",mimeType="application/octet-stream")] 
		private var  FredVikiScript : Class;

		[Embed(source="story/hints.txt",mimeType="application/octet-stream")] 
		private var  HintScript : Class;

		
		/*
		 * Cinematic timelines per character
		 */
		public var cCins : Array; // Captain
		public var vCins : Array; // Viki
		public var fCins : Array; // Fred
		public var dCins : Array; // Droid
		public var dfCins : Array; // Droid+Fred
		public var vdCins : Array; // Viki+Droid
		public var fvCins : Array; // Fred+Viki

		public var hints : Array;

		public var oldState : Object;

		public function Script(state : Object = null) {
			oldState = state;
			
			cCins = new Array();
			vCins = new Array();
			fCins = new Array();
			dCins = new Array();
			dfCins = new Array();
			vdCins = new Array();
			fvCins = new Array();
			
			hints = new Array();
			
			// Read in script files, store cinematics in neat-o data structures
			parseScript(cCins, new CaptainScript());
			parseScript(vCins, new VikiScript());
			parseScript(fCins, new FredScript());
			parseScript(dCins, new DroidScript());
			parseScript(dfCins, new DroidFredScript());
			parseScript(vdCins, new VikiDroidScript());
			parseScript(fvCins, new FredVikiScript());
			
			// Read in the Captain's hints
			parseHints(new HintScript());

//			var o:Object = new Object;
//			o["Viki"] = NAV;
//		    o["Fred"] = SHIELD;
//			o["Droid"] = WEP;
//			test();
//			test();	
//			test();
//			test();
//			test();
		}

		private function test(o : Object = null) : Boolean {
			if (o == null) {
				o = new Object();
				o["Viki"] = NAV;
				o["Fred"] = SHIELD;
				o["Droid"] = WEP;
			}

			var cinematics : Array = getCinematics(o);
			
			trace(cinematics.length);
			for each (var c:Cinematic in cinematics) {				
				trace(c);
				//for each (var a:Action in c.actions) {trace(a);}
			}
			
//			var hint : Cinematic = getHint().pop();
//			trace(hint);
//			trace(hint.actions[0]);
//			trace("--------");
			 
			FlxG.level++;
			 
			return true;
		}

		// Synthetic cinematic for when Viki is smoking people out
		private function smoke() : Cinematic {
			var smoke : Cinematic = new Cinematic(0, "Smoke");
			smoke.pushAction(new Action());
			return  smoke;
		}

		// Read a hint file and store them
		private function parseHints(script : String) : void {
			var lineRE : RegExp = /(.)([:\.])(.*)/;
			var commentRE : RegExp = /^[#!].*/;
			
			script.replace(commentRE, "");
			
			var lines : Array = script.split(lineRE);
			
			for (var j : uint = 0;j < lines.length;j += 4) {
				var character : String = lines[j + 1];
				var content : String = lines[j + 3];
					
				if (content != null) {
					hints.push(new Action(character, "speak", content));
				}
			}
		}

		// Returns the Captain's next quip to a losing player
		public function getHint(state : Object = null) : Array {
			// Everyone's dead, cappy's not happy
			if (state["Droid"] == DEAD && state["Viki"] == DEAD && state["Fred"] == DEAD) {
    			
				var crazyCin : Cinematic = new Cinematic(-1, "Cappy Goes Crazy");
				var crazyAct : Action = new Action("C", "speak", "Oh no, we don't have enough crew to man the ship, " + "I might be stuck here forever?!?");
				var endAct : Action = new Action("C", "END");
    			
				crazyCin.pushAction(crazyAct);
				crazyCin.pushAction(endAct);	
				
				return new Array(crazyCin);
			} else { 
				// No, it's cool, someone's alive
				var eulogy : Array = checkDeath(state);
				
				if (eulogy != null) {
					return eulogy;
				} else if (hints.length > 0) {
					return new Array(new Cinematic(-1, "A Word from Our Captain", hints.pop()));
				} else {
					return null;
				}	
			}			
		}

		// Read a script file and store the relevant info
		private function parseScript(cins : Array, contents : String) : void {
			// a blank line terminates a cinematic block
			var sceneRE : RegExp = /\[(\d+):(.*)\]([^\n]*)\n/;
			var lineRE : RegExp = /(.)([:\.])(.*)/;
			var commentRE : RegExp = /^[#!].*/;
			var actionRE : RegExp = /(\w+)\((\w*)\)/;
			
			contents.replace(commentRE, "");
			//contents.replace(actionRE, "");

			var blocks : Array = contents.split(sceneRE);

			// Parse each entire cinematic block
			for (var i : uint = 1;i < blocks.length;i += 4) {
				var level : uint = uint(blocks[i]);
				var title : String = String(blocks[i + 1]);
				//var options : String = String(blocks[i + 2]);
				var script : String = String(blocks[i + 3]);
			
				var lines : Array = script.split(lineRE);
				
				var cinematic : Cinematic = new Cinematic(level, title);
				
				// Parse the individual actions/lines
				for (var j : uint = 0;j < lines.length;j += 4) {
					var character : String = lines[j + 1];
					var content : String = lines[j + 3];
					
					// Parse lines of speech
					if (lines[j + 2] == ":") { 
						// Line of text
						cinematic.pushAction(new Action(character, "speak", content));
					} else if (lines[j + 2] == ".") { // Parse an Action
						// Line specifying an action
						var action : Array = content.split(actionRE);
						var command : String = action[1];
						var param : String = action[2];
						
						if (command == "MOVE") {
							if (param == "SHIELD" || param == "SHIELDS") {
								param = SHIELD.toString();
							} else if (command == "NAV") {
								param = NAV.toString();
							} else if (command == "WEP") {
								param = WEP.toString();
							}
						}
						
						cinematic.pushAction(new Action(character, command, param));
					}
				}
				
				cins.push(cinematic);
			}
		}

		// See if anyone died and update old state reference
		private function checkDeath(newState : Object) : Array {
			if (newState != null) {
				for (var character : String in newState) {
					if (newState[character] == DEAD && oldState[character] != DEAD) {
						oldState = newState;
						return mourn(character);
					}
				}
			}
			
			return null;
		}

		// Don't mourn later
		public function kill(c : String) : void {
			oldState[c] = DEAD;
		}

		// Whoever was closest to this character should say some words
		private function mourn(c : String) : Array {
			var closest : String = "C"; // Who knew them best

			var cinematics : Array = new Array();
			var capEulogy : Cinematic = new Cinematic(0, "Mourning the loss of " + c + " :(");
			capEulogy.pushAction(new Action("C", "speak", c + " was a good crew member, and will be remembered fondly."));
			cinematics.push(capEulogy);
			
			var secondEulogy : Cinematic = new Cinematic(0, "We were so close");
			secondEulogy.pushAction(new Action(closest, "speak", "Too bad "+c+" suffocated before we could make it back in time."));
			cinematics.push(secondEulogy);
			
			return cinematics;
		}

		/*
		 * Input: Dictionary containing the location of each character
		 * Output: An array of cinematics relevant to this configuration
		 */
		public function getCinematics(state : Object = null) : Array {
			var cinematics : Array = new Array();

			var mournCin : Array = checkDeath(state);
			if (mourn.length > 0) cinematics.concat(mournCin);

			// Play 1 Dialog and 1 Monologue
			if (state["Fred"] != DEAD
				&& state["Fred"] == state["Viki"]
				&& fvCins[0].level <= FlxG.level) {
				nextCin(fvCins, cinematics);
			}
			if (state["Viki"] != DEAD 
			    && state["Viki"] == state["Droid"]
			    && vdCins[0].level <= FlxG.level) { 
				nextCin(vdCins, cinematics);
			}
			if (state["Droid"] != DEAD 
			    && state["Droid"] == state["Fred"]
			    && dfCins[0].level <= FlxG.level) { 
				nextCin(dfCins, cinematics);
			}
			
			if (isAlone("Fred", state)) {
				nextCin(fCins, cinematics);
			}
			if (isAlone("Viki", state)) {
					nextCin(vCins, cinematics);
			}
			if (isAlone("Droid", state)) {
				nextCin(dCins, cinematics);
			}

     		// see if anyone died, if so, mourn their loss
     		for each (var c : Cinematic in cinematics) {
     			for each (var a : Action in c.actions) {
     				if (a.type == "DIE") {
     					// say a few words about the deceased
     					cinematics.concat(mourn(a.character));
     					
     					if (a.character == "F") state["Fred"] = DEAD;
     					else if (a.character == "V") state["Viki"] = DEAD;
     					else if (a.character == "D") state["Droid"] = DEAD;
     				}
     			}
     		}
     		
     		// Always play the captain's monologue
    		// If everyone else is dead, he get's wacky
    		if (state["Droid"] == DEAD &&
    			state["Viki"] == DEAD &&
    			state["Fred"] == DEAD &&
    			FlxG.level <= 6) {
    			
    			cinematics.push(goCrazy());	
    		} else { // No, it's cool, someone's alive
				nextCin(cCins, cinematics);
			}
     
			return cinematics;
		}

		private function goCrazy() : Cinematic {
			var crazyCin : Cinematic = new Cinematic(-1, "Cappy Goes Crazy");
			var crazyAct : Action = new Action("C", "speak", "Oh no, we don't have enough crew to man the ship, " + "I might be stuck here forever?!?");
			var endAct : Action = new Action("C", "END");
    			
			crazyCin.pushAction(crazyAct);
			crazyCin.pushAction(endAct);
    			
			return crazyCin;
		}

		// Is the given player alone?
		private function isAlone(c : String, state : Object) : Boolean {
			if (state[c] == DEAD) return false;
						
			for (var c2 : String in state) {
				if (c != c2 && state[c] == state[c2]) {
				 	return false;
				}
			}
			return true;
		}

		/*
		 * Input: Array of cinematics
		 * Output: The earliest cinematic not yet played in a sequence
		 */
		public function nextCin(cins : Array, output : Array = null) : Cinematic {
			for each (var c:Cinematic in cins) {
				if (c != null && !c.used && c.level <= FlxG.level) {
					if (output != null) {
						c.used = true;
						output.push(c);
					}
					return c; 
				}
			}
			return null;
		}
	}
}