package {
	import org.flixel.FlxObject;
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxU;

	/**
	 * @author Yori Kvitchko
	 */
	public class SpaceCharacter extends FlxSprite {
		[Embed(source='../data/characters/ejected/fred.png')]
		private const ImgFredEjected:Class;
		
		[Embed(source='../data/characters/ejected/fred_head.png')]
		private const ImgFredEjectedHead:Class;
		
		[Embed(source='../data/characters/ejected/viki.png')]
		private const ImgVikiEjected:Class;
		
		[Embed(source='../data/characters/ejected/viki_head.png')]
		private const ImgVikiEjectedHead:Class;
		
		[Embed(source='../data/characters/ejected/droid.png')]
		private const ImgDroidEjected:Class;
		
		[Embed(source='../data/characters/ejected/droid_head.png')]
		private const ImgDroidEjectedHead:Class;
		
		/*
		 * Character being ejected
		 */
		public var char:uint;
		
		public var ejected:Boolean;
		
		private var _time:Number;
		private var _head:FlxSprite;
		private var _headCollisionDummy:FlxObject;
		
		public function SpaceCharacter(c:uint) {
			super(-100, -100);
			char = c;
			ejected = false;
			
			if(char == InternalTwoShotView.FRED) {
				loadGraphic(ImgFredEjected, true, false, 24, 36);
			} else if(char == InternalTwoShotView.VIKI) {
				loadGraphic(ImgVikiEjected, true, false, 24, 36);
			} else if(char == InternalTwoShotView.DROID) {
				loadGraphic(ImgDroidEjected, true, false, 24, 36);
			}
			
			addAnimation("default", [0, 1, 2, 1], 5);
			play("default");
			
			_head = new FlxSprite(-20, -38);
			_head.solid = false;
			_head.moves = false;
			if(char == InternalTwoShotView.FRED) {
				_head.loadGraphic(ImgFredEjectedHead, true);
			} else if(char == InternalTwoShotView.VIKI) {
				_head.loadGraphic(ImgVikiEjectedHead, true);
			} else if(char == InternalTwoShotView.DROID) {
				_head.loadGraphic(ImgDroidEjectedHead, true);
			}
			_head.addAnimation("grow", [0, 1, 2, 3, 4, 5, 6, 7, 8, 9], 1, false);
			
			_headCollisionDummy = new FlxObject();
			_headCollisionDummy.moves = false;
			
			exists = false;
			_time = 0;
		}

		override public function update():void {
			_time += FlxG.elapsed;
			
			super.update();
			
			_head.overlaps(this);
			
			// Move in relation to character
			_head.x = x - 16;
			_head.y = y - 38 + (_head.frame < 4 ? -2 : 0); // Adjust for smaller head sizes
			_head.update();
			
			// Scale the head based on a sin wave
			_head.scale.x = _head.scale.y = 1 + (Math.sin(_time * 2 * Math.PI) * 0.2);
			
			// Update the head collision dummy
			_headCollisionDummy.width = _headCollisionDummy.height = 16 + 4 * _head.frame;
			_headCollisionDummy.x = _head.x + (_head.width - _headCollisionDummy.width) / 2;
			_headCollisionDummy.y = _head.y + (_head.height - _headCollisionDummy.height);
			
			if(x < 0) {
				x = 0;
				velocity.x = -velocity.x;
			}
			if((x + width) > SpaceView.LEVEL_WIDTH) {
				x = SpaceView.LEVEL_WIDTH - width;
				velocity.x = -velocity.x;
			}
			if(y < 0) {
				y = 0;
				velocity.y = -velocity.y;
			}
			if((y + width) > SpaceView.LEVEL_HEIGHT) {
				y = SpaceView.LEVEL_HEIGHT - height;
				velocity.y = -velocity.y;
			}
			
			// Head Pop
			if(_time > 11) {
				kill();
			}
		}

		/*
		 * Custom overlap function
		 */
		override public function overlaps(Obj:FlxObject):Boolean {
			if(super.overlaps(Obj) || _headCollisionDummy.overlaps(Obj)) {
				// Overlap with body or head
				return true;
			} else {
				// No overlap
				return false;
			}
		}

		public function eject(X:Number, Y:Number):void {
			reset(X, Y);
			_time = 0;
			_head.play("grow", true);
			velocity = FlxU.rotatePoint(200, 0, 0, 0, Math.random() * 360);
			ejected = true;
			
			// Move us away from the ship
			x += velocity.x * 0.5;
			y += velocity.y * 0.5;
		}

		
		override public function reset(X:Number, Y:Number):void {
			super.reset(X, Y);
			_head.x = x - 16;
			_head.y = y - 38 + (_head.frame < 4 ? -2 : 0); // Adjust for smaller head sizes
			_head.update();
		}

		override public function render():void {
			super.render();
			_head.render();
		}

		override public function kill():void {
			super.kill();
			PlayState(FlxG.state).sv.ejecteeKill(char);
			FlxG.flash.start(0xffff0000, 0.5);
			ejected = true;
		}
		
		public function pickUp():void {
			x = -100;
			y = -100;
			exists = false;
			ejected = false;
		}
	}
}
