package {
	import base.Proj;

	import bosses.Boss;
	import bosses.BossComp;
	import bosses.BossHealth;

	import player.BangImpact;
	import player.PlayerShip;
	import player.PlayerShipHealth;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSound;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;
	import org.flixel.FlxTileblock;
	import org.flixel.FlxU;

	public class SpaceView extends FlxGroup {
		// CONSTANTS
		public static const LEVEL_WIDTH:int = 800;
		public static const LEVEL_HEIGHT:int = 800;

		/*
		 * Time until level start
		 */
		private static const COUNTDOWN:Number = 3;
		/*
		 * Countdown until escape has succeeded
		 */
		private static const ESCAPE_COUNTDOWN:Number = 3;
		/*
		 * Time from defeat/victory to moving onto next level
		 */
		private static const RESULT_COUNTDOWN:Number = 3;

		// VARIABLES
		
		/*
		 * Group of Impact Objects
		 */
		public static var impacts:FlxGroup;
		/*
		 * Player ship
		 */
		public static var ship:PlayerShip;
		/*
		 * Player ship's group of projectiles
		 */
		public static var shipProjectiles:FlxGroup;
		/*
		 * Ship healthbar
		 */
		public static var shipHealth:PlayerShipHealth;
		/*
		 * Ejected from the ship character
		 */
		public static var ejectee:SpaceCharacter;
		/*
		 * Bang visual for when the ship gets hit
		 */
		private var shipBang:BangImpact;
		/*
		 * The current boss of the level
		 */
		public static var enemy:Boss;
		/*
		 * Enemy health bar
		 */
		public static var enemyHealth:BossHealth;
		/*
		 * The projectiles of the current boss of the level
		 */
		public static var enemyProjectiles:FlxGroup;
		/*
		 * Progressive explosions indicating health
		 */
		public static var enemyExplosions:ExplosionSet;
		/*
		 * A set of explosions for when the boss dies
		 */
		public static var explosions:ExplosionSet;
		/*
		 * The name of the level / boss
		 */
		public var enemyName:String;

		private var _border:FlxGroup;
		
		private var _wormhole:FlxSprite;
		
		/*
		 * Shields down warning graphic
		 */
//		private var _shieldsDown:FlxSprite;
//		private var _shieldsDownAlphaDelta:Number;
		private var _countdownText:FlxText;
		private var _countdownTimer:Number;
		private var _countdownSound:FlxSound;
		private var _resultText:FlxText;
		private var _resultTimer:Number;
		private var _nameText:FlxText;
		private var _ejecteeText:FlxText;
		private var _ejectedThisRound:Boolean;
		private var _ejecteeSound:FlxSound;
		// Trying to escape?
		private var _escaping:Boolean;
		// Escape succeeded?
		private var _escaped:Boolean;
		// Escape Timer
		private var _escapeTimer:Number;

		// Kill Timer
		private var _killTimer:Number;
		// Kill Timer Text
		private var _killText:FlxText;
		
		private var _playsound:Boolean=true;

		// EMBEDDED RESOURCES

		/*
		 * Stars background
		 */
		[Embed(source='../data/bgtiles.png')]
		private const ImgBG:Class;
		
		[Embed(source="../data/wormhole.png")] 
		private var ImgWormhole:Class;
		
		[Embed(source="../data/cursor.png")] 
		private var ImgCursor:Class;

		/*
		 * Shields down warning splat
		 */
		[Embed(source='../data/shields_down.png')]
		private const ImgShieldsDown:Class;
		
		[Embed(source='../data/sounds/enemy_loop.mp3')]
		private const SndEnemyLoop:Class;
		
		[Embed(source='../data/sounds/count_in.mp3')]
		private const SndCountdown:Class;
		
		[Embed(source='../data/sounds/help.mp3')]
		private const SndEjectee:Class;
		
		[Embed(source='../data/sounds/warp_in.mp3')]
		private const SndWarpIn:Class;
		
		[Embed(source='../data/sounds/warp_out.mp3')]
		private const SndWarpOut:Class;
		
//		[Embed(source='../data/sounds/cheering.mp3')]
//		private const SndEnemyDoomstroyed:Class;
//
//		[Embed(source='../data/sounds/you_doomstroyed.mp3')]
//		private const SndYouDoomstroyed:Class;
//
//		[Embed(source='../data/sounds/yougotaway.mp3')]
//		private const SndComeBackHere:Class;


		/*
		 * Passed an array of ship upgrades from 0 to 2
		 * [ Weapons, Shields, Navigation ]
		 */
		public function SpaceView(shipUpgrades:Array, name:String):void {
			super();

			// Name
			enemyName = name;

			// Background
			initBG();
			
			// Impact group
			impacts = new FlxGroup();

			// Wormhole
			_wormhole = new FlxSprite(0, 0); // y=492 for exact center at bottom border
			_wormhole.loadGraphic(ImgWormhole, true, false, 420, 296);
			_wormhole.addAnimation("pulse", [4, 3, 2, 1, 0], 10);
			_wormhole.addAnimation("suck", [0, 1, 2, 3, 4], 10);
			_wormhole.play("pulse");
			add(_wormhole);

			// Player Ship
			initPlayerShip(shipUpgrades);
			add(ship);
			add(shipProjectiles);
			
			_wormhole.exists = false;
			_wormhole.x = ship.x + ship.origin.x - _wormhole.origin.x;
			_wormhole.y = ship.y + ship.origin.y - _wormhole.origin.y;
			
			// Player Ship Bang (when it gets hit)
			shipBang = new BangImpact();
			impacts.add(shipBang);
			
			// Boss of the level
			enemyProjectiles = new FlxGroup();
			initBoss();
			// Update once in case initial position depends on one call of the update function
			enemy.update();
			// De-Activate the boss until the countdown has started
			enemy.active = false;
			
			// Boss Progressive Explosions
			enemyExplosions = new ExplosionSet();
			enemyExplosions.explodeSubTime = 0;
			add(enemyExplosions);
			
			// Boss Death Explosions
			explosions = new ExplosionSet();
			add(explosions);
			
			// Add Splat and Thud Graphics
			impacts.add(enemy.splatImagePool);
			impacts.add(enemy.thudImagePool);

			// Add Impacts
			add(impacts);
			
			// Ejected character Picking
			
			// Add living characters
			var chars:Array = new Array();
			if(TwoShotCharacter(PlayState(FlxG.state).itsv.characters[InternalTwoShotView.FRED]).location != Script.DEAD) {
				chars.push(InternalTwoShotView.FRED);
			}
			if(TwoShotCharacter(PlayState(FlxG.state).itsv.characters[InternalTwoShotView.VIKI]).location != Script.DEAD) {
				chars.push(InternalTwoShotView.VIKI);
			}
			if(TwoShotCharacter(PlayState(FlxG.state).itsv.characters[InternalTwoShotView.DROID]).location != Script.DEAD) {
				chars.push(InternalTwoShotView.DROID);
			}
			
			// Pick a random living character
			ejectee = new SpaceCharacter(chars[Math.floor(Math.random() * chars.length)]);
			add(ejectee);
			_ejectedThisRound = false;
			
			// Text to inform you someone has been ejected
			_ejecteeText = new FlxText(0, FlxG.height - 80, FlxG.width, "");
			_ejecteeText.scrollFactor.x = 0;
			_ejecteeText.scrollFactor.y = 0;
			_ejecteeText.alignment = "center";
			_ejecteeText.size = 32;
			add(_ejecteeText);

			// Ship Health
			shipHealth = new PlayerShipHealth(10, 616);
			add(shipHealth);
			
			// Boss Health
			enemyHealth = new BossHealth(486, 616, enemy);
			add(enemyHealth);

			// Level Boundary
			add(createBorder());
			
//			// Shields Down Warning Graphic
//			_shieldsDown = new FlxSprite(0, 0, ImgShieldsDown);
//			_shieldsDown.scrollFactor.x = 0;
//			_shieldsDown.scrollFactor.y = 0;
//			_shieldsDown.solid = false;
//			_shieldsDown.moves = false;
//			_shieldsDown.visible = false;
//			_shieldsDown.alpha = 0.01;
//			_shieldsDownAlphaDelta = 0.01;
//			add(_shieldsDown);
			
			// Countdown text
			_countdownText = new FlxText(0, FlxG.height / 2 - 50, FlxG.width, " ");
			_countdownText.scrollFactor.x = 0;
			_countdownText.scrollFactor.y = 0;
			_countdownText.alignment = "center";
			_countdownText.size = 128;
			add(_countdownText);
			
			_countdownSound = new FlxSound();
			_countdownSound.loadEmbedded(SndCountdown);
			
			// Name Text
			_nameText = new FlxText(0, FlxG.height / 3 - 60, FlxG.width, name);
			_nameText.scrollFactor.x = 0;
			_nameText.scrollFactor.y = 0;
			_nameText.alignment = "center";
			_nameText.size = 64;
			add(_nameText);
			
			// Start countdown timer
			_countdownTimer = COUNTDOWN;
			
			// Kill Timer
			_killTimer = 0;
			_killText = new FlxText(5, 5, FlxG.width, "Time: 0");
			_killText.scrollFactor.x = 0;
			_killText.scrollFactor.y = 0;
			_killText.size = 16;
			add(_killText);
			
			// Level result text
			_resultText = new FlxText(0, FlxG.height / 2 - 100, FlxG.width, "");
			_resultText.scrollFactor.x = 0;
			_resultText.scrollFactor.y = 0;
			_resultText.alignment = "center";
			_resultText.size = 64;
			_resultText.visible = false;
			add(_resultText);
			
			// Initialize result timer
			_resultTimer = RESULT_COUNTDOWN;
			
			// Initialize escape logic
			_escapeTimer = 0;
			_escaping = false;
			_escaped = false;
		}

		
		
		private function startFollow():void {
			FlxG.follow(ship, 5);
			FlxG.followBounds(0, 0, LEVEL_WIDTH, LEVEL_HEIGHT);
		}

		private function initBG():void {
			addStarGroup(ImgBG, 10);
		}

		/*
		 * Initialize this Level's Boss
		 */
		protected function initBoss():void {
			// No Default boss
		}

		/*
		 * Initialize the player ship
		 */
		private function initPlayerShip(shipUpgrades:Array):void {
			// Ship and object pool for bullets
			shipProjectiles = new FlxGroup();
			ship = new PlayerShip(LEVEL_WIDTH / 2, LEVEL_HEIGHT - 100, shipProjectiles, shipUpgrades);
			ship.update();
			ship.active = false;
			ship.exists = false;
		}

		/*
		 * Create the Border
		 */
		private function createBorder(x:Number = 0, y:Number = 0, w:Number = LEVEL_WIDTH, h:Number = LEVEL_HEIGHT):FlxGroup {
			_border = new FlxGroup();
			_border.visible = false;
			// Ceiling
			_border.add(new FlxTileblock(x, y, w, 1));
			// Floor
			_border.add(new FlxTileblock(x, h - 1, w, 1));
			// Left
			_border.add(new FlxTileblock(x, y + 1, 1, h - 2));
			// Right
			_border.add(new FlxTileblock(w - 1, y + 1, 1, h - 2));
			return _border;
		}

		private function addStarGroup(Img:Class, e:Number = 100):void {
			var _bg:FlxTileblock = new FlxTileblock(0, 0, LEVEL_WIDTH, LEVEL_HEIGHT);
			_bg.loadGraphic(Img, e);
			_bg.solid = false;
			add(_bg);
		}

		override public function update():void {
			// Timers
			_countdownTimer -= FlxG.elapsed;
			
			// Result Timer
			if(ship.dead || enemy.dead) {
				_resultTimer -= FlxG.elapsed;
				
				ejectee.active = false;
				
				updateWormhole(_resultTimer / RESULT_COUNTDOWN);

				// Result
				_resultText.visible = true;
				if(ship.dead) {
					if(_escaped) {
						_resultText.text = "YOU GOT AWAY!";
						_resultText.color = 0xffffffff;
//						if(_playsound)
//						{
//							FlxG.play(SndComeBackHere);
//							_playsound=false;
//						}
					} else {
						_resultText.text = "YOU HAVE BEEN\nDOOMSTROYED!";
						_resultText.color = 0xffff0000;
//						if(_playsound)
//						{
//							_playsound=false;
//							FlxG.play(SndYouDoomstroyed);
//						}
					}
					ship.stopFiring();
				}
				if(enemy.dead) {
					_resultText.text = "ENEMY\nDOOMSTROYED!";
					_resultText.color = 0xffffffff;
					if(_playsound)
					{
						_playsound=false;
//						FlxG.play(SndEnemyDoomstroyed);
					}
					
				}
				
				if(!_wormhole.exists) {
//					FlxG.play(SndWarpOut);
					_wormhole.exists = true;
					_wormhole.play("suck");
				}
				_wormhole.x = ship.x + ship.origin.x - _wormhole.origin.x;
				_wormhole.y = ship.y + ship.origin.y - _wormhole.origin.y;
			}
			
			// Kill Timer
			if(enemy.active && enemy.exists) {
				_killTimer += FlxG.elapsed;
				_killText.text = "Time: " + _killTimer.toFixed(2);
			}
			
			// Escape Timer
			if(_escaping) {
				_escapeTimer -= FlxG.elapsed;
				
				if(_escapeTimer < 0) {
					_escaped = true;
					ship.kill();
				}
			}
			
			// Countdown Logic (checks for enemy active to prevent extra work once we've started the level
			if(!enemy.active) {
				updateWormhole(_countdownTimer / COUNTDOWN);
				
				if((_countdownTimer < (COUNTDOWN - 0.5)) && !_countdownSound.playing) {
					_countdownSound.play();
//					FlxG.play(SndWarpIn);
				}
				_countdownText.text = Math.ceil(_countdownTimer).toString();
				if(_countdownTimer < 0) {
					FlxG.playMusic(SndEnemyLoop);
					enemy.active = true;
					ship.active = true;
					_wormhole.exists = false;
					ship.startFiring();
					_countdownText.exists = false;
					remove(_nameText);
				}
			}
			
			super.update();
			
			//Overlap
			FlxU.overlap(ship, enemy, enemyShipOverlap);
			FlxU.overlap(ship, enemyProjectiles, enemyShipOverlap);
			FlxU.overlap(shipProjectiles, enemy, bulletEnemyOverlap);
			// Overlap Ejectee with Ship
			if(!ship.invincible && ejectee.exists && ejectee.overlaps(ship))
				ejecteePickup();
			// Overlap Reflected Bullets
			overlapReflectedGroup(enemy, enemyProjectiles);
			//Collision
			FlxU.collide(ship, _border);
			
			if(_resultTimer < 0) {
				if(!PlayState.viewedTutorials[7] && _ejectedThisRound) {
					PlayState.loadTutorial(7);
				} else {
					if(ship.dead)
						PlayState(FlxG.state).defeat(_escaped);
					if(enemy.dead)
						PlayState(FlxG.state).victory();
				}
			}
			
			//			// Shields Down Warning
//			if(ship.health == 1) {
//				_shieldsDown.alpha += _shieldsDownAlphaDelta;
//				
//				_shieldsDown.visible = true;
//				if(_shieldsDown.alpha > 0.5) {
//					_shieldsDown.alpha = 0.5;
//					_shieldsDownAlphaDelta = -_shieldsDownAlphaDelta;
//				}
//				if(_shieldsDown.alpha <= 0) {
//					_shieldsDown.alpha = 0;
//					_shieldsDownAlphaDelta = -_shieldsDownAlphaDelta;
//				}
//			} else {
//				_shieldsDown.visible = false;
//			}
			
			// Escape
			if(FlxG.keys.justPressed("R")) {
				_escaping = true;
				_escapeTimer = ESCAPE_COUNTDOWN;
				_countdownText.exists = true;
			}
			// Escaping
			if(_escaping) {
				if(_escapeTimer > 0) {
					_countdownText.text = Math.ceil(_escapeTimer).toString();
				} else {
					_countdownText.exists = false;
				}
			}
			
			// TEMPORARY, REMOVE LATER, USED TO SKIP LEVELS
//			if(FlxG.keys.justPressed("ESCAPE"))
//				enemy.kill();
		}

		private function updateWormhole(percent:Number):void {
			if(percent > 0.5) {
				// Move ejectee towards wormhole
				if(ejectee.exists) {
					var x:Number;
					var y:Number;
					x = (_wormhole.x + _wormhole.origin.x) + (ejectee.x - (_wormhole.x + _wormhole.origin.x)) * 0.97;
					y = (_wormhole.y + _wormhole.origin.y) + (ejectee.y - (_wormhole.y + _wormhole.origin.y)) * 0.97;
					ejectee.reset(x, y);
				}
				//
				_wormhole.exists = true;
				_wormhole.scale.x = _wormhole.scale.y = 1 - (percent - 0.5) * 2;
			} else if (percent > 0) {
				// Very ghetto
				if(_countdownTimer > 0)
					ship.exists = true;
				else
					ship.exists = false;
					
				//
				if(ejectee.exists) {
					ejecteePickup();
				}
				_wormhole.exists = true;
				_wormhole.scale.x = _wormhole.scale.y = percent * 2;
			} else {
				_wormhole.exists = false;
			}
		}

		/*
		 * Eject
		 */
		public function ejecteeEject():void {
			ejectee.eject(ship.x, ship.y);
			_ejecteeSound = FlxG.play(SndEjectee);
			switch (ejectee.char) { 
				case InternalTwoShotView.FRED:	
					_ejecteeText.text = "Save FRED he just Ejected!";
					break ; 
				case InternalTwoShotView.VIKI:	
					_ejecteeText.text = "Save VIKI she just Ejected!";
					break ; 
				case InternalTwoShotView.DROID:	
					_ejecteeText.text = "Save BRO he just Ejected!";
					break ; 
			}
			
			_ejectedThisRound = true;
		}

		/*
		 * Pick up a lost ejectee
		 */
		private function ejecteePickup():void {
			ejectee.pickUp();
			_ejecteeSound.stop();
			_ejecteeText.text = "";
		}
		
		/*
		 * Kill Eejectee
		 */
		public function ejecteeKill(char:uint):void {
			switch (char) { 
				case InternalTwoShotView.FRED:	
					_ejecteeText.text = "FRED has PERISHED";
					break ; 
				case InternalTwoShotView.VIKI:	
					_ejecteeText.text = "VIKI has PERISHED";
					break ; 
				case InternalTwoShotView.DROID:	
					_ejecteeText.text = "DROID has PERISHED";
					break ; 
			}
			FlxG.levels[char] = 3;
			PlayState(FlxG.state).itsv.killCharacter(char);
		}

		/*
		 * Recursively overlap a group, checking first to see if a group member has been
		 * reflected before doing the actual overlap processing
		 */
		private function overlapReflectedGroup(Target:FlxObject, grp:FlxGroup):void {
			var grpl:int = grp.members.length;
			var p:Proj;
			
			for (var i:int = 0;i < grpl;i++) {
				if(grp.members[i] is FlxGroup) {
					overlapReflectedGroup(Target, FlxGroup(grp.members[i]));
				} else {
					p = Proj(grp.members[i]);
					if(p.reflected) {
						FlxU.overlap(p, enemy, reflectedBulletEnemyOverlap);
					}
				}
			}
		}
		
		private function bulletEnemyOverlap(Obj1:FlxObject, Obj2:FlxObject):void {
			var impactGraphic:FlxObject;
			
			if(Obj1 is Proj) {
				if(!Proj(Obj1).reflected) {
					// Get Impact Graphic
					if(BossComp(Obj2).invincible)
						impactGraphic = enemy.thudImagePool.getNextAvail();
					else
						impactGraphic = enemy.splatImagePool.getNextAvail();
					// Reset Impact Graphic to Location of Impact
					impactGraphic.reset(Obj1.x + Obj1.origin.x - impactGraphic.origin.x, Obj1.y + Obj1.origin.y - impactGraphic.origin.y);
					// Kill Projectile and deal damage				
					Obj1.kill();
					Obj2.hurt(ship.damage);
				}
			} else {
				if(!Proj(Obj2).reflected) {
					// Get Impact Graphic
					if(BossComp(Obj1).invincible)
						impactGraphic = enemy.thudImagePool.getNextAvail();
					else
						impactGraphic = enemy.splatImagePool.getNextAvail();
					// Reset Impact Graphic to Location of Impact
					impactGraphic.reset(Obj2.x + Obj2.origin.x - impactGraphic.origin.x, Obj2.y + Obj2.origin.y - impactGraphic.origin.y);
					// Kill Projectile and deal damage
					Obj2.kill();
					Obj1.hurt(ship.damage);
				}
			}
		}

		private function reflectedBulletEnemyOverlap(Obj1:FlxObject, Obj2:FlxObject):void {
			var impactGraphic:FlxObject;
			
			if(Obj1 is Proj) {
				// Get Impact Graphic
				if(BossComp(Obj2).invincible)
					impactGraphic = enemy.thudImagePool.getNextAvail();
				else
					impactGraphic = enemy.splatImagePool.getNextAvail();
				// Reset Impact Graphic to Location of Impact
				impactGraphic.reset(Obj1.x + Obj1.origin.x - impactGraphic.origin.x, Obj1.y + Obj1.origin.y - impactGraphic.origin.y);
				// Kill Projectile and deal damage				
				Obj1.kill();
				Obj2.hurt(Proj(Obj1).reflectedDamage);
			} else {
				// Get Impact Graphic
				if(BossComp(Obj1).invincible)
					impactGraphic = enemy.thudImagePool.getNextAvail();
				else
					impactGraphic = enemy.splatImagePool.getNextAvail();
				// Reset Impact Graphic to Location of Impact
				impactGraphic.reset(Obj2.x + Obj2.origin.x - impactGraphic.origin.x, Obj2.y + Obj2.origin.y - impactGraphic.origin.y);
				// Kill Projectile and deal damage
				Obj2.kill();
				Obj1.hurt(Proj(Obj2).reflectedDamage);
			}
		}

		private function enemyShipOverlap(Obj1:FlxObject, Obj2:FlxObject):void {
			if(Obj1 is PlayerShip) {
				if(Obj2 is Proj) {
					if(!Proj(Obj2).reflected) {
						if(!ship.invincible)
							shipBang.reset(Obj2.x + Obj2.origin.x - shipBang.origin.x, Obj2.y + Obj2.origin.y - shipBang.origin.y);
						Obj2.kill();
						} else {
						return;
					}
				}
				Obj1.hurt(1);
			} else {
				if(Obj1 is Proj) {
					if(!Proj(Obj1).reflected) {
						if(!ship.invincible)
							shipBang.reset(Obj1.x + Obj1.origin.x - shipBang.origin.x, Obj1.y + Obj1.origin.y - shipBang.origin.y);
						Obj1.kill();
					} else {
						return;
					}
				}
				Obj2.hurt(1);
			}
		}
		
		public static function enemyDelayedKill():void {
			// Only do this once, so we use ship.firing to tell if we've done this before
			if(ship.firing) {
				for each (var p : FlxObject in enemyProjectiles.members) {
					if(p.exists)
								p.exists = false;
				}
			}
			ship.stopFiring();
			ship.flicker(-1);
			ship.active = false;
		}

		/*
		 * Triggered by PlayState to indicate a view switch
		 */
		public function switchTo():void {
			FlxG.music.stop();
			startFollow();
			FlxG.mouse.load(ImgCursor);
			ship.active = false;
			PlayState.loadTutorial(2);
		}
	}
}