package {
	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class TwoShotCharacter extends FlxSprite {
		public var name:String;
		public var location:uint;
		public var sub_location:uint;
		
		// States
		private var _sliding:Boolean;
		private var _slideDelta:Number;
		private var _slideTime:Number;
		
		// Components
		private var _components:Array;
		private var _clen:uint;
		private var _c:FlxObject;
		
		// View character belongs to
		private var _itsv:InternalTwoShotView;
		
		public function TwoShotCharacter(Name:String, CharGraphic:Class, ITSV:InternalTwoShotView) {
			// Start off screen
			super(-1000, -1000, CharGraphic);
			moves = false;
			visible = false;
			location = 0;
			
			name = Name;
			_itsv = ITSV;
			
			_components = new Array();
			_clen = 0;
			
			// Init states
			_sliding = false;
			_slideDelta = 0;
			_slideTime = 0;
		}

		override public function update():void {
			super.update();
			
			if(_sliding) {
				// Progress sliding
				_slideTime -= FlxG.elapsed;
				x += _slideDelta * FlxG.elapsed;
				
				// End Sliding
				if(_slideTime <= 0) {
					_sliding = false;
				}
			}
			
			// Update Components
			for (var i : int = 0; i < _clen; i++) {
				_c = TwoShotCharacterComponent(_components[i]);
				_c.x = x;
				_c.y = y;
				_c.update();
			}
		}
		
		public function addComponent(comp:TwoShotCharacterComponent):void {
			_components.push(comp);
			_clen++;
		}

		/*
		 * Start Sliding
		 */
		public function slide(distance:Number, duration:Number):void {
			_sliding = true;
			_slideDelta = distance / duration;
			_slideTime = duration;
		}

		override public function render():void {
			super.render();
			
			// Render components
			for (var i : int = 0; i < _clen; i++) {
				_c = TwoShotCharacterComponent(_components[i]);
				_c.x = x;
				_c.y = y;
				_c.render();
			}
		}
	}
}
