package {
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	import flash.geom.Point;
	import flash.geom.Rectangle;

	/**
	 * @author Yori Kvitchko
	 */
	public class TwoShotChatBubble extends FlxSprite {
		private static var FADE_TIME:Number = 1.5;
		

		[Embed(source="../data/characters/chat_bubble_top.png")] 
		private var ImgChatBubbleTop:Class;

		[Embed(source="../data/characters/chat_bubble_bot.png")] 
		private var ImgChatBubbleBot:Class;

		[Embed(source="../data/characters/chat_bubble_fill.png")] 
		private var ImgChatBubbleFill:Class;

		// Fade
		private var _fading:Boolean;
		private var _fadeInterval:Interval;

		// Text
		private var _text:FlxText;

		// Use myself for top
		private var _top:FlxSprite;
		private var _bot:FlxSprite;
		private var _fillGraphic:FlxSprite;

		// Temporary stuff for filling the chat bubble
		private var _fillRect:Rectangle;
		private var _fillPoint:Point;

		public function TwoShotChatBubble(X:Number = 0, Y:Number = 0) {
			super(X, Y);
			createGraphic(280, 400, 0xffffffff, true);
			
			// Init
			width = 280;
			
			_top = new FlxSprite(X, Y);
			_top.loadGraphic(ImgChatBubbleTop, false, true, 280, 20);
			
			_bot = new FlxSprite(X, Y);
			_bot.loadGraphic(ImgChatBubbleBot, false, true, 280, 20);
			
			_fillGraphic = new FlxSprite(-500, -500);
			_fillGraphic.loadGraphic(ImgChatBubbleFill, false, true);
			
			_fillRect = new Rectangle(0, 0, _fillGraphic.width, _fillGraphic.height);
			_fillPoint = new Point(0, 0);
			
			_text = new FlxText(X, Y, 240, "");
			_text.alignment = "left";
			_text.color = 0xff000000;
			_text.size = 13;
			setText("", FlxSprite.LEFT);
			
			// Fade
			_fading = false;
			_fadeInterval = new Interval([FADE_TIME]);
		}

		override public function update():void {
			super.update();
			
			// Fade
			if(_fading) {
				_fadeInterval.elapse(FlxG.elapsed);
				
				alpha = (_fadeInterval.percentTicked < 0.5) ? 1 : (1 - (_fadeInterval.percentTicked - 0.5)*2);
				
				if(_fadeInterval.justTicked) {
					stopFade();
				}
			}
		}

		public function startFade():void {
			_fading = true;
			_fadeInterval.reset();
		}

		public function stopFade():void {
			_fading = false;
			visible = false;
			alpha = 1.0;
		}

		public function setText(t:String, f:uint):void {
			var i:int;
			
			// Facing
			facing = _bot.facing = _top.facing = _fillGraphic.facing = f;
			
			// Calculate Format
			var beg:int;
			var end:int;
			
			beg = t.indexOf("@");
			t = t.replace(/@/, "");
				
			end = t.indexOf("@");
			t = t.replace(/@/, "");
			
			// Text
			_text.text = t;
			height = _text.height + 24;
			
			// Clear format
			_text.setFormat(_text.font, _text.size, _text.color, _text.alignment, _text.shadow);
			
			// Apply format
			if(beg != -1 && end != -1) {
				_text.setFormat(_text.font, _text.size, 0xffff0000, _text.alignment, _text.shadow, beg, end);
			}
			
			// Clear the fill
			_fillRect.height = 400;
			pixels.fillRect(_fillRect, 0x00000000);
			
			// Fill it with the proper graphics
			draw(_top, 0, 0);
			for (i = 0;i < height - _bot.height * 2;i++) {
				draw(_fillGraphic, 0, 20 + i);
			}
			draw(_bot, 0, height - _bot.height);
			draw(_text, (facing == LEFT ? 32 : 12), 12);
		}
	}
}
