package {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class TwoShotConversation extends FlxGroup {
		private static var BUBBLE_SPACING:Number = 10;
		private static var SCROLL_SPEED:Number = 150;

		private var _skip:Boolean;
		private var _bubbles:Array;
		private var _currentIndex:uint;
		private var _numBubbles:uint;
		private var _lastBubble:TwoShotChatBubble;
		private var _currentBubble:TwoShotChatBubble;

		private var _yOrig:Number;
		private var _scrolling:Boolean;
		private var _scrollingYTarget:Number;

		private var _textQueue:Array;
		private var _facingQueue:Array;

		public function TwoShotConversation(X:Number, Y:Number) {
			super();
			
			x = X;
			y = _yOrig = Y;
			_skip = true;
			
			_scrolling = false;
			_scrollingYTarget = 0;
			
			_currentIndex = 0;
			_numBubbles = 20;
			_bubbles = new Array(_numBubbles);
			_textQueue = new Array();
			_facingQueue = new Array();
			
			for (var i:int = 0;i < _numBubbles;i++) {
				_currentBubble = new TwoShotChatBubble();
				_currentBubble.visible = false;
				_bubbles[i] = TwoShotChatBubble(add(_currentBubble));
			}
			clear();
		}

		override public function update():void {
			if(_scrolling) {
				reset(x, y - SCROLL_SPEED * FlxG.elapsed);
				
				if(y <= _scrollingYTarget) {
					reset(x, _scrollingYTarget);
					_scrolling = false;
					
					_currentBubble.x = x;
					_currentBubble.y = y;
					_currentBubble.visible = true;
					
					_lastBubble.startFade();
					
					y = _yOrig;
				}
			} else if(_textQueue.length > 0) {
				_lastBubble = TwoShotChatBubble(_currentBubble);
				_currentBubble = TwoShotChatBubble(_bubbles[_currentIndex]);
				_currentBubble.setText(_textQueue.shift(), _facingQueue.shift());

				_scrolling = true;
				_scrollingYTarget = y - _currentBubble.height - BUBBLE_SPACING;

				if(!_lastBubble.visible)
					reset(x, _scrollingYTarget);
				
				_currentIndex++;
				if(_currentIndex >= _numBubbles)
					_currentIndex = 0;
			}
			
			super.update();
		}
		
		public function clear():void {
			for(i = 0; i < _textQueue.length; i++) {
				_textQueue.shift();
				_facingQueue.shift();
			}
			for (var i:int = 0;i < _numBubbles; i++) {
				TwoShotChatBubble(_bubbles[i]).stopFade();
			}
			_currentIndex = 1;
			_currentBubble = TwoShotChatBubble(_bubbles[0]);
			_currentBubble.height = 0;
			y = _yOrig;
			_scrolling = false;
		}

		public function pushText(text:String, f:uint):void {
			_textQueue.push(text);
			_facingQueue.push(f);
		}
		
		public function queueLength():int {
			return _textQueue.length;
		}
	}
}
