package base {
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class Proj extends FlxSprite {
		/*
		 * Whether or not this projectile is in a normal or reflected state
		 */
		public var reflected:Boolean;
		/*
		 * Damage done after being reflected
		 */
		public var reflectedDamage:Number;

		public function Proj() {
			super(-100, -100);
			reflected = false;
			exists = false;
			dead = true;
			reflectedDamage = 5;
			initGraphics();
		}

		
		override public function update():void {
			super.update();
			removeOffScreen();
		}

		protected function removeOffScreen():void {
			if(x > (SpaceView.LEVEL_WIDTH + width) || y > (SpaceView.LEVEL_HEIGHT + height) || x < -width || y < -height)
				exists = false;
		}

		/*
		 * Override this function to implement initializing a graphic
		 * for the bullet
		 */
		public function initGraphics():void {
			// No default graphics
		}

		/*
		 * Override this function to implement custom send code
		 */
		public function send(X:Number, Y:Number, locationIndex:int):void {
			// Just reset the bullet to the given position
			this.reset(X - origin.x, Y - origin.y);
			reflected = false;
		}

		/*
		 * Function for telling a bullet to reflect itself
		 * if it is reflectable or kill it
		 */
		public function reflect():void {
			reflected = true;
			velocity.x = -velocity.x;
			velocity.y = -velocity.y;
			angle += 180;
		}
	}
}
