package bosses {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class Boss extends FlxGroup {
		private static const DEATH_TIME:Number = 4;
		
		public var projectiles:FlxGroup;
		public var percentHealth:Number;
		private var maxHealth:Number;
		
		protected var _delayedKill:Boolean;
		private var _delayedKillTimer:Number;
		
		/*
		 * Object Pool of Sprites representing splat graphics
		 * Splay graphics display when a bullet hits a vulnerable boss component
		 */
		public var splatImagePool:ObjectPool;
		/*
		 * Object Pool of Sprites representing thud graphics
		 * Splay graphics display when a bullet hits an invulnerable boss component
		 */
		public var thudImagePool:ObjectPool;

		/*
		 * FlxGroup representing a single boss or many components all of which
		 * together compromise the boss 
		 */
		public function Boss(Projectiles:FlxGroup, X:Number, Y:Number, Width:Number, Height:Number, Health:Number) {
			super();
			moves = false;
			health = maxHealth = Health;
			percentHealth = 1.0;
			
			projectiles = Projectiles;
			x = X;
			y = Y;
			width = Width;
			height = Height;
			
			_delayedKill = false;
			_delayedKillTimer = DEATH_TIME;
			
			splatImagePool = new ObjectPool(SplatImpact, 20);
			thudImagePool = new ObjectPool(ThudImpact, 20);
			
			// Initialize origin
			origin.x = Width / 2;
			origin.y = Height / 2;
		}

		override public function hurt(Damage:Number):void {
			super.hurt(Damage);
			percentHealth = health / maxHealth;
			
			if(!SpaceView.enemyExplosions.exploding && !_delayedKill) {
				SpaceView.enemyExplosions.reset(x, y);
				
				if(percentHealth < 0.3) {
					SpaceView.enemyExplosions.explodeSubTime = 0.2;
					SpaceView.enemyExplosions.explode(x, y, width, height, 1);
				} else if(percentHealth < 0.6) {
					SpaceView.enemyExplosions.explodeSubTime = 0.3;
					SpaceView.enemyExplosions.explode(x, y, width, height, 1);
				} else if(percentHealth < 0.9) {
					SpaceView.enemyExplosions.explodeSubTime = 0.4;
					SpaceView.enemyExplosions.explode(x, y, width, height, 1);
				}
			}
		}
		
		override public function update():void {
			if(_delayedKill) {
				if(_delayedKillTimer > 0) {
					_delayedKillTimer -= FlxG.elapsed;
				} else {
					kill();
				}
			} else {
				super.update();
			}
		}

		override public function kill():void {
			if(_delayedKillTimer <= 0) {
				super.kill();
				_delayedKill = false;
			} else {
				SpaceView.enemyDelayedKill();
				SpaceView.explosions.explode(x, y, width, height, DEATH_TIME);
				_delayedKill = true;
			}
		}
	}
}
