package bosses {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class BossComp extends FlxSprite {
		/*
		 * FlxObject representing the parent of this component
		 * Null means the component is self-reliant
		 * Is often a FlxGroup
		 */
		protected var _parent:FlxObject;

		protected var _invincible:Boolean;

		public var projectiles:FlxGroup;

		/*
		 * Hurt sound
		 */
		[Embed(source="../../data/sounds/thud_1.mp3")] 
		private var SndThud1:Class;
		[Embed(source="../../data/sounds/thud_2.mp3")] 
		private var SndThud2:Class;
		[Embed(source="../../data/sounds/thud_3.mp3")] 
		private var SndThud3:Class;
		
		[Embed(source="../../data/sounds/squelch_1.mp3")] 
		private var SndHurt1:Class;
		[Embed(source="../../data/sounds/squelch_2.mp3")] 
		private var SndHurt2:Class;
		[Embed(source="../../data/sounds/squelch_3.mp3")] 
		private var SndHurt3:Class;
		[Embed(source="../../data/sounds/squelch_4.mp3")] 
		private var SndHurt4:Class;

		/*
		 * A component of a larger boss, can be either independant or dependant on a parent
		 */
		public function BossComp(Projectiles:FlxGroup = null, X:Number = 0, Y:Number = 0, Parent:FlxObject = null) {
			super(X, Y);
			projectiles = Projectiles;
			_parent = Parent;
			_invincible = false;
			initGraphics();
		}

		/*
		 * Override this function to implement initializing a graphic
		 * for the boss component otherwise it will be invisible
		 * SHOULD NOT BE CALLED BY THE OVERRIDING CLASS (ex: super.initGraphics())
		 */
		public function initGraphics():void {
			visible = false;
		}

		/*
		 * If a parent object has been specified then hurt the parent otherwise hurt itself
		 */
		override public function hurt(Damage:Number):void {
			var i:int;
			
			if(!_invincible) {
				if(_parent) {
					FlxObject(_parent).hurt(Damage);
				} else {
					super.hurt(Damage);
				}
				i = Math.floor(Math.random()*4);
				if(i == 0)
					FlxG.play(SndHurt1);
				else if(i == 1)
					FlxG.play(SndHurt2);
				else if(i == 2)
					FlxG.play(SndHurt3);
				else if(i == 4)
					FlxG.play(SndHurt4);
			} else {
				i = Math.floor(Math.random()*3);
				if(i == 0)
					FlxG.play(SndThud1);
				else if(i == 1)
					FlxG.play(SndThud2);
				else if(i == 2)
					FlxG.play(SndThud3);
			}
		}

		public function get invincible():Boolean {
			return _invincible;
		}
	}
}
