package bosses.doomstroyer {
	import bosses.Boss;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class DoomBoss extends Boss {
		private static const SPAWNER_1_ON_INTERVAL:Number = 0.3;
		private static const SPAWNER_1_OFF_INTERVAL:Number = 0.6;
		private static const SPAWNER_2_ON_INTERVAL:Number = 1;
		private static const SPAWNER_2_OFF_INTERVAL:Number = 1;
		private static const SPAWNER_2_ANGLE_DELTA:Number = 20;
		private static const SWITCH_INTERVAL:Number = 5;

		private var _time:Number;
		private var _spawner_1:DoomCircleSpawner;
		private var _spawner_2:DoomCircleSpawner;
		private var _spawner_2_SwapInterval:Interval;
		private var _spawner_1_TriggerInterval:Interval;
		
		// Used to switch between spawners 1 and 2
		private var _switchInterval:Interval;

		private var _currentSpawner1:Boolean;

		public function DoomBoss(Projectiles:FlxGroup) {
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2 - 250, 30, 500, 350, 800);
			
			_time = 0;
			// Spawner 1 Swap
			_spawner_1_TriggerInterval = new Interval([SPAWNER_1_ON_INTERVAL, SPAWNER_1_OFF_INTERVAL]);;
			// Spawner 2 Rotate
			_spawner_2_SwapInterval = new Interval([SPAWNER_2_OFF_INTERVAL, SPAWNER_2_ON_INTERVAL, SPAWNER_2_OFF_INTERVAL, SPAWNER_2_ON_INTERVAL, SPAWNER_2_OFF_INTERVAL]);
			
			// Switch
			_switchInterval = new Interval([SWITCH_INTERVAL]);
			_currentSpawner1 = true;
			
			add(new DoomBossComp(Projectiles, 90, 70, this));
			add(new DoomBossTurretComp(Projectiles, 190, 240, this));
			
			// Spawner 1
			_spawner_1 = new DoomCircleSpawner(0, 0, 0.05, 90, this);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.addLocation(250, 300);
			_spawner_1.stop();
			Projectiles.add(_spawner_1.projectilePool);
			add(_spawner_1);
			
			// Spawner 2
			_spawner_2 = new DoomCircleSpawner(0, 0, 0.1, 90, this);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.stop();
			Projectiles.add(_spawner_2.projectilePool);
			add(_spawner_2);
		}

		override public function update():void {
			if(!_delayedKill) {
				_time += FlxG.elapsed / 2;
				_switchInterval.elapse(FlxG.elapsed);
			
				if(_switchInterval.justTicked) {
					// Switch Spawners
					_currentSpawner1 = !_currentSpawner1;
				
					// Stop/Start as appropriate
					if(_currentSpawner1) {
						_spawner_1.projAngle = _spawner_2.projAngle;
						_spawner_1_TriggerInterval.reset();
					} else {
						_spawner_2.projAngle = _spawner_1.projAngle;
						_spawner_2_SwapInterval.reset();
					}
					
					_spawner_1.stop();
					_spawner_2.stop();
				}
			
				if(_currentSpawner1) {
					// Spawner 1 Logic
					_spawner_1_TriggerInterval.elapse(FlxG.elapsed);
					
					if(_spawner_1_TriggerInterval.justTicked) {
						if(_spawner_1_TriggerInterval.currentInterval == 0) {
							_spawner_1.projAngle += 22.5;
							_spawner_1.start();
						} else {
							_spawner_1.stop();
						}
					}
				} else {
					// Spawner 2 Logic
					_spawner_2_SwapInterval.elapse(FlxG.elapsed);
				
					if(_spawner_2_SwapInterval.currentInterval < 2)
					_spawner_2.projAngle += SPAWNER_2_ANGLE_DELTA * FlxG.elapsed;
				else
					_spawner_2.projAngle -= SPAWNER_2_ANGLE_DELTA * FlxG.elapsed;
				
					if(_spawner_2_SwapInterval.justTicked) {
						if(_spawner_2_SwapInterval.currentInterval % 2 == 0)
						_spawner_2.stop();
					else
						_spawner_2.start();
					}
				}
			
				reset(SpaceView.LEVEL_WIDTH / 2 - 250 + Math.sin(_time) * 150, 30);
			}
			
			super.update();
		}
	}
}
