package bosses.eye
{
	import bosses.Boss;
	
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	
	public class EyeBoss extends Boss
	{
		// Constants
		private static const BULLET_INTERVAL:Number = 0.01;
		private static const ANGULAR_SPEED:Number = 50;
		private static const CHASE_ANGULAR_SPEED:Number = 60;
		private static const EYEBALL_MOVE_RADIUS:Number = 200;
		private static const IRIS_ALPHA:Number = 1;
		private static const RADIUS_DECR:Number = 100;
		private static const RADIUS_INCR:Number = 100;
		private static const HEAL_RATE:Number = 50;
		public static const EYE_RADIUS:Number = 250;
		
		// Variables
		private var _deltaRotateAngle:Number;
		private var _deltaChaseAngle:Number;
		private var _chaseDir:int;
		private var _time:Number;
		private var _blinkInterval:Interval;
		private var _rotateInterval:Interval;
		/**
		 *  two phases, 1 and 2
		 **/
		private var _phase:int;
		private var _deltaRadius:Number;
		private var _deltaAlpha:Number;
		
		// Boss Pupil Component
		public var pupilComp:EyeBossPupilComp;
		// Dummy component for collision detection with the iris
		private var _dummyComp:EyeBossDummyComp;
		// Iris Background Component
		private var _irisBGComp:EyeBossIrisBGComp;
		// Laser Spawner
		private var _chaserSpawner:EyeBossLaserSpawner;
		private var _fixedSpawner1:EyeBossLaserSpawner;
		private var _fixedSpawner2:EyeBossLaserSpawner;
		private var _fixedSpawner3:EyeBossLaserSpawner;
		private var _fixedSpawner4:EyeBossLaserSpawner;
		private var _fixedSpawner5:EyeBossLaserSpawner;
		private var _fixedSpawner6:EyeBossLaserSpawner;
		private var _rotatingSpawner1:EyeBossLaserSpawner;
		private var _rotatingSpawner2:EyeBossLaserSpawner;
		private var _rotatingSpawner3:EyeBossLaserSpawner;
		
		
		public function EyeBoss(Projectiles:FlxGroup)
		{
			super(Projectiles, 
			200, 
			200,
			400,
			400,
			650);
			
			_deltaRadius = 0;
			_deltaAlpha = 0;
			_time = 0;
			_phase = 1;
			_blinkInterval = new Interval([5,2.5]);
			_rotateInterval = new Interval([5]);
			_deltaChaseAngle = 0;
			_chaseDir = 1;
			_deltaRotateAngle = 0;
			
			// Main Collision Components
			var eb_comp:EyeBossComp;
			var x_loc:Number;
			var y_loc:Number;
			var numComps:int;
			numComps = 360;
			for(var i:int = 0; i < numComps; i++)
			{
				x_loc = origin.x+EYE_RADIUS*Math.cos(2*Math.PI*i/numComps);
				y_loc = origin.y+EYE_RADIUS*Math.sin(2*Math.PI*i/numComps);
				eb_comp = new EyeBossComp(Projectiles, x_loc, y_loc, this);
				add(eb_comp);
			}
			
			// Iris BG Component
			var tmpoffset:Number = origin.x - EYE_RADIUS*1.8;
			_irisBGComp = new EyeBossIrisBGComp(Projectiles, tmpoffset, tmpoffset);
			//add(_irisBGComp);
			
			// Dummy Component for collision
			tmpoffset = origin.x-EYE_RADIUS+10;
			_dummyComp = new EyeBossDummyComp(Projectiles, tmpoffset, tmpoffset);
			add(_dummyComp);
			
			// Pupil Component
			pupilComp = new EyeBossPupilComp(Projectiles, origin.x-25, origin.y-25, this);
			add(pupilComp);
			
			
			// Add lasers
			_chaserSpawner = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_chaserSpawner.addLocation(0, 0);
			_chaserSpawner.cycle = false;
			_chaserSpawner.start();
			Projectiles.add(_chaserSpawner.projectilePool);
			add(_chaserSpawner);
			
			_fixedSpawner1 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner1.addLocation(0, 0);
			_fixedSpawner1.cycle = false;
			_fixedSpawner1.stop();
			Projectiles.add(_fixedSpawner1.projectilePool);
			add(_fixedSpawner1);
			
			_fixedSpawner2 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner2.addLocation(0, 0);
			_fixedSpawner2.cycle = false;
			_fixedSpawner2.stop();
			Projectiles.add(_fixedSpawner2.projectilePool);
			add(_fixedSpawner2);
			
			_fixedSpawner3 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner3.addLocation(0, 0);
			_fixedSpawner3.cycle = false;
			_fixedSpawner3.stop();
			Projectiles.add(_fixedSpawner3.projectilePool);
			add(_fixedSpawner3);
			
			_fixedSpawner4 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner4.addLocation(0, 0);
			_fixedSpawner4.cycle = false;
			_fixedSpawner4.stop();
			Projectiles.add(_fixedSpawner4.projectilePool);
			add(_fixedSpawner4);
			
			_fixedSpawner5 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner5.addLocation(0, 0);
			_fixedSpawner5.cycle = false;
			_fixedSpawner5.stop();
			Projectiles.add(_fixedSpawner5.projectilePool);
			add(_fixedSpawner5);
			
			_fixedSpawner6 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_fixedSpawner6.addLocation(0, 0);
			_fixedSpawner6.cycle = false;
			_fixedSpawner6.stop();
			Projectiles.add(_fixedSpawner6.projectilePool);
			add(_fixedSpawner6);
			
			_rotatingSpawner1 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_rotatingSpawner1.addLocation(0, 0);
			_rotatingSpawner1.cycle = false;
			_rotatingSpawner1.stop();
			Projectiles.add(_rotatingSpawner1.projectilePool);
			add(_rotatingSpawner1);
			
			_rotatingSpawner2 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_rotatingSpawner2.addLocation(0, 0);
			_rotatingSpawner2.cycle = false;
			_rotatingSpawner2.stop();
			Projectiles.add(_rotatingSpawner2.projectilePool);
			add(_rotatingSpawner2);
			
			_rotatingSpawner3 = new EyeBossLaserSpawner(200, 200, BULLET_INTERVAL, 0);
			_rotatingSpawner3.addLocation(0, 0);
			_rotatingSpawner3.cycle = false;
			_rotatingSpawner3.stop();
			Projectiles.add(_rotatingSpawner3.projectilePool);
			add(_rotatingSpawner3);
		}
		
		override public function update():void
		{
			super.update();
			if(!_delayedKill) 
			{ 
				_time += FlxG.elapsed;
				
				if(_phase == 1)
				{
					_blinkInterval.elapse(FlxG.elapsed);
//					var tmpX:Number;
//					var tmpY:Number;
//					var destAngle:Number;
//					var currentAngle:Number;
//					var deltaAngle:Number; 
					var radiansAngle:Number;
					
					_deltaChaseAngle += FlxG.elapsed*CHASE_ANGULAR_SPEED*_chaseDir;
					_chaserSpawner.setAngle(_deltaChaseAngle); 
					radiansAngle = Math.PI*_deltaChaseAngle/180;
					//var radiansAngle = Math.PI*(currentAngle+_deltaChaseAngle)/180;
					pupilComp.reset(375 + Math.cos(radiansAngle)*EYEBALL_MOVE_RADIUS,
									 375 + Math.sin(radiansAngle)*EYEBALL_MOVE_RADIUS); 
					_chaserSpawner.reset(pupilComp.x+25, pupilComp.y+25);
					
					if(_blinkInterval.justTicked)
					{
						/**
						 * shoots out 6 beams which partition off the circular area
						 * causing the player to have to stop moving and lets the
						 * beam catch up
						 **/
						if(_blinkInterval.currentInterval == 1)
						{
							startFixedLasers();
						}
						else if(_blinkInterval.currentInterval == 0)
						{
							stopFixedLasers();
							_chaserSpawner.stop();
							_deltaRadius = EYEBALL_MOVE_RADIUS;
							_deltaAlpha = IRIS_ALPHA;
							_phase = 2;
						} 
					}
				}	//~ phase 1
				// Move the eye ball back to the center
				else if(_phase == 2)
				{
					_deltaRadius -= FlxG.elapsed*RADIUS_DECR;
					_deltaAlpha = _deltaRadius/EYEBALL_MOVE_RADIUS*IRIS_ALPHA;
					if(_deltaRadius > 0)
					{
						radiansAngle = Math.PI*_deltaChaseAngle/180;
						pupilComp.reset(375 + Math.cos(radiansAngle)*_deltaRadius,
										 375 + Math.sin(radiansAngle)*_deltaRadius);
						_dummyComp.alpha = _deltaAlpha;
					}	
					else
					{
						pupilComp.reset(375, 375);
						_dummyComp.irisOpenning = true;
						startRotatingLasers();
						_phase = 3;
					}
				}	//~ phase2
				else if(_phase == 3)
				{
					_rotateInterval.elapse(FlxG.elapsed);
					/**
					 *  Boss heals automatically
					 **/
					health += HEAL_RATE*FlxG.elapsed;
					if(health >= 650)
						health = 650;
					
					/**
					 *  Three rotating laser beams
					 **/
					_deltaRotateAngle += FlxG.elapsed*ANGULAR_SPEED*(-1)*_chaseDir;
					
					_rotatingSpawner1.setAngle(_deltaRotateAngle);
					_rotatingSpawner2.setAngle(_deltaRotateAngle+120);
					_rotatingSpawner3.setAngle(_deltaRotateAngle+240);
					if(_rotateInterval.justTicked)
					{
						_deltaRadius = 0;
						_deltaAlpha = 0;
						stopRotatingLasers();
						_chaseDir = -_chaseDir;
						_phase = 4;
					}
				} // ~phase3
				// Move the eye ball to the border
				else if(_phase == 4)
				{
					_deltaRadius += FlxG.elapsed*RADIUS_INCR;
					_deltaAlpha = _deltaRadius/EYEBALL_MOVE_RADIUS*IRIS_ALPHA;
					if(_deltaRadius < EYEBALL_MOVE_RADIUS)
					{
						radiansAngle = Math.PI*_deltaChaseAngle/180;
						pupilComp.reset(375 + Math.cos(radiansAngle)*_deltaRadius,
										 375 + Math.sin(radiansAngle)*_deltaRadius);
						_dummyComp.alpha = _deltaAlpha;
					}	
					else
					{
						_dummyComp.irisOpenning = false;
						_chaserSpawner.start();
						_chaseDir = -_chaseDir;
						_phase = 1;
					}	//~ phase4
				}
				// The iris bg component should follow the pupil
				_irisBGComp.reset(pupilComp.x+pupilComp.origin.x-_irisBGComp.width/2,
								pupilComp.y+pupilComp.origin.y-_irisBGComp.height/2); 
				//_dummyComp.draw(_irisBGComp, -100, -100);
				
			}
			
		}
		
		override public function render():void
		{
			_dummyComp.draw(_irisBGComp, _irisBGComp.x-_dummyComp.x, _irisBGComp.y-_dummyComp.y);
			super.render();
		}
		
		private function startFixedLasers():void
		{
			var currentAngle:Number = _chaserSpawner.getAngle();
			_fixedSpawner1.setAngle(currentAngle);
			_fixedSpawner2.setAngle(currentAngle + 60);
			_fixedSpawner3.setAngle(currentAngle + 120);
			_fixedSpawner4.setAngle(currentAngle + 180);
			_fixedSpawner5.setAngle(currentAngle + 240);
			_fixedSpawner6.setAngle(currentAngle + 300);
			
			_fixedSpawner1.start();
			_fixedSpawner2.start();
			_fixedSpawner3.start();
			_fixedSpawner4.start();
			_fixedSpawner5.start();
			_fixedSpawner6.start();
		}
		
		private function stopFixedLasers():void
		{
			_fixedSpawner1.stop();
			_fixedSpawner2.stop();
			_fixedSpawner3.stop();
			_fixedSpawner4.stop();
			_fixedSpawner5.stop();
			_fixedSpawner6.stop();
		}
		
		private function startRotatingLasers():void
		{
			_rotatingSpawner1.start();
			_rotatingSpawner2.start();
			_rotatingSpawner3.start();
		}
		
		private function stopRotatingLasers():void
		{
			_rotatingSpawner1.stop();
			_rotatingSpawner2.stop();
			_rotatingSpawner3.stop();
		}
		
	}
}