package bosses.gates {
	import base.ProjSpawner;

	import bosses.Boss;
	import bosses.BossComp;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	public class GatesBoss extends Boss {
		// Constants
		private static const BULLET_NUMBER:int = 8;
		private static const GATES_X_MOVEMENT_FAST:Number = 80;
		private static const GATES_Y_MOVEMENT_FAST:Number = 80;
		private static const GATES_X_MOVEMENT_SLOW:Number = 20;
		private static const GATES_Y_MOVEMENT_SLOW:Number = 20;
		private static const BOSS_X_MOVEMENT_SPEED:Number = 70;
		private static const BOSS_Y_MOVEMENT_SPEED:Number = 70;
		private static const EXPLODE_TIME:Number = 4;

		// Variables
		private var _time:Number;
		private var _xDelta:Number;
		private var _yDelta:Number;
		private var _brSpawner1:GatesSpawner;
		private var _brSpawner2:GatesSpawner;
		private var _blSpawner1:GatesSpawner;
		private var _blSpawner2:GatesSpawner;
		private var _trSpawner1:GatesSpawner;
		private var _trSpawner2:GatesSpawner;
		private var _tlSpawner1:GatesSpawner;
		private var _tlSpawner2:GatesSpawner;

		private var _laserSpawner1:GatesLaserSpawnerV;
		private var _laserSpawner2:GatesLaserSpawnerV;
		private var _laserSpawner3:GatesLaserSpawnerH;
		private var _laserSpawner4:GatesLaserSpawnerH;

		private var _blocker1:GatesBossBlockerComp;
		private var _blocker2:GatesBossBlockerComp;
		private var _blocker3:GatesBossBlockerComp;
		private var _blocker4:GatesBossBlockerComp;
		private var _blocker5:GatesBossBlockerComp;
		private var _blocker6:GatesBossBlockerComp;
		private var _blocker7:GatesBossBlockerComp;
		private var _blocker8:GatesBossBlockerComp;

		private var _weapon:GatesWeaponComp;
		private var _heart:GatesBossCoreComp;
		private var _phaseInterval:Interval;
		public var _laserInterval:Interval;
		public var _laserShooting:Boolean;

		private var _bossMovementPhase:int;
		private var _blockerCount:int;
		private var _blockerOpenning1:Boolean;
		private var _blockerOpenning2:Boolean;
		private var _blockerOpenning3:Boolean;
		private var _blockerOpenning4:Boolean;
		public var _blockerAlive1:Boolean;
		public var _blockerAlive2:Boolean;
		public var _blockerAlive3:Boolean;
		public var _blockerAlive4:Boolean;
		public var _blockerExploding1:Boolean;
		public var _blockerExploding2:Boolean;
		public var _blockerExploding3:Boolean;
		public var _blockerExploding4:Boolean;

		[Embed(source="../../../data/gates_boss/core.png")]
		private var ImgGatesCore:Class;

		public function GatesBoss(Projectiles:FlxGroup) {
			super(Projectiles, 0, 0, 400, 400, 550);
			
			_time = 0;
			_phaseInterval = new Interval([2,2]);
			_laserInterval = new Interval([5]);
			_laserShooting = false;
			_blockerCount = 0;
			_bossMovementPhase = 0;
			_xDelta = 0;
			_yDelta = 0;
			_blockerOpenning1 = false;
			_blockerOpenning2 = false;
			_blockerOpenning3 = false;
			_blockerOpenning4 = false;
			_blockerAlive1 = true;
			_blockerAlive2 = true;
			_blockerAlive3 = true;
			_blockerAlive4 = true;
			
			_blockerExploding1 = false;
			_blockerExploding2 = false;
			_blockerExploding3 = false;
			_blockerExploding4 = false;			
			
			// Heart
			_heart = new GatesBossCoreComp(Projectiles, 150, 150, this);
			add(_heart);
			
			// Visual
			var bc:BossComp;
			bc = new BossComp(Projectiles, 0, 0, this);
			bc.loadGraphic(ImgGatesCore);
			bc.solid = false;
			bc.visible = true;
			add(bc);
			
			var gb_comp:GatesBossComp;
			
			gb_comp = new GatesBossComp(Projectiles, 25, 25, this, 0);
			gb_comp.visible = false;
			add(gb_comp);
			gb_comp = new GatesBossComp(Projectiles, 250, 25, this, 0);
			gb_comp.visible = false;
			add(gb_comp);
			gb_comp = new GatesBossComp(Projectiles, 25, 250, this, 0);
			gb_comp.visible = false;
			add(gb_comp);
			gb_comp = new GatesBossComp(Projectiles, 250, 250, this, 0);
			gb_comp.visible = false;
			add(gb_comp);
			
			// Weapon
			_laserSpawner1 = new GatesLaserSpawnerV(0);
			_laserSpawner1.addLocation(200, 225);
			_laserSpawner1.cycle = false;
			_laserSpawner1.stop();
			Projectiles.add(_laserSpawner1.projectilePool);
			add(_laserSpawner1);
			
			_laserSpawner2 = new GatesLaserSpawnerV(1);
			_laserSpawner2.addLocation(200, 175);
			_laserSpawner2.cycle = false;
			_laserSpawner2.stop();
			Projectiles.add(_laserSpawner2.projectilePool);
			add(_laserSpawner2);
			
			_laserSpawner3 = new GatesLaserSpawnerH(0);
			_laserSpawner3.addLocation(225, 200);
			_laserSpawner3.cycle = false;
			_laserSpawner3.stop();
			Projectiles.add(_laserSpawner3.projectilePool);
			add(_laserSpawner3);
			
			_laserSpawner4 = new GatesLaserSpawnerH(1);
			_laserSpawner4.addLocation(175, 200);
			_laserSpawner4.cycle = false;
			_laserSpawner4.stop();
			Projectiles.add(_laserSpawner4.projectilePool);
			add(_laserSpawner4);
			
			
			// Blockers
			_blocker1 = new GatesBossBlockerComp(Projectiles, 50, 270, this, 2);
			_blocker2 = new GatesBossBlockerComp(Projectiles, 270, 270, this, 3);
			add(_blocker1);
			add(_blocker2);
			
			_blocker3 = new GatesBossBlockerComp(Projectiles, 270, 50, this, 0);
			_blocker4 = new GatesBossBlockerComp(Projectiles, 270, 270, this, 1);
			add(_blocker3);
			add(_blocker4);
			
			_blocker5 = new GatesBossBlockerComp(Projectiles, 50, 50, this, 2);
			_blocker6 = new GatesBossBlockerComp(Projectiles, 270, 50, this, 3);
			add(_blocker5);
			add(_blocker6);
			
			_blocker7 = new GatesBossBlockerComp(Projectiles, 50, 50, this, 0);
			_blocker8 = new GatesBossBlockerComp(Projectiles, 50, 270, this, 1);
			add(_blocker7);
			add(_blocker8); 
			
			// Spawners
			var i:int;
			
			// bottom right
			_brSpawner1 = new GatesSpawner(BULLET_NUMBER, 0);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_brSpawner1.addLocation(250, 250);
			}
			_brSpawner1.cycle = false;
			_brSpawner1.stop();
			Projectiles.add(_brSpawner1.projectilePool);
			add(_brSpawner1);
			
			_brSpawner2 = new GatesSpawner(BULLET_NUMBER, 45);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_brSpawner2.addLocation(250, 250);
			}
			_brSpawner2.cycle = false;
			_brSpawner2.stop();
			Projectiles.add(_brSpawner2.projectilePool);
			add(_brSpawner2);
			
			// bottom left
			_blSpawner1 = new GatesSpawner(BULLET_NUMBER, 135);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_blSpawner1.addLocation(150, 250);
			}
			_blSpawner1.cycle = false;
			_blSpawner1.stop();
			Projectiles.add(_blSpawner1.projectilePool);
			add(_blSpawner1);
			
			_blSpawner2 = new GatesSpawner(BULLET_NUMBER, 90);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_blSpawner2.addLocation(150, 250);
			}
			_blSpawner2.cycle = false;
			_blSpawner2.stop();
			Projectiles.add(_blSpawner2.projectilePool);
			add(_blSpawner2);
			
			// top right
			_trSpawner1 = new GatesSpawner(BULLET_NUMBER, -45);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_trSpawner1.addLocation(250, 150);
			}
			_trSpawner1.cycle = false;
			_trSpawner1.stop();
			Projectiles.add(_trSpawner1.projectilePool);
			add(_trSpawner1);
			
			_trSpawner2 = new GatesSpawner(BULLET_NUMBER, -90);
			for(i = 0;i < BULLET_NUMBER;i++) { 
				_trSpawner2.addLocation(250, 150);
			}
			_trSpawner2.cycle = false;
			_trSpawner2.stop();
			Projectiles.add(_trSpawner2.projectilePool);
			add(_trSpawner2);
			
			// top left
			_tlSpawner1 = new GatesSpawner(BULLET_NUMBER, -180);
			for(i = 0;i < BULLET_NUMBER;i++) {
				_tlSpawner1.addLocation(150, 150);
			}
			_tlSpawner1.cycle = false;
			_tlSpawner1.stop();
			Projectiles.add(_tlSpawner1.projectilePool);
			add(_tlSpawner1);
			
			_tlSpawner2 = new GatesSpawner(BULLET_NUMBER, -135);
			for(i = 0;i < BULLET_NUMBER;i++) {
				_tlSpawner2.addLocation(150, 150);
			}
			_tlSpawner2.cycle = false;
			_tlSpawner2.stop();
			Projectiles.add(_tlSpawner2.projectilePool);
			add(_tlSpawner2);
		}

		override public function update():void {
			if(!_delayedKill) { 
				_time += FlxG.elapsed;
				_phaseInterval.elapse(FlxG.elapsed);
				
				// Boss movement
				if(x + width > 800) {
					_bossMovementPhase = 1;
					_xDelta = 0;
				}
				if(y + height > 800) {
					_bossMovementPhase = 2;
					_yDelta = 0;
				}
				if(x < 0) {
					_bossMovementPhase = 3;
					_xDelta = 0;
				}
				if(y < 0) {
					_bossMovementPhase = 0;
					_yDelta = 0;
				}
				
				if(_bossMovementPhase == 0) {
					_xDelta += BOSS_X_MOVEMENT_SPEED * FlxG.elapsed;
					reset(_xDelta, 0);
				}
				else if(_bossMovementPhase == 1) {
					_yDelta += BOSS_X_MOVEMENT_SPEED * FlxG.elapsed;
					reset(800 - width, _yDelta);
				}
				else if(_bossMovementPhase == 2) {
					_xDelta += BOSS_X_MOVEMENT_SPEED * FlxG.elapsed;
					reset(400 - _xDelta, 800 - height);
				}
				else if(_bossMovementPhase == 3) {
					_yDelta += BOSS_X_MOVEMENT_SPEED * FlxG.elapsed;
					reset(0, 400 - _yDelta);
				} 
				
		
				if(_phaseInterval.justTicked) {
					if(_phaseInterval.currentInterval == 1) {
						triggerProj1(x, y);
					}
					else if(_phaseInterval.currentInterval == 0) {
						triggerProj2(x, y);
					}
				}
				
				if(!_laserShooting) {
					_laserInterval.elapse(FlxG.elapsed);
					if(_laserInterval.justTicked) {
						triggerLaser();
						_laserShooting = true;
					}
				}
				
				_blockerOpenning1 = SpaceView.ship.y > _blocker1.y ? (false || !_blockerAlive1) : true;
				_blockerOpenning2 = SpaceView.ship.x > _blocker3.x ? (false || !_blockerAlive2) : true;
				_blockerOpenning3 = SpaceView.ship.y < _blocker5.y ? (false || !_blockerAlive3) : true;
				_blockerOpenning4 = SpaceView.ship.x < _blocker7.x ? (false || !_blockerAlive4) : true;
				
				blockerMovement();
			}
			super.update();
		}

		public function triggerProj1(X:Number, Y:Number):void {
			ProjSpawner(_brSpawner1).reset(X, Y);
			ProjSpawner(_brSpawner1).trigger();
			ProjSpawner(_blSpawner1).reset(X, Y);
			ProjSpawner(_blSpawner1).trigger();
			ProjSpawner(_trSpawner1).reset(X, Y);
			ProjSpawner(_trSpawner1).trigger();
			ProjSpawner(_tlSpawner1).reset(X, Y);
			ProjSpawner(_tlSpawner1).trigger();
		}

		public function triggerProj2(X:Number, Y:Number):void {
			ProjSpawner(_brSpawner2).reset(X, Y);
			ProjSpawner(_brSpawner2).trigger();
			ProjSpawner(_blSpawner2).reset(X, Y);
			ProjSpawner(_blSpawner2).trigger();
			ProjSpawner(_trSpawner2).reset(X, Y);
			ProjSpawner(_trSpawner2).trigger();
			ProjSpawner(_tlSpawner2).reset(X, Y);
			ProjSpawner(_tlSpawner2).trigger();
		}

		private function blockerMovement():void {
			if(_blockerOpenning1) {
				if(_blocker1.x > x) {
					_blocker1.x -= GATES_X_MOVEMENT_FAST * FlxG.elapsed;
					_blocker2.x += GATES_X_MOVEMENT_FAST * FlxG.elapsed;
					_blocker1.setActive();
					_blocker2.setActive();
				} else {
					_blocker1.setPassive();
					_blocker2.setPassive();
				}
			} else if(_blockerAlive1) {
				if(_blocker1.x + _blocker1.width < _blocker2.x) {
					_blocker1.x += GATES_X_MOVEMENT_SLOW * FlxG.elapsed;
					_blocker2.x -= GATES_X_MOVEMENT_SLOW * FlxG.elapsed;
				}
				_blocker1.setActive();
				_blocker2.setActive();
			} else {
				_blocker1.setPassive();
				_blocker2.setPassive();
			}
			
			if(_blockerOpenning2) {
				if(_blocker3.y > y) {
					_blocker3.y -= GATES_Y_MOVEMENT_FAST * FlxG.elapsed;
					_blocker4.y += GATES_Y_MOVEMENT_FAST * FlxG.elapsed;
					_blocker3.setActive();
					_blocker4.setActive();
				} else {
					_blocker3.setPassive();
					_blocker4.setPassive();
				}
			} else if(_blockerAlive2) {
				if(_blocker3.y + _blocker3.height < _blocker4.y) {
					_blocker3.y += GATES_Y_MOVEMENT_SLOW * FlxG.elapsed;
					_blocker4.y -= GATES_Y_MOVEMENT_SLOW * FlxG.elapsed;	
				}
				_blocker3.setActive();
				_blocker4.setActive();
			} else {
				_blocker3.setPassive();
				_blocker4.setPassive();
			}
			
			if(_blockerOpenning3) {
				if(_blocker5.x > x) {
					_blocker5.x -= GATES_X_MOVEMENT_FAST * FlxG.elapsed;
					_blocker6.x += GATES_X_MOVEMENT_FAST * FlxG.elapsed;
					_blocker5.setActive();
					_blocker6.setActive();
				} else {
					_blocker5.setPassive();
					_blocker6.setPassive();
				}
			}
			else if(_blockerAlive3) {
				if(_blocker5.x + _blocker5.width < _blocker6.x) {
					_blocker5.x += GATES_X_MOVEMENT_SLOW * FlxG.elapsed;
					_blocker6.x -= GATES_X_MOVEMENT_SLOW * FlxG.elapsed;	
				}
				_blocker5.setActive();
				_blocker6.setActive();
			} else {
				_blocker5.setPassive();
				_blocker6.setPassive();
			}
			
			if(_blockerOpenning4) {
				if(_blocker7.y > y) {
					_blocker7.y -= GATES_Y_MOVEMENT_FAST * FlxG.elapsed;
					_blocker8.y += GATES_Y_MOVEMENT_FAST * FlxG.elapsed;
					_blocker7.setActive();
					_blocker8.setActive();
				} else {
					_blocker7.setPassive();
					_blocker8.setPassive();
				}
			} else if(_blockerAlive4) {
				if(_blocker7.y + _blocker7.height < _blocker8.y) {
					_blocker7.y += GATES_Y_MOVEMENT_SLOW * FlxG.elapsed;
					_blocker8.y -= GATES_Y_MOVEMENT_SLOW * FlxG.elapsed;
				}
				_blocker7.setActive();
				_blocker8.setActive();
			} else {
				_blocker7.setPassive();
				_blocker8.setPassive();
			}
		}

		public function triggerLaser(X:Number = 0, Y:Number = 0 ):void {
			//ProjSpawner(_laserSpawner1).reset(X, Y);
			_laserSpawner1.trigger();
			_laserSpawner2.trigger();
			_laserSpawner3.trigger();
			_laserSpawner4.trigger();
		}
	}
}