package bosses.jellyfish {
	import base.ProjSpawner;

	import bosses.Boss;

	import player.PlayerShip;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	import flash.geom.Point;

	/**
	 * @author Yori Kvitchko
	 */
	public class JellyFishBoss extends Boss {

		private static const BIG_INTERVAL:Number = 0.8;
		private static const BIG_BOOST:Number = 150;
		private static const SMALL_INTERVAL:Number = 1;
		private static const SMALL_BOOST:Number = 300;		
		private static const BABY_INTERVAL:Number = 0.4;

		private var _time:Number;
		private var _spawner1:JellyFishBabySpawner;
		private var _spawner2:JellyFishBabySpawner;
		private var _spawner3:JellyFishBabySpawner;
		private var _spawner4:JellyFishBabySpawner;

		private var _comp:JellyFishBossComp;

		private var _ship:PlayerShip;

		private	var projX:Number = 0;
		private	var projY:Number = 0;	

		private var _interval:Interval;
		private var _movementInterval:Interval;
		private var isSpawning:Boolean;

		private static var _tmp:Point;

		private var _hitBoundary:Boolean = true;

		public function JellyFishBoss(Projectiles:FlxGroup) {
			
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2 - 250, 200, 200, 200, 1300);
			
			_time = 0;
			
			_comp = new JellyFishBossComp(Projectiles, 0, 0, this);
			add(_comp);
			
			_spawner1 = new JellyFishBabySpawner(0, 0, BABY_INTERVAL, this);
			_spawner1.addLocation(this.width / 2 - 80, this.height / 2);
			_spawner1.stop();
			Projectiles.add(_spawner1.projectilePool);
			add(_spawner1);
			
			_spawner2 = new JellyFishBabySpawner(0, 0, BABY_INTERVAL, this);
			_spawner2.addLocation(this.width / 2 - 80, this.height / 2 - 50);
			_spawner2.stop();
			Projectiles.add(_spawner2.projectilePool);
			add(_spawner2);
			
			isSpawning = false;
			_interval = new Interval([BABY_INTERVAL]);
			_movementInterval = new Interval([SMALL_INTERVAL, SMALL_INTERVAL, SMALL_INTERVAL, SMALL_INTERVAL]);
			_tmp = new Point(0, 0);
			
			moves = true;
			
			drag.x = 200;
			drag.y = 200;
		}

		override public function update():void {
						
			//follow player ship (find x and y that cut player ship off)
			var shipHeadingX:Number = SpaceView.ship.velocity.x + SpaceView.ship.x;
			var shipHeadingY:Number = SpaceView.ship.velocity.y + SpaceView.ship.y;
		
			_time += FlxG.elapsed;
			_interval.elapse(FlxG.elapsed);
			
			if(_interval.justTicked) {
				// Switch Spawners
				isSpawning = !isSpawning;
				
				// Stop/Start as appropriate
				if(isSpawning) {
					ProjSpawner(_spawner1).start();
					//ProjSpawner(_spawner2).start();
				} else {
					ProjSpawner(_spawner1).stop();
					//ProjSpawner(_spawner2).stop();
				}
			}	
			
			//hit right bounadry
			if((this.x < -20 || this.x + this.width > 820)) {	
				velocity.x = -velocity.x;
			}
			if((this.y < -20 || this.y + this.height > 820)) {
				velocity.y = -velocity.y;
			}
			
			_movementInterval.elapse(FlxG.elapsed);
			if(_movementInterval.justTicked) {
				// Set _tmp to the difference between my location and ship's location
				_tmp.x = shipHeadingX - this.x;
				_tmp.y = shipHeadingY - this.y;
				
				// If the current interval is the first interval (which is the BIG_INTERVAL) give me a BIG_BOOST
				// Otherwise just use a SMALL_BOOST
				// This is done by normalizing the difference vector stored in _tmp to the BOOST value we want
				if(_movementInterval.currentInterval == 0)
					_tmp.normalize(BIG_BOOST);
				else
					_tmp.normalize(SMALL_BOOST);
				
				// Add the boost to the current velocity
				velocity.x += _tmp.x;
				velocity.y += _tmp.y;	
			}
		
		
				
			super.update();
		}
	}
}
