package bosses.squid {
	import base.ProjSpawner;

	import bosses.BossComp;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxPoint;

	/**
	 * @author Yori Kvitchko
	 */
	public class SquidBossComp extends BossComp {
		// CONSTANTS
		
		/*
		 * Fire interval for bullets
		 */
		private static const BULLET_INTERVAL:Number = 0.15;
		/*
		 * Interval for "waves" of bullets
		 */
		private static const BULLET_WAVE_INTERVAL:Number = 2;

		// VARIABLES
		/*
		 * Timer for Bullet Waves
		 */
		private var _waveTimer:Number;
		/*
		 * Spawner for front bullets
		 */
		private var _bulletSpawner:ProjSpawner;
		/*
		 * Spawner for side bullets
		 */
		private var _sideBulletSpawner:ProjSpawner;
		/*
		 * We'll use polar coordinates for the squid's motion
		 * Theta (angle) in radians
		 */
		private var _theta:Number;
		/*
		 * Radius for polar coordinates
		 */
		private var _radius:Number;
		/*
		 * Center of polar coordinate system
		 */
		private var _center:FlxPoint;
		/*
		 * Pre-movement location, used to compute "velocity"
		 */
		private var _last:FlxPoint;

		// EMBEDDED RESOURCES

		/*
		 * Animated squid graphic
		 */
		[Embed(source='../../../data/boss_squid.png')]
		private const ImgSquid:Class;

		public function SquidBossComp(Projectiles:FlxGroup, X:Number, Y:Number, Theta:Number, Radius:Number, Parent:FlxObject = null) {
			// Basic init
			super(Projectiles, X, Y, Parent);
			fixed = true;
			health = 250;
			
			// Initialize polar coordinate movement
			_center = new FlxPoint(X, Y);
			_last = new FlxPoint(X, Y);
			_theta = Theta;
			_radius = Radius;
			_waveTimer = BULLET_WAVE_INTERVAL;
			
			// Front Bullet Spawner
			_bulletSpawner = new SquidProjFrontSpawner(this.x, this.y, BULLET_INTERVAL, this);
			_bulletSpawner.addLocation(15, 30);
			// Add the spawner's projectile pool to the bullets group
			projectiles.add(_bulletSpawner.projectilePool);
			
			// Side Bullet Spawner
			_sideBulletSpawner = new SquidProjSideSpawner(this.x, this.y, BULLET_INTERVAL, this);
			_sideBulletSpawner.addLocation(15, 30);
			_sideBulletSpawner.addLocation(15, 30);
			_sideBulletSpawner.start();
			// Add the spawner's projectile pool to the bullets group
			projectiles.add(_sideBulletSpawner.projectilePool);
		}

		// Initialize Graphics		
		override public function initGraphics():void {
			// Animation and Graphic
			loadGraphic(ImgSquid, true, false, 105, 182);
			addAnimation("default", [0, 1, 2, 3, 4, 5, 6], 5, true);
			// Randomize which frame we start with
			_curFrame = Math.floor(Math.random() * 7);
			play("default");
			
			// Size and Collision
			// Size and collision
			width = 30;
			height = 60;
			offset.x = 30;
			offset.y = 80;
		}

		override public function update():void {
			// Wave Timer
			_waveTimer -= FlxG.elapsed;
			
			if(_waveTimer < 0) {
				_bulletSpawner.swap();
				_waveTimer = BULLET_WAVE_INTERVAL;
			}
			
			// Move based on polar coordinates
			_last.x = x;
			_last.y = y;
			x = _center.x + Math.cos(_theta) * _radius - origin.x;
			y = _center.y + Math.sin(_theta) * _radius - origin.y;
			// Since we're faking velocity make sure there's no leftover velocity for
			// the update function to work with
			velocity.x = 0;
			velocity.y = 0;
			
			// Rotate around
			_theta += 0.5 * FlxG.elapsed;
			
			// Update thyself
			super.update();
			
			// Set fake Velocity for use of anything that expects to see it (ONLY DO THIS IF YOU NEED TO USE VELOCITY FOR DIRECTION)
			velocity.x = (x - _last.x) / FlxG.elapsed;
			velocity.y = (y - _last.y) / FlxG.elapsed;
			
			//Fire bullets (this does not need to be done if the spawner 
			//is added to the boss's group as is ususally the case).
			_bulletSpawner.update();
			_sideBulletSpawner.update();
		}
	}
}
