package bosses.wave {
	import base.ProjSpawner;
	
	import bosses.Boss;
	
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	
	public class WaveBoss extends Boss
	{
		private static const ANGLE_INTERVAL:Number = 0.01;
		private static const WAVE_OFFSCREEN_X:Number = 400;
		private static const WAVE_OFFSCREEN_Y:Number = 400;
		/**
		 * Time spent at each clover piece
		 **/
		private static const CLOVER_INTERVAL:Number = 2;
		private static const CORE_MOVE_X:Number = 200;
		private static const CORE_MOVE_Y:Number = 200;
		private static const WEAPON_MOVE_X:Number = 200;
		private static const WEAPON_MOVE_Y:Number = 200;
		
		private static const HALF_PI:Number = Math.PI/2;
		
		private var _time:Number;
		private var _switchInterval:Interval;
		private var _xDelta:Number;
		private var _yDelta:Number;
		private var _xDeltaInterval:Number;
		private var _yDeltaInterval:Number;
		
		private var _xInitial:Number;
		private var _yInitial:Number;
		
		private var _radius:Number;
		private var _angleParam:Number;
		private var _angleIncrement:Number;
		
		private var _bossHitWall:Boolean;
		
		/**
		 * timer for controlling the movement
		 **/
		private var _gameTimer:Number;
		
		/**
		 * angle timer for triggering a growing projectile
		 **/
		private var _growProjAngleTimer:Number;
		
		/**
		 * switching between horizontal and vertical
		 **/
		private var _horizontalClover:Boolean;
		private var _verticalClover:Boolean;
		
		private var _core:WaveBossComp;
		private var _weaponh:WaveBossWeaponComp;
		private var _weaponv:WaveBossWeaponComp;
		
		/**
		 *  The explosions
		 **/
		private var _explodeSpawner:WaveExplodeSpawner;
		private var _growSpawner:WaveGrowSpawner;
		
		/**
		 * 	0 stands for the clover movement
		 *  1 stands for the wave attack
		 **/
		private var _phase:int;
		private var _crossCount:int;
		private var _waveCount:int;
		
		public function WaveBoss(Projectiles:FlxGroup)
		{
			super(Projectiles, 
			300, 
			300,
			200,
			200,
			1250);
			
			_xInitial = x;
			_yInitial = y;
			
			_time = 0;
			
			_xDelta = 0;
			_yDelta = 0;
			_xDeltaInterval = 200;
			_yDeltaInterval = 200;
			
			_angleParam = -HALF_PI;
			_growProjAngleTimer = -HALF_PI;
			
			_bossHitWall = false;
			
			/**
			 * number of angle intervals: (CLOVER_INTERVAL/2) / (ANGLE_INTERVAL)
			 * _angleIncrement = (Math.PI/2) / number of angle intervals
			 **/
			_angleIncrement = (Math.PI*ANGLE_INTERVAL)/CLOVER_INTERVAL;
			
			_gameTimer = ANGLE_INTERVAL;
			
			_horizontalClover = true;
			_verticalClover = false;
			
			_phase = 0;
			_crossCount = 0;
			_waveCount = 0;			
			
			_core = new WaveBossComp(Projectiles, 0, 0, this);
			add(_core);
			
			_weaponh = new WaveBossWeaponComp(Projectiles, 0 , 0, this, 0);
			_weaponh.visible = false;
			add(_weaponh);
			
			_weaponv = new WaveBossWeaponComp(Projectiles, 0, 0, this, 1); 
			_weaponv.visible = false;
			add(_weaponv);

			
			// Grow Spawner
			_growSpawner = new WaveGrowSpawner(x, y, this);
			_growSpawner.addLocation(100, 26); // 100, 26 is center (88, 14)
			_growSpawner.cycle = false;
			_growSpawner.stop();
			Projectiles.add(_growSpawner.projectilePool);
			add(_growSpawner);
			
			// Explosion Spawners
			_explodeSpawner = new WaveExplodeSpawner();
			// 12 locations (directions)
			for(var i:int = 0; i < 12; i++)
			{
				_explodeSpawner.addLocation(0, 0);			
			}
			_explodeSpawner.cycle = false;
			_explodeSpawner.stop();
			Projectiles.add(_explodeSpawner.projectilePool);
			add(_explodeSpawner);
		}
		
		override public function update():void
		{
			if(!_delayedKill)
			{
				_time += FlxG.elapsed;
						
				_gameTimer -= FlxG.elapsed;
				if(_gameTimer < 0)
				{
					// clover path movement
					if(_phase == 0)
					{
						/* _weaponh.visible = true; */
						
						setWeaponOffScreen();
						
						/**
						 * Increase the angle parameter for the curve
						 **/
						_angleParam += _angleIncrement;
						_growProjAngleTimer += _angleIncrement;
						
						// Movement
						/**
						 *  Switching between the horizontal and verical clovers
						 **/
						if(_angleParam >= 3*HALF_PI)
						{
							_horizontalClover = !_horizontalClover;
							_verticalClover = !_verticalClover;
							_angleParam = -HALF_PI;
							_crossCount++;
						}
						
						 if(_horizontalClover)
						{
							_xDelta = 300*Math.cos(_angleParam)/(1+Math.sin(_angleParam)*Math.sin(_angleParam));
							_yDelta = 1.1*Math.sin(_angleParam)*_xDelta; 
						} 
						
						if(_verticalClover)
						{
							_yDelta = -300*Math.cos(_angleParam)/(1+Math.sin(_angleParam)*Math.sin(_angleParam));
							_xDelta = 1.1*Math.sin(_angleParam)*_yDelta; 
						}
		
						reset(_xInitial+_xDelta, _yInitial+_yDelta);
//						if(x+200 >= 800)
//							trace(x);
//						else if(x <= 0)
//							trace(x);
//						if(y+200 >= 800)
//							trace("y: " + y);
//						else if(y <= 0)
//							trace("y: " + y);
						
						// Trigger a growing projectile
						if(_growProjAngleTimer >= 0)
						{
							triggerProj(x, y);
							_growProjAngleTimer = -HALF_PI;
						}
						if(_crossCount == 2)
						{
							_waveCount = Math.floor(Math.random()*4);
							//_waveCount = 3;
							resetWaveWeapon();
							_phase = 1;
						}
						
					}
					// wave attack
					else if(_phase == 1)
					{
						// top
						if(_waveCount == 0)
						{
							if(y > 0)
							{
								_weaponh.visible = false;
								_yDelta += CORE_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, _yInitial-_yDelta);
								//resetWaveWeapon();
							}
							else
							{
								if(!_bossHitWall)
								{
									resetWaveWeapon();
									FlxG.quake.start(0.01, 2);
									_bossHitWall = true;
								}
								_weaponh.visible = true;
								_weaponh.x = 0;
								_weaponh.y += WEAPON_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, 0);
								if(_weaponh.y >= 800)
								{
									_yDelta = 0;
									_phase = 2;
								}
							}
						}
						// right
						else if(_waveCount == 1)
						{
							if(x < 800-_core.width)
							{
								_weaponv.visible = false;
								_xDelta += CORE_MOVE_X*FlxG.elapsed;
								reset(_xInitial+_xDelta, _yInitial);
								resetWaveWeapon();
							}
							else
							{
								if(!_bossHitWall)
								{
									resetWaveWeapon();
									FlxG.quake.start(0.01, 1);
									_bossHitWall = true;
								}
								_weaponv.visible = true;
								_weaponv.x -= WEAPON_MOVE_X*FlxG.elapsed;
								_weaponv.y = 0;
								reset(800-_core.width, _yInitial);
								if(_weaponv.x <= 0)
								{
									_xDelta = 0;
									_phase = 2;
								}
							}						
						}
						// bottom
						else if(_waveCount == 2)
						{
							if(y < 800-_core.height)
							{
								_weaponh.visible = false;
								_yDelta += CORE_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, _yInitial+_yDelta);
								resetWaveWeapon();
							}
							else
							{
								if(!_bossHitWall)
								{
									resetWaveWeapon();
									FlxG.quake.start(0.01, 1);
									_bossHitWall = true;
								}
								_weaponh.visible = true;
								_weaponh.x = 0;
								_weaponh.y -= WEAPON_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, 800-_core.height);
								if(_weaponh.y <= 0)
								{
									_yDelta = 0;
									_phase = 2;
								}
							}
						}
						// left
						else if(_waveCount == 3)
						{
							if(x > 0)
							{
								_weaponv.visible = false;
								_xDelta += CORE_MOVE_X*FlxG.elapsed;
								reset(_xInitial-_xDelta, _yInitial);
								resetWaveWeapon();
							}
							else
							{
								if(!_bossHitWall)
								{
									resetWaveWeapon();
									FlxG.quake.start(0.01, 1);
									_bossHitWall = true;
								}
								_weaponv.visible = true;
								_weaponv.x += WEAPON_MOVE_X*FlxG.elapsed;
								_weaponv.y = 0;
								reset(0, _yInitial);
								if(_weaponv.x >= 800)
								{
									_xDelta = 0;
									_phase = 2;
								}
							}
						}
					}
					// move the boss back to the center
					else if(_phase == 2)
					{
						_bossHitWall = false;
						// top
						if(_waveCount == 0)
						{
							if(y < 300)
							{
								_yDelta += CORE_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, _yDelta);
							}
							else
							{
								resetClover();
							}
						}
						// right
						else if(_waveCount == 1)
						{
							if(x > 300)
							{
								_xDelta += CORE_MOVE_X*FlxG.elapsed;
								reset(800-_core.width-_xDelta, _yInitial);
							}
							else
							{
								resetClover();
							}
						}
						// bottom
						else if(_waveCount == 2)
						{
							if(y > 300)
							{
								_yDelta += CORE_MOVE_Y*FlxG.elapsed;
								reset(_xInitial, 800-_core.height-_yDelta);
							}
							else
							{
								resetClover();
							}
						}
						// left
						else if(_waveCount == 3)
						{
							if(x < 300)
							{
								_xDelta += CORE_MOVE_X*FlxG.elapsed;
								reset(_xDelta, _yInitial);
							}
							else
							{
								resetClover();
							}
						}
					}
					
					_gameTimer = ANGLE_INTERVAL;
				}
			}
			super.update();
		}
		
		private function resetWaveWeapon():void
		{
			// top
			if(_waveCount == 0)
			{
				_weaponh.x = 0;
				_weaponh.y = -_weaponh.height-WAVE_OFFSCREEN_Y;
			}
			// right
			else if(_waveCount == 1)
			{
				_weaponv.x = 800+WAVE_OFFSCREEN_X;
				_weaponv.y = 0;
			}
			// bottom
			else if(_waveCount == 2)
			{
				_weaponh.x = 0;
				_weaponh.y = 800+WAVE_OFFSCREEN_Y;
			}
			// left
			else if(_waveCount == 3)
			{
				_weaponv.x = -_weaponv.width-WAVE_OFFSCREEN_X;
				_weaponv.y = 0;
			}
		}
		
		private function resetClover():void
		{
			_crossCount = 0;
			_angleParam = -HALF_PI;
			_phase = 0;
			setWeaponOffScreen();
		}
		
		/**
		 * Move the weapons off screen
		 **/ 
		private function setWeaponOffScreen():void
		{
			_weaponh.x = -1000;
			_weaponh.y = -1000;
			_weaponv.x = -1000;
			_weaponv.y = -1000;
		}
		
		public function triggerExplosion(X:Number, Y:Number):void
		{
			ProjSpawner(_explodeSpawner).reset(X,Y);
			ProjSpawner(_explodeSpawner).trigger();
		}
		
		public function triggerProj(X:Number, Y:Number):void
		{
			//ProjSpawner(_growSpawner).reset(X,Y);
			ProjSpawner(_growSpawner).trigger();
		}
	}
}