package bosses.whip {
	import base.ProjSpawner;
	
	import bosses.Boss;
	
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class WhipBoss extends Boss {
		private static const BULLET_INTERVAL:Number = 0.10;
		private static const BULLET_INTERVAL_2:Number = 0.2;

		private static const SPIRAL_ROTATE_INTERVAL:Number = 0.5;
		

		private var _phaseInterval:Interval;

		private var _core:WhipCoreBossComp;
		private var _spirals:Array;
		private var _spiralInterval:Interval;

		private var _tlCorner:WhipSpawner;
		private var _trCorner:WhipSpawner;
		private var _blCorner:WhipSpawner;
		private var _brCorner:WhipSpawner;

		/*
		 * Used for one-off explosions caused when the whip "cracks"
		 */
		private var _explodeSpawners:Array;
		
		/**
		 *  Added by Ming 04-24-2010
		 * 	used for cardioid movement
		 **/
		private static const CURVE_RADIUS:Number = 30;
		// Time spent at each period
		private static const CURVE_PERIOD:Number = 5;
		// How many seconds between each two sampling
		private static const SAMPLE_INTERVAL:Number = 0.01;
		private var _xInitial:Number;
		private var _yInitial:Number;
		private var _xDelta:Number;
		private var _yDelta:Number;
		private var _paramT:Number;
		private var _paramTIncr:Number;
		private var _movementInterval:Interval;

		public function WhipBoss(Projectiles:FlxGroup) {
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2 - 50, SpaceView.LEVEL_HEIGHT / 2 - 100, 100, 100, 750);
			
			_core = new WhipCoreBossComp(Projectiles, 0, 0, this);
			add(_core);

			var s:WhipSpiralBossComp;
			
			_spirals = new Array(4);
			
			s = new WhipSpiralBossComp(Projectiles, -4, -4, this);
			s.solid = false;
			s.angle = -90;
			_spirals[0] = add(s);
			
			s = new WhipSpiralBossComp(Projectiles, 88, -4, this);
			s.solid = false;
			s.angle = 0;
			_spirals[1] = add(s);
			
			s = new WhipSpiralBossComp(Projectiles, -4, 88, this);
			s.solid = false;
			s.angle = 90;
			_spirals[2] = add(s);
			
			s = new WhipSpiralBossComp(Projectiles, 88, 88, this);
			s.solid = false;
			s.angle = 180;
			_spirals[3] = add(s);
			
			_spiralInterval = new Interval([SPIRAL_ROTATE_INTERVAL]);
			
			// Interval for switching between phases
			_phaseInterval = new Interval([5, 5]);
			
			// Top Left
			_tlCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 225, this);
			_tlCorner.addLocation(14, 14);
			_tlCorner.cycle = false;
			_tlCorner.start();
			Projectiles.add(_tlCorner.projectilePool);
			add(_tlCorner);
			
			// Top Right
			_trCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 315, this);
			_trCorner.addLocation(106, 14);
			_trCorner.cycle = false;
			_trCorner.start();
			Projectiles.add(_trCorner.projectilePool);
			add(_trCorner);
			
			// Bottom Left
			_blCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 135, this);
			_blCorner.addLocation(14, 106);
			_blCorner.cycle = false;
			_blCorner.start();
			Projectiles.add(_blCorner.projectilePool);
			add(_blCorner);
			
			// Bottom Right
			_brCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 45, this);
			_brCorner.addLocation(106, 106);
			_brCorner.cycle = false;
			_brCorner.start();
			Projectiles.add(_brCorner.projectilePool);
			add(_brCorner);
			
			// Explode Spawners
			_explodeSpawners = new Array(4);
			
			var _explodeSpawner:WhipExplodeSpawner;
			
			for(var i:int = 0;i < 4;i++) {
			
				_explodeSpawner = new WhipExplodeSpawner();
				// 12 locations
				for(var j:int = 0;j < 12;j++) {
					_explodeSpawner.addLocation(0, 0);
				}
				_explodeSpawner.cycle = false;
				_explodeSpawner.stop();
				Projectiles.add(_explodeSpawner.projectilePool);
				add(_explodeSpawner);
				
				_explodeSpawners[i] = _explodeSpawner;
			}
			
			/**
			 *  Added by Ming 04-24-2010
			 * 	used for cardioid movement
			 **/
			_xInitial = x;
			_yInitial = y;
			_xDelta = 0;
			_yDelta = 0;
			
			/**
			 *	Number of samples: CURVE_PERIOD / SAMPLE_INTERVAL
			 * 	_paramTIncr: (2*Math.PI) / Number of samples
			 **/
			_paramTIncr = 2*Math.PI*SAMPLE_INTERVAL / CURVE_PERIOD;
			_paramT = 0;
			_movementInterval = new Interval([SAMPLE_INTERVAL]);
		}

		
		override public function update():void {
			if(!_delayedKill) {
				_phaseInterval.elapse(FlxG.elapsed);
				_spiralInterval.elapse(FlxG.elapsed);
				_movementInterval.elapse(FlxG.elapsed);
			
				/**
				 * 	Added by Ming 04-24-2010
				 *  Movement
				 **/
				if(_movementInterval.justTicked) {
					_paramT += _paramTIncr;
					_xDelta = CURVE_RADIUS * Math.sin(_paramT) * (1 - Math.cos(_paramT));
					_yDelta = -CURVE_RADIUS * Math.cos(_paramT) * (1 - Math.cos(_paramT));
				}
				reset(_xInitial + _xDelta, _yInitial + _yDelta);
			
				if(_spiralInterval.justTicked) {
					for (var i:int = 0;i < _spirals.length;i++) {
						WhipSpiralBossComp(_spirals[i]).angle += 90;
					}
				}
			
				if(_phaseInterval.justTicked) {
					if(_phaseInterval.currentInterval) {
						_tlCorner.interval = _trCorner.interval = _blCorner.interval = _brCorner.interval = BULLET_INTERVAL_2;
					} else {
						_tlCorner.interval = _trCorner.interval = _blCorner.interval = _brCorner.interval = BULLET_INTERVAL;
					}
				}
			}
			super.update();
		}

		public function triggerExplosion(X:Number, Y:Number, index:int = 0):void {
			ProjSpawner(_explodeSpawners[index]).reset(X, Y);
			ProjSpawner(_explodeSpawners[index]).trigger();
		}
	}
}
