package bosses.whip {
	import base.Proj;
	import base.ProjSpawner;

	import org.flixel.FlxG;
	import org.flixel.FlxObject;
	import org.flixel.FlxU;

	/**
	 * @author Yori Kvitchko
	 */
	public class WhipSpawner extends ProjSpawner {
		private static const HALF_PI:Number = Math.PI / 2;
		
		private var _time:Number;
		/*
		 * Starting angle
		 */
		private var _startAngle:Number;
		/*
		 * Angle offset from starting
		 */
		private var _offsetAngle:Number;
		private var _angleSign:Number;
		private var _justSwitched:Boolean;
		
		public function WhipSpawner(X:Number, Y:Number, Interval:Number, startAngle:Number, Parent:FlxObject = null) {
			super(X, Y, Interval, WhipProj, 15, Parent);
			_startAngle = startAngle;
			_time = -HALF_PI;
			_offsetAngle = 0;
			_angleSign = 1;
			
			_justSwitched = true;
		}

		
		override public function update():void {
			_time += FlxG.elapsed * 0.75;
			
			if(_time > HALF_PI) {
				_time -= Math.PI;
				_angleSign = -_angleSign;
				_justSwitched = true;
			}
			
			_offsetAngle = Math.sin(_time) * _angleSign * 60;
			
			super.update();
		}

		override public function sendProjectile(X:Number, Y:Number, p:Proj, locationIndex:int):void {
			var tmpAngle:Number;
			
			// Send it
			super.sendProjectile(X, Y, p, locationIndex);
			
			// Initial Angle
			tmpAngle = _startAngle + _offsetAngle;
			
			// Set Parameters
			Proj(p).velocity = FlxU.rotatePoint(100, 0, 0, 0, tmpAngle);
			// Set Angle
			Proj(p).angle = tmpAngle;
			
			// Exploding bullet on switching rotations
			if(_justSwitched) {
				// Which exploder to use
				WhipProj(p).exploding = Math.floor((_startAngle - 45) / 90) + 1;
				_justSwitched = false;
			}
		}
	}
}
