package cinematics
{
	import org.flixel.FlxEmitter;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;
	
	public class Fireworks extends FlxGroup
	{
		// Constant
		private static const NUM_PARTICLE:Number = 15;
		
		private var _emitter:FlxEmitter;
		private var _animationInterval:Interval;
		private var _launched:Boolean;
		private var _tail:FlxSprite;
		private var _initX:Number;
		private var _initY:Number;
		
		[Embed(source="../../data/sounds/launch_firework.mp3")]
		private var SndFireworks:Class;

		[Embed(source="../../data/sounds/explosion_firework.mp3")]
		private var SndFireworksExplosion:Class;

		
		public function Fireworks(X:Number, Y:Number)
		{
			super();
			_initX = x = X;
			_initY = y = Y;
			
			_animationInterval = new Interval([1+0.5*Math.random(), 1]);
			
			_tail = new FlxSprite(0, 0);
			_tail.createGraphic(5, 20, 0xffffff00);
			_tail.visible = false;
			add(_tail);
			
			_emitter = new FlxEmitter();
			_emitter.gravity = 300;
			_emitter.setXSpeed(-150, 150);
			_emitter.setYSpeed(-100, 100);
//			_emitter.setSize(80+Math.random()*50, 50+Math.random()*50);
			_emitter.setSize(0, 0);
			var emitexplosion:Array = new Array(NUM_PARTICLE);
			for(var i:int = 0; i < NUM_PARTICLE; i++)
			{
				var tmpSprite:FlxSprite = new FlxSprite();
				tmpSprite.createGraphic(5, 5, 0xffffff00);
				emitexplosion[i] = tmpSprite;
			}
			_emitter.members = emitexplosion;
			
			add(_emitter);
			_launched = false;
		}
		
		public function launch():void
		{
			FlxG.play(SndFireworks);
			velocity.x = 0;
			velocity.y = -150-Math.random()*100;
			_tail.visible = true;
			_launched = true;
		}
		
		override public function update():void
		{
			super.update();
			if(_launched)
			{
				_animationInterval.elapse(FlxG.elapsed);
				
				if(_animationInterval.currentInterval == 1)
				{
					if(_animationInterval.justTicked)
					{
						// start the emitter
						_emitter.reset(x-_emitter.width/2,y-_emitter.height/2);
						_tail.visible = false;
						_emitter.start(true, 1);	
						FlxG.play(SndFireworksExplosion,1,false);
					}
				}
				if(_animationInterval.justLooped)
				{
					_emitter.stop();
					resetAll();
				}
			}
		}
		
		public function resetAll():void
		{
			reset(_initX, _initY);
			velocity.x = 0;
			velocity.y = 0;
			_tail.visible = false;
			_launched = false;
		}
		
		public function get getLaunched():Boolean
		{
			return _launched;
		}

	}
}