package cinematics{
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	/**
	 * @author Yori Kvitchko
	 */
	public class IntroState extends FadeState {
		// Intro data
		private var _introText:Array;
		private var _charInterval:Interval;

		private var _ship:FlxSprite;
		private var _thruster_left:FlxSprite;
		private var _thruster_right:FlxSprite;

		// Announcements / intro text
		private var _titleText:FlxText;

		// Internal Two Shot View
		private var _itsv:InternalTwoShotView;

		// Graphics
		[Embed(source="../../data/ship_intro.png")]
		private var ImgShipIntro:Class;
		
		[Embed(source="../../data/thrusters_l.png")]
		private var ImgThrustLeft:Class;

		[Embed(source="../../data/thrusters_r.png")]
		private var ImgThrustRight:Class;

		// Music
		[Embed(source="../../data/sounds/intro.mp3")] 
		private var SndIntro:Class;

		override public function create():void {
			
			// Internal
			_itsv = new InternalTwoShotView();
			_itsv.moveCamera(InternalTwoShotView.SUB_LOC_FAR_LEFT);
			_itsv._controlAfterCinematic = false;
			add(_itsv);
			
			// Ship
			_ship = new FlxSprite(-100, -100, ImgShipIntro);
			_thruster_left = new FlxSprite(0, 0, ImgThrustLeft);
			_thruster_left.visible = _thruster_left.moves = false;
			_thruster_right = new FlxSprite(0, 0, ImgThrustRight);
			_thruster_right.visible = _thruster_right.moves = false;
			_itsv.add(_thruster_left);
			_itsv.add(_thruster_right);
			_itsv.add(_ship);
			
			// Text
			_titleText = new FlxText(120, 180, 400, "\nEscape\n from\nPlanet Doom");
			_titleText.alignment = "center";
			_titleText.size = 36;
			add(_titleText);
			
			// Init intro text
			_introText = new Array(5);
			_charInterval = new Interval([4, 4, 4, 4, 4, 6]);
			
			// Captain
			_introText[0] = "Starring\nKAL BUTLER\nas\nThe Captain";
			// Viki
			_introText[1] = "\nVERONICA STONE\nas\nLt. Viki";
			// Fred
			_introText[2] = "\nPHIL PHINNIGAN\nas\nEng. Fred";
			// Droid
			_introText[3] = "\nBRO-807\nas\nBio Robotic Organism";
			// The Ship
			_introText[4] = "Also starring...\n\nTHE BOLT";
			
			// Fade initialize
			super.create();
			
			// Music
			FlxG.play(SndIntro);
			FlxG.mouse.hide();
		}

		override public function update():void {
			super.update();
			
			// Elapse Interval
			_charInterval.elapse(FlxG.elapsed);
			
			// End of Intro
			if(_charInterval.justLooped || _itsv.justSkipped) {
				fadeOut(0.5, true, Play);
				_itsv.justSkipped = false;
				return;
			}
			
			// Update thrusters
			_thruster_left.x = _thruster_right.x = _ship.x;
			_thruster_left.y = _thruster_right.y = _ship.y;
			_thruster_left.scale.x = _thruster_right.scale.x = _ship.scale.x;
			_thruster_left.scale.y = _thruster_right.scale.y = _ship.scale.y;
			
			// Increase ship size
			if(_charInterval.currentInterval == 5 && _ship.velocity.x == 0) {
				_ship.scale.y = _ship.scale.x *= 1.02; 
				_ship.x += _ship.scale.x;
				_thruster_left.visible = true;
				_thruster_right.visible = false;
			} else if (_ship.x > 200) {
				_thruster_left.visible = false;
				_thruster_right.visible = true;
			}
			if(_ship.scale.x > 8) {
				_ship.visible = false;
				_thruster_left.visible = false;
				_thruster_right.visible = false;
			}
			
			// Character intro
			if(_charInterval.justTicked) {
				// Remove old character
				if(_charInterval.currentInterval > 1) {
					_itsv.moveCharacter(_charInterval.currentInterval - 2, InternalTwoShotView.SUB_LOC_OFF);
				}
				
				if(_charInterval.currentInterval < 5) {
					// Place new character (left or right side)
					if(_charInterval.currentInterval % 2 == 0) {
						_titleText.x = 220;
						_titleText.text = _introText[_charInterval.currentInterval - 1];
						_itsv.moveCharacter(_charInterval.currentInterval - 1, InternalTwoShotView.SUB_LOC_FAR_LEFT);
						_itsv.slideCharacter(_charInterval.currentInterval - 1, 100, 4);
					} else {
						_titleText.x = 20;
						_titleText.text = _introText[_charInterval.currentInterval - 1];
						_itsv.moveCharacter(_charInterval.currentInterval - 1, InternalTwoShotView.SUB_LOC_CENTER);
						_itsv.slideCharacter(_charInterval.currentInterval - 1, -100, 4);
					}
				} else {
					// The Ship
					_titleText.x = 120;
					_titleText.text = _introText[_charInterval.currentInterval - 1];
					
					_ship.x = -100;
					_ship.y = 285;
					_ship.velocity.x = 500;
					_ship.drag.x = 300;
				}
			}
		}

		private function Play():void {
			FlxG.state = new StartCinState();
		}
	}
}
