package cinematics {
	import org.flixel.FlxEmitter;
	import org.flixel.FlxG;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	/**
	 * @author Yori Kvitchko
	 */
	public class LoseCinState extends FadeState {
		// Visuals
		private var _text:FlxText;
		private var _bg:FlxSprite;
		private var _wormhole:FlxSprite;
		private var _shipParts:FlxEmitter;
		
		private var _stageInterval:Interval;

		// EMBEDDED RESOURCES
		[Embed(source="../../data/menu_bg.png")] 
		private var ImgBG:Class;

		[Embed(source="../../data/wormhole.png")] 
		private var ImgWormhole:Class;

		// Ship

		[Embed(source="../../data/ship_pieces.png")] 
		public var ImgShipPieces:Class;
		
		[Embed(source="../../data/sounds/doomstroyed_voice3.mp3")] 
		public var SndLosingVoice:Class;

		override public function create():void {
			// VISUALS
			
			// Background
			_bg = new FlxSprite(0, 0);
			_bg.loadGraphic(ImgBG);
			add(_bg);
			
			// Wormhole
			_wormhole = new FlxSprite(110, 460); // y=492 for exact center at bottom border
			_wormhole.loadGraphic(ImgWormhole, true, false, 420, 296);
			_wormhole.addAnimation("pulse", [0, 1, 2, 3, 4], 10);
			_wormhole.play("pulse");
			_wormhole.scale.x = _wormhole.scale.y = 0;
			add(_wormhole);
			
			// Ship pieces
			_shipParts = new FlxEmitter(_wormhole.x + _wormhole.origin.x, _wormhole.y + _wormhole.origin.y);
			_shipParts.minParticleSpeed.x = -75;
			_shipParts.minParticleSpeed.y = -100;
			_shipParts.maxParticleSpeed.x = 75;
			_shipParts.maxParticleSpeed.y = -50;
			_shipParts.minRotation = 100;
			_shipParts.maxRotation = 200;
			_shipParts.gravity = 0;
			_shipParts.createSprites(ImgShipPieces, 18, 16, true);
			add(_shipParts);
			
			// Text
			_text = new FlxText(0, 100, FlxG.width, "The crew of the Bolt is forever...\nLost Amongst the Stars");
			_text.size = 28;
			_text.alignment = "center";
			add(_text);

			// Stages
			// 0: Wormhole Grow
			// 1: Wormhole Spit
			// 2: Wormhole Shrink
			_stageInterval = new Interval([1, 1, 5, 1, 1]);

			super.create();
			
			// Background Color
			bgColor = 0xff000216;
			
			fadeIn(1, true);
			
			
			FlxG.play(SndLosingVoice);
		}

		override public function update():void {
			_stageInterval.elapse(FlxG.elapsed);
			
			if(_stageInterval.justLooped || (!fading && (FlxG.keys.justPressed("SPACE") || FlxG.mouse.justPressed()))) {
				fadeOut(1, true, End);
			}
			
			
			if(_stageInterval.currentInterval == 1) {
				_wormhole.scale.x = _wormhole.scale.y = _stageInterval.percentTicked;
			} else if(_stageInterval.currentInterval == 2) {
				if(_stageInterval.justTicked) {
					_wormhole.scale.x = _wormhole.scale.y = 1;
					_shipParts.start(true, 60);
				}
			} else if(_stageInterval.currentInterval == 3) {
				_wormhole.scale.x = _wormhole.scale.y = 1 - _stageInterval.percentTicked;
			}
			
			super.update();
		}

		private function End():void {
			FlxG.state = new MenuState();
		}
	}
}
