package cinematics {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	/**
	 * @author Yori Kvitchko
	 */
	public class StartCinState extends FadeState {
		// Timing
		private var _stageInterval:Interval;
		/*
		 * Used for text fade
		 */
		private var _textInterval:Interval;

		// Visuals
		private var _text:FlxText;
		private var _bg:FlxSprite;
		private var _planet:FlxSprite;
		private var _wormhole:FlxSprite;

		private var _shipGrp:FlxGroup;
		private var _ship:FlxSprite;
		private var _th_l:FlxSprite;
		private var _th_r:FlxSprite;
		private var _th_t:FlxSprite;
		private var _th_b:FlxSprite;

		private var _doomstroyer:FlxSprite;
		private var _doomstroyerTurret:FlxSprite;
		private var _doomstroyerLaser:FlxSprite;

		private var _explosions:ExplosionSet;

		// EMBEDDED RESOURCES
		[Embed(source="../../data/menu_bg.png")] 
		private var ImgBG:Class;

		[Embed(source="../../data/menu_exitbutton.png")] 
		private var ImgPlanet:Class;

		// Wormhole

		[Embed(source="../../data/wormhole.png")] 
		private var ImgWormhole:Class;

		// Doomstroyer

		[Embed(source='../../data/boss_doomstroyer_body.png')]
		private const ImgDoomstroyer:Class;

		[Embed(source='../../data/boss_doomstroyer_canon.png')]
		private const ImgDoomstroyerTurret:Class;

		[Embed(source='../../data/doom_laser.png')]
		private const ImgDoomstroyerLaser:Class;

		// Ship

		[Embed(source="../../data/ship_externalview_body.png")] 
		public var ImgShip:Class;

		[Embed(source="../../data/thrusters_t.png")]
		private var ImgThrustTop:Class;

		[Embed(source="../../data/thrusters_b.png")]
		private var ImgThrustBot:Class;

		[Embed(source="../../data/thrusters_l.png")]
		private var ImgThrustLeft:Class;

		[Embed(source="../../data/thrusters_r.png")]
		private var ImgThrustRight:Class;
		
		[Embed(source="../../data/sounds/recap.mp3")]
		private var SndRecapMusic:Class;
		
		[Embed(source="../../data/sounds/powerup2.mp3")]
		private var SndPowerUp:Class;

		[Embed(source="../../data/sounds/lazer.mp3")]
		private var SndLazer:Class;
		
		[Embed(source="../../data/sounds/suckup.mp3")]
		private var SndSuckup:Class;


		override public function create():void {
			// VISUALS
			
			// Background
			_bg = new FlxSprite(0, 0);
			_bg.loadGraphic(ImgBG);
			add(_bg);
			
			// Planet
			_planet = new FlxSprite(0, 400);
			_planet.loadGraphic(ImgPlanet);
			add(_planet);
			
			// Wormhole
			_wormhole = new FlxSprite(110, 460); // y=492 for exact center at bottom border
			_wormhole.loadGraphic(ImgWormhole, true, false, 420, 296);
			_wormhole.addAnimation("pulse", [0, 1, 2, 3, 4], 10);
			_wormhole.play("pulse");
			_wormhole.exists = false;
			add(_wormhole);
			
			// Doomstroyer
			_doomstroyer = new FlxSprite(FlxG.width, 50, ImgDoomstroyer);
			add(_doomstroyer);
			// Turret
			_doomstroyerTurret = new FlxSprite(_doomstroyer.x + 100, _doomstroyer.y + 170);
			_doomstroyerTurret.loadRotatedGraphic(ImgDoomstroyerTurret, 32);
			add(_doomstroyerTurret);
			// Laser
			_doomstroyerLaser = new FlxSprite(277, 242, ImgDoomstroyerLaser);
			_doomstroyerLaser.exists = false;
			add(_doomstroyerLaser);
			
			// Explosions
			_explosions = new ExplosionSet();
			add(_explosions);
			
			// Ship
			_shipGrp = new FlxGroup();
			_ship = new FlxSprite(0, 0, ImgShip);
			_th_l = new FlxSprite(-32, -32, ImgThrustLeft);
			_th_r = new FlxSprite(-32, -32, ImgThrustRight);
			_th_t = new FlxSprite(-32, -32, ImgThrustTop);
			_th_b = new FlxSprite(-32, -32, ImgThrustBot);
			_th_l.visible = _th_r.visible = _th_t.visible = _th_b.visible = false;
			
			_shipGrp.add(_ship);
			_shipGrp.add(_th_l);
			_shipGrp.add(_th_r);
			_shipGrp.add(_th_t);
			_shipGrp.add(_th_b);
			
			_shipGrp.exists = false;
			add(_shipGrp);
			
			// Text
			_text = new FlxText(0, 100, FlxG.width, "Last time on...\n\nESCAPE FROM PLANET DOOM");
			_text.size = 32;
			_text.alpha = 0;
			_text.alignment = "center";
			_text.exists = false;
			add(_text);
			_textInterval = new Interval([1, 4, 1]);
			
			// 0: Nothing
			// 1: Last time on planet doom
			// 2: Show planet with doomstroyer AND fade out text
			// 3: Doomstroyer powers up cannon
			// 4: Ship starts escapes
			// 5: Fire cannon and Black hole where planet used to be
			// 6: Both sucked in
			// 7: Wormhole shring
			// 8: Fade out
			_stageInterval = new Interval([0.5, 1.5, 7, 3, 4, 3, 6, 2.5, 0.5]);

			super.create();
			
			// Background Color
			bgColor = 0xff000216;
			
			// Music
			FlxG.playMusic(SndRecapMusic);
			
			fadeIn(1, false);
		}

		override public function update():void {
			// Move time forward
			_stageInterval.elapse(FlxG.elapsed);
			
			// Text Fading
			if(_text.exists) {
				_textInterval.elapse(FlxG.elapsed);
				
				if(_textInterval.currentInterval == 0) {
					// Fade in
					_text.alpha = _textInterval.percentTicked;
				} else if (_textInterval.currentInterval == 1) {
					// Stay put
					_text.alpha = 1;
				} else {
					// Fade out
					_text.alpha = 1 - _textInterval.percentTicked;
				}
				
				// Turn off after we're done looping
				if(_textInterval.justLooped) {
					_text.exists = false;
				}
			}

			// Stages
			if(_stageInterval.currentInterval == 1) {
				// Start Fading Text
				if(_stageInterval.justTicked) {
					_text.exists = true;
				}
			} else if(_stageInterval.currentInterval == 2) {
				// Start sliding doomstroyer over
				if(_stageInterval.justTicked) {
					_doomstroyer.velocity.x = -50;
					_doomstroyerTurret.velocity.x = -50;
				}
			} else if(_stageInterval.currentInterval == 3) {
				if(_stageInterval.justTicked) {
					// Stop sliding doomstroyer over
					_doomstroyer.velocity.x = 0;
					_doomstroyerTurret.velocity.x = 0;
					// Start powering up cannon
					_doomstroyerTurret.angularAcceleration = 100;
					_doomstroyerTurret.maxAngular = 3000;
				
					FlxG.play(SndPowerUp)
				}
			} else if(_stageInterval.currentInterval == 4) {
				if(_stageInterval.justTicked) {
					// Ship starts escaping
					_shipGrp.reset(200, 550);
					_shipGrp.velocity.x = -30;
					_shipGrp.velocity.y = -100;
					scaleShip(0);
					_explosions.explodeSingle(_ship.x + _ship.origin.x, _ship.y + _ship.origin.y);
				}
				// Scale the ship up
				if(_ship.scale.x < 1) {
					scaleShip(_ship.scale.x + FlxG.elapsed);
				}
				// Create Laser
				if(_stageInterval.percentTicked > 0.5 && !_doomstroyerLaser.exists) {
					_doomstroyerLaser.exists = true;
					FlxG.play(SndLazer);
				}
			} else if(_stageInterval.currentInterval == 5) {
				if(_stageInterval.justTicked) {
					// Set ship to scale 1 just in case
					scaleShip(1);
					// Start sucking ship in
					_shipGrp.acceleration.x = 20;
					_shipGrp.acceleration.y = 50;
					// Start quake and flash
					FlxG.quake.start(0.05, 3);
					FlxG.flash.start(0xffffffff, 3);
					// Create wormhole
					_wormhole.exists = true;
					_wormhole.scale.x = _wormhole.scale.y = 0.1;
					// Blow up the turret
					_explosions.explode(_doomstroyerTurret.x, _doomstroyerTurret.y, _doomstroyerTurret.width, _doomstroyerTurret.height, 2);
				}
				// Alpha out planet
				_planet.alpha = 1 - _stageInterval.percentTicked;
				_doomstroyerLaser.alpha = 1 - _stageInterval.percentTicked;
				// Grow wormhole
				_wormhole.scale.x = _wormhole.scale.y += FlxG.elapsed * 0.1;
			} else if(_stageInterval.currentInterval == 6) {
				if(_stageInterval.justTicked) {
					// Suck in everything

					// Remove planet and laser
					_planet.exists = false;
					_doomstroyerLaser.exists = false;

					// Suck ship in
					_shipGrp.acceleration.x = 10;
					_shipGrp.acceleration.y = 50;
					// Suck doomstroyer in
					_doomstroyerTurret.angularAcceleration = 0;
					_doomstroyerTurret.angularDrag = 500;
					_doomstroyer.acceleration.x = -15;
					_doomstroyer.acceleration.y = 50;
					_doomstroyerTurret.acceleration.x = -30;
					_doomstroyerTurret.acceleration.y = 100;
				}
				// Grow wormhole
				if(_wormhole.scale.x < 1) {
					_wormhole.scale.x = _wormhole.scale.y += FlxG.elapsed * 0.2;
				}
				FlxG.play(SndSuckup, 1);
				// Shrink ship, doomstroyer, and turret
				scaleShip(_ship.scale.x - FlxG.elapsed * 0.2);
				_doomstroyer.scale.x = _doomstroyer.scale.y -= FlxG.elapsed * 0.2;
				_doomstroyerTurret.scale.x = _doomstroyerTurret.scale.y -= FlxG.elapsed * 0.4;
				
				// Remove ship and doomstroyer once they're in the wormhole
				if(_ship.y > 570) {
					_ship.exists = false;
				}
				if(_doomstroyer.y > 480) {
					_doomstroyer.exists = false;
				}
				if(_doomstroyerTurret.y > 550) {
					_doomstroyerTurret.exists = false;
				}
			} else if(_stageInterval.currentInterval == 7) {
				if(_stageInterval.justTicked) {
					// Remove ship and doomstroyer
					_ship.exists = false;
					_doomstroyer.exists = false;
					_doomstroyerTurret.exists = false;
					
					// Make sure wormhole is full sized
					_wormhole.scale.x = _wormhole.scale.y = 1;
				}
				
				// Shrink wormhole
				if(_wormhole.scale.x > 0) {
					_wormhole.scale.x = _wormhole.scale.y -= FlxG.elapsed * 0.4;
				}
			} else if(_stageInterval.currentInterval == 8) {
				if(_stageInterval.justTicked) {
					// Remove wormhole
					_wormhole.exists = false;
				}
			}
			
			// End
			if(!fading && (FlxG.keys.justPressed("SPACE") || _stageInterval.justTicked && _stageInterval.currentInterval == (_stageInterval.numIntervals - 1))) {
				fadeOut(1, true, End);
			}
			
			super.update();
		}

		private function scaleShip(sc:Number):void {
			_ship.scale.x = _th_l.scale.x = _th_r.scale.x = _th_t.scale.x = _th_b.scale.x = sc;
			_ship.scale.y = _th_l.scale.y = _th_r.scale.y = _th_t.scale.y = _th_b.scale.y = sc;
		}

		private function End():void {
			if(FlxG.level == -1) {
				FlxG.state = new MenuState();
			} else {
				FlxG.state = new PlayState();
			}
		}
	}
}
