package player {
	import base.Proj;

	import org.flixel.FlxG;
	import org.flixel.FlxPoint;
	import org.flixel.FlxU;

	import flash.geom.Point;

	/**
	 * @author Yori Kvitchko
	 */
	public class PlayerProjSpecial extends Proj {
		// **** TESTING ****
		private static const BEZ_CENTER_OR_NORMAL:Boolean = false; // false = center, true = normal
		private static const BEZ_SHOW_PATH:Boolean = false;
		public static var _tmp:Point = new Point();
		// **** TESTING ****
		
		/*
		 * Bullet speed
		 */
		public static const BULLET_SPEED:Number = PlayerProj.BULLET_SPEED * 2;
		/*
		 * The extra allowance given to the player's mouse
		 * motion before "switching" the weapon speacial to the other side
		 */
		private static const SWAP_ALLOWANCE:Number = 5;

		public static var _bezierDistance:Number = 2;
		private static var _bossCenter:Point = new Point();
		private static var _shipCenter:Point = new Point();
		private static var _dotProduct:Number;
		private static var _leftOfCenter:Boolean = true;

		private var point_1:FlxPoint;
		private var point_2:FlxPoint;
		private var point_3:FlxPoint;
		private var _timeToTarget:Number;
		private var _last:Point;
		private var _bezierTimer:Number;

		[Embed(source="../../data/ship_bullet.png")] 
		private var ImgBullet:Class;

		public function PlayerProjSpecial() {
			super();
			moves = false; // Don't use built in physics

			point_1 = new FlxPoint();
			point_2 = new FlxPoint();
			point_3 = new FlxPoint();
			
			_last = new Point();
		}

		/*
		 * Initialize Graphics
		 */
		override public function initGraphics():void {
			loadRotatedGraphic(ImgBullet, 4);
		}

		override protected function removeOffScreen():void {
			// No off screen removal
		}

		/*
		 * Resets and sends the bullet on its way
		 */
		override public function send(X:Number, Y:Number, locationIndex:int):void {
			// Handles resetting for us
			super.send(X, Y, locationIndex);
			// Set up bezier plot points
			_bezierTimer = 0;
			
			// Center of the screen and ship
			var sv:SpaceView = SpaceView(FlxG.levels[FlxG.level]);
			
//			// Screen center point
//			_bossCenter.x = -FlxG.scroll.x + FlxG.width / 2;
//			_bossCenter.y = -FlxG.scroll.y + FlxG.height / 2;

			// Boss center point
			_bossCenter.x = SpaceView.enemy.x + SpaceView.enemy.origin.x;
			_bossCenter.y = SpaceView.enemy.y + SpaceView.enemy.origin.y;
			
			_shipCenter.x = SpaceView.ship.x + 20;
			_shipCenter.y = SpaceView.ship.y + 20;
			
			// Always start at the bullet spawner
			point_1.x = X;
			point_1.y = Y;
			
			if(BEZ_CENTER_OR_NORMAL) {
			
				// *** PERPENDICULAR TO SHIP->MOUSE DIRECTION SECOND POINT END AT MOUSE
				
				// We use _last as a temporary here for the vector between the mouse and ship
				_last.x = (FlxG.mouse.x - _shipCenter.x); // * _bezierDistance;
				_last.y = (FlxG.mouse.y - _shipCenter.y); // * _bezierDistance;
				_last.normalize(400);
				// Rotate the vector to point to the side
				FlxU.rotatePoint(_last.x, _last.y, 0, 0, 90, point_2);
				// Dot Product to determine if we need to invert the vector
				_dotProduct = point_2.x * (_bossCenter.x - _shipCenter.x) + point_2.y * (_bossCenter.y - _shipCenter.y);
				_dotProduct = 100000 / _dotProduct;
				
				if(_leftOfCenter && _dotProduct < 0) {
					if(_dotProduct > -SWAP_ALLOWANCE)
						_leftOfCenter = false;
				}
				if(!_leftOfCenter && _dotProduct > 0) {
					if(_dotProduct < SWAP_ALLOWANCE)
						_leftOfCenter = true;
				}
				
				if(_leftOfCenter) {
					point_2.x = -point_2.x;
					point_2.y = -point_2.y;
				}
				// Add our vector to the mouse position to get the second bezier point
				point_2.x += FlxG.mouse.x;
				point_2.y += FlxG.mouse.y;
				
				// End at the mouse
				point_3.x = FlxG.mouse.x;
				point_3.y = FlxG.mouse.y;
				
				// ******************************************************
			
			} else {
			
				// *** EXTEND TO MOUSE AS SECOND POINT END AT CENTER ***
	
				_last.x = _bossCenter.x - FlxG.mouse.x;
				_last.y = _bossCenter.y - FlxG.mouse.y;
				_last.normalize(-_last.length - 200);
				point_2.x = _bossCenter.x + _last.x;
				point_2.y = _bossCenter.y + _last.y;
				point_2.x = FlxG.mouse.x + _last.x;
				point_2.y = FlxG.mouse.y + _last.y;
	
	
				// End at Boss Center
				point_3.x = _bossCenter.x;
				point_3.y = _bossCenter.y;
	
				// ******************************************************

			}

			// Calculate very approximate distance for use in bullet speed calculations
			_timeToTarget = 0;
			_last.x = point_3.x - point_2.x;
			_last.y = point_3.y - point_2.y;
			_timeToTarget += _last.length;
			_last.x = point_2.x - point_1.x;
			_last.y = point_2.y - point_1.y;
			_timeToTarget += _last.length;
			_timeToTarget = Math.max(_timeToTarget, 300);
			// Divide by bullet_speed to get speed in the form of time it takes to get to destination
			_timeToTarget /= BULLET_SPEED;
			
			// Set last position
			_last.x = X;
			_last.y = Y;
			
			renderPath();
		}

		override public function update():void {
			_bezierTimer += FlxG.elapsed;
			super.update();
			_last.x = x;
			_last.y = y;
			x = interpolate(interpolate(point_1.x, point_2.x, _bezierTimer, _timeToTarget), interpolate(point_2.x, point_3.x, _bezierTimer, _timeToTarget), _bezierTimer, _timeToTarget);
			y = interpolate(interpolate(point_1.y, point_2.y, _bezierTimer, _timeToTarget), interpolate(point_2.y, point_3.y, _bezierTimer, _timeToTarget), _bezierTimer, _timeToTarget);
			angle = Math.atan2(y - _last.y, x - _last.x) * 180 / Math.PI;
		}

		private function interpolate(num1:Number, num2:Number, time:Number, maxTime:Number):Number {
			var weight:Number = time / maxTime;
			return (num1 * (1 - weight)) + (num2 * weight);
		}

		override public function render():void {
			super.render();
		}

		private function renderPath():void {
			if(BEZ_SHOW_PATH) {
				for (var i:int = 0;i < 500;i++) {
					_tmp.x = interpolate(interpolate(_shipCenter.x, point_2.x, i / 100.0, _timeToTarget), interpolate(point_2.x, point_3.x, i / 100.0, _timeToTarget), i / 100.0, _timeToTarget);
					_tmp.y = interpolate(interpolate(_shipCenter.y, point_2.y, i / 100.0, _timeToTarget), interpolate(point_2.y, point_3.y, i / 100.0, _timeToTarget), i / 100.0, _timeToTarget);
					FlxG.buffer.setPixel(_tmp.x + FlxG.scroll.x, _tmp.y + FlxG.scroll.y, 0xffffffff);
					FlxG.buffer.setPixel(_tmp.x + FlxG.scroll.x + 1, _tmp.y + FlxG.scroll.y, 0xffffffff);
					FlxG.buffer.setPixel(_tmp.x + FlxG.scroll.x, _tmp.y + FlxG.scroll.y + 1, 0xffffffff);
					FlxG.buffer.setPixel(_tmp.x + FlxG.scroll.x + 1, _tmp.y + FlxG.scroll.y + 1, 0xffffffff);
				}
			}
			
//			// Draw the points
//			for (var j:int = 0;j < 4;j++) {
//				for (var k:int = 0;k < 4;k++) {
//					FlxG.buffer.setPixel(j + point_1.x + FlxG.scroll.x, k + point_1.y + FlxG.scroll.y, 0xffffffff);
//					FlxG.buffer.setPixel(j + point_2.x + FlxG.scroll.x, k + point_2.y + FlxG.scroll.y, 0xffffffff);
//					FlxG.buffer.setPixel(j + point_3.x + FlxG.scroll.x, k + point_3.y + FlxG.scroll.y, 0xffffffff);
//				}
//			}
		}
	}
}
