package player {
	import base.Proj;
	import base.ProjSpawner;

	import bosses.gates.GatesLaserProjH;
	import bosses.gates.GatesLaserProjV;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSound;
	import org.flixel.FlxSprite;

	import flash.geom.Point;

	//This is the class declaration for the ship you fly
	public class PlayerShip extends FlxSprite {
		// CONSTANTS
		
		/*
		 * Bullet Damage
		 */
		public static const BULLET_DAMAGE:Number = 0.8;
		/*
		 * Single Upgraded bullet damage
		 */
		public static const BULLET_DAMAGE_SINGLE_UPGRADED:Number = 1.0;
		/*
		 * Double Upgraded bullet damage
		 */
		public static const BULLET_DAMAGE_DOUBLE_UPGRADED:Number = 1.2;
		/*
		 * Fire interval for bullets
		 */
		private static const BULLET_INTERVAL:Number = 0.03;
		/*
		 * Movement Speed
		 */
		private static const SHIP_SPEED:Number = 300;
		/*
		 * Single Upgraded Movement Speed
		 */
		private static const SHIP_SPEED_SINGLE_UPGRADED:Number = 350;
		/*
		 * Double Upgraded Movement Speed
		 */
		private static const SHIP_SPEED_DOUBLE_UPGRADED:Number = 400;
		/*
		 * Movement Acceleration
		 */
		private static const SHIP_ACCEL:Number = 4000;
		/*
		 * Movement Drag
		 */
		private static const SHIP_DRAG:Number = 1500;
		/*
		 * Duration of invincibility after being hurt
		 */
		private static const INVINICBILITY_DURATION:Number = 2;
		/*
		 * Invincibility is shorter after ejecting a character
		 */
		private static const INVINICBILITY_EJECT_DURATION:Number = 1;
		/*
		 * Time between keys which counts as a double tap
		 */
		//		private static const DOUBLE_TAP_TIME:Number = 0.2;
		/*
		 * Distance travelled by a teleport
		 */
		private static const TELEPORT_DISTANCE:Number = 200;
		/*
		 * Cooldown of the Nav Special, Teleport
		 */
		private static const TELEPORT_COOLDOWN:Number = 1;
		/*
		 * Cooldown of the Shield Special, Reflect
		 */
		private static const REFLECT_COOLDOWN:Number = 3;
		/*
		 * Cooldown of the Shield Special, Reflect
		 */
		private static const REFLECT_RADIUS:Number = 100;

		// VARIABLES
		
		/*
		 * Boolean for turning firing on/off
		 */
		private var _firing:Boolean;
		/*
		 * FlxGroup for storing list of bullets
		 */
		private var _bulletSpawner:ProjSpawner;
		/*
		 * Determines whether or not the ship is invincible (happens after being hit)
		 */
		private var _invincible:Boolean;
		/*
		 * Timer for invincibility
		 */
		private var _invincibilityTimer:Number;
		/*
		 * Key which was previously pressed
		 * Used to keep track of double tapping
		 */
		private var _doubleTapKey:String;
		/*
		 * Timer for double tap timing
		 */
		//		private var _doubleTapTimer:Number;
		/*
		 * Is the teleport special turned on?
		 */
		private var _teleportSpecial:Boolean;
		/*
		 * Timer for teleport cooldown
		 */
		private var _teleportTimer:Number;
		/*
		 * Stores temporary x teleport value
		 */
		private var _tx:int;
		/*
		 * Storese temporary y teleport value
		 */
		private var _ty:int;
		/*
		 * Damage done by bullets of the ship
		 */
		private var _damage:Number;
		/*
		 * Ship speed
		 */
		private var _speed:Number;
		/*
		 * Shield graphic
		 */
		private var _shield:FlxSprite;
		/*
		 * Reflect field graphic
		 */
		private var _reflectField:FlxSprite;
		/*
		 * Is the reflect special turned on?
		 */
		private var _reflectSpecial:Boolean;
		/*
		 * Timer for the duration of the reflect special
		 */
		private var _reflectTimer:Number;
		/*
		 * Variable for holding the firing sound
		 */
		private var _fireSound:FlxSound;

		// Thrusters
		private var _thruster_top:FlxSprite;
		private var _thruster_bot:FlxSprite;
		private var _thruster_left:FlxSprite;
		private var _thruster_right:FlxSprite;

		// TEMPORARY VARIABLES
		/*
		 * Temporary point used to conserve memory
		 */
		private var _tmp:Point = new Point();

		// EMBEDDED RESOURCES

		/*
		 * Ship Graphic
		 */
		[Embed(source="../../data/ship_externalview_body.png")] 
		public var ImgShip:Class;

		/*
		 * Ship Turret
		 */
		[Embed(source="../../data/ship_externalview_turret.png")] 
		public var ImgShipTurret:Class;

		/*
		 * Thruster graphics
		 */
		[Embed(source="../../data/thrusters_t.png")]
		private var ImgThrustTop:Class;

		[Embed(source="../../data/thrusters_b.png")]
		private var ImgThrustBot:Class;

		[Embed(source="../../data/thrusters_l.png")]
		private var ImgThrustLeft:Class;

		[Embed(source="../../data/thrusters_r.png")]
		private var ImgThrustRight:Class;

		/*
		 * Ship Shields Graphic
		 */
		[Embed(source="../../data/ship_shield.png")] 
		private var ImgShield:Class;
		
		/*
		 * Ship Reflect Field
		 */
		[Embed(source="../../data/ship_reflect.png")] 
		private var ImgReflect:Class;

		/*
		 * Ship Hit Crash Sound 
		 */
		[Embed(source="../../data/sounds/crash.mp3")] 
		private var SndCrash:Class;

		/*
		 * Fire sound
		 */
		[Embed(source="../../data/sounds/fire_loop.mp3")] 
		private var SndFire:Class;

		/*
		 * Reflect sound
		 */
		[Embed(source="../../data/sounds/reflect.mp3")] 
		private var SndReflect:Class;

		/*
		 * Teleport sound
		 */
		[Embed(source="../../data/sounds/teleport.mp3")] 
		private var SndTeleport:Class;
		
		/*
		 * Death sound
		 */
		[Embed(source="../../data/sounds/nooo.mp3")] 
		private var SndDeath:Class;
		
		/*
		 * Reflect reload sound
		 */
		[Embed(source="../../data/sounds/shield_reload.mp3")] 
		private var SndReflectReload:Class;
		

		/*
		 * Creates the ship at X, Y, using bullets as the group to add its bullets to
		 * UpgradeLevels is an array of 3 ints. Index of each determined by Script.as constants
		 * the upgrade level of each
		 * 0 = Regular
		 * 1 = Upgraded
		 * 2 = Special
		 */
		public function PlayerShip(X:Number, Y:Number, bullets:FlxGroup, UpgradeLevels:Array) {
			super();
			
			// Initialize Timer
			_invincibilityTimer = 0;
			//			_doubleTapTimer = 0;
			_teleportTimer = 0;
			_reflectTimer = 0;
			
			// Init Double Tap
			_doubleTapKey = "";

			// Initialize position
			loadRotatedGraphic(ImgShip, 1);
			this.x = X;
			this.y = Y;
			this.drag.x = SHIP_DRAG;
			this.drag.y = SHIP_DRAG;
			width = 36;
			height = 36;
			origin.x = 18;
			origin.y = 18;
			this.offset.x = 20;
			this.offset.y = 20;

			// Initialize Thrusters
			_thruster_top = new FlxSprite(0, 0, ImgThrustTop);
			_thruster_top.visible = _thruster_top.moves = false;
			_thruster_bot = new FlxSprite(0, 0, ImgThrustBot);
			_thruster_bot.visible = _thruster_bot.moves = false;
			_thruster_left = new FlxSprite(0, 0, ImgThrustLeft);
			_thruster_left.visible = _thruster_left.moves = false;
			_thruster_right = new FlxSprite(0, 0, ImgThrustRight);
			_thruster_right.visible = _thruster_right.moves = false;

			// Initialize Weapons Damage
			if(UpgradeLevels[Script.WEP] < 1) {
				// Regular Weapons
				_damage = BULLET_DAMAGE;
			} else if(UpgradeLevels[Script.WEP] < 2)  {
				// Upgraded Weapons
				_damage = BULLET_DAMAGE_SINGLE_UPGRADED;
			} else {
				// Upgraded Weapons
				_damage = BULLET_DAMAGE_DOUBLE_UPGRADED;
			}
			_firing = false;

			// Initialize Shields
			if(UpgradeLevels[Script.SHIELD] < 1) {
				// Regular Shields
				health = 2;
			} else if(UpgradeLevels[Script.SHIELD] < 2) {
				// Single Upgraded Shields
				health = 3;
			} else {
				// Double Upgraded Shields
				health = 4;
				// Special
				_reflectSpecial = (UpgradeLevels[Script.SHIELD] > 1);
			}
			
			// Shield graphic
			_shield = new FlxSprite(x, y);
			_shield.loadGraphic(ImgShield, true, false, 124, 124);
			
			// Reflect graphic
			_reflectField = new FlxSprite(x, y, ImgReflect);
			
			// Initialize Ship Speed
			if(UpgradeLevels[Script.NAV] < 1) {
				// Regular Navigation
				_speed = SHIP_SPEED;
			} else if(UpgradeLevels[Script.NAV] < 2) {
				// Single Upgraded Navigation
				_speed = SHIP_SPEED_SINGLE_UPGRADED;
			} else {
				// Single Upgraded Navigation
				_speed = SHIP_SPEED_DOUBLE_UPGRADED;
				// Special
				_teleportSpecial = (UpgradeLevels[Script.NAV] > 1);
			}
			
			// Initialize Bullet Spawner at our location with a pool and interval following ourselves
			if(UpgradeLevels[Script.WEP] < 2) {
				// Regular Bullets
				_bulletSpawner = new ProjSpawner(this.x, this.y, BULLET_INTERVAL, PlayerProj, 100, this);
			} else {
				// Special Bullets
				_bulletSpawner = new ProjSpawner(this.x, this.y, BULLET_INTERVAL, PlayerProjSpecial, 100, this);
			}
			var turret:FlxSprite;
			//1
			turret = new FlxSprite(50, 50);
			turret.loadGraphic(ImgShipTurret, true);
			turret.addAnimation("default", [0, 1], 4);
			turret.play("default");
			_bulletSpawner.addLocation(-10, -14, turret);
			//3
			turret = new FlxSprite();
			turret.loadGraphic(ImgShipTurret, true);
			turret.addAnimation("default", [0, 1], 4);
			turret.play("default");
			_bulletSpawner.addLocation(-10, 50, turret);
			//2
			turret = new FlxSprite(25, 25);
			turret.loadGraphic(ImgShipTurret, true);
			turret.addAnimation("default", [0, 1], 4);
			turret.play("default");
			_bulletSpawner.addLocation(46, -14, turret);
			//4
			turret = new FlxSprite();
			turret.loadGraphic(ImgShipTurret, true);
			turret.addAnimation("default", [0, 1], 4);
			turret.play("default");
			_bulletSpawner.addLocation(46, 50, turret);
			// done
			_bulletSpawner.cycle = true;
			// Add the bullet spawner's projectile pool to the bullets group
			bullets.add(_bulletSpawner.projectilePool);
		}

		//The main game loop function
		override public function update():void {
			//Timers
			_invincibilityTimer -= FlxG.elapsed;
			if(_teleportSpecial) {
				//				_doubleTapTimer -= FlxG.elapsed;
				_teleportTimer -= FlxG.elapsed;
			}
			if(_reflectSpecial) {
				// Reflect Timer
				if(_reflectTimer > 0) {
					_reflectTimer -= FlxG.elapsed;
				}
//				// Reflect Recharge
//				if(_reflectTimer < 0) {
//					_reflectTimer = 0;
//					FlxG.play(SndReflectReload);
//				}
				
				_reflectField.alpha = Math.max(0, _reflectTimer - (REFLECT_COOLDOWN - 1));
			}
			
			// Invincibility Logic
			if(_invincibilityTimer < 0)
				_invincible = false;
				
			// Reflect Logic
			if(_reflectSpecial && FlxG.mouse.justPressed() && _reflectTimer <= 0) {
				reflect();
			}
				
			//			// Double Tap Logic (Used for Teleport
			//			if(_teleportSpecial) {
			//				if(FlxG.keys.justPressed("A")) {
			//					if(_doubleTapKey == "A" && _doubleTapTimer > 0 && _teleportTimer < 0) {
			//						teleport(-1, 0);
			//					} else {
			//						_doubleTapKey = "A";
			//						_doubleTapTimer = DOUBLE_TAP_TIME;
			//					}
			//				}
			//				if(FlxG.keys.justPressed("S")) {
			//					if(_doubleTapKey == "S" && _doubleTapTimer > 0 && _teleportTimer < 0) {
			//						teleport(0, 1);
			//					} else {
			//						_doubleTapKey = "S";
			//						_doubleTapTimer = DOUBLE_TAP_TIME;
			//					}
			//				}
			//				if(FlxG.keys.justPressed("D")) {
			//					if(_doubleTapKey == "D" && _doubleTapTimer > 0 && _teleportTimer < 0) {
			//						teleport(1, 0);
			//					} else {
			//						_doubleTapKey = "D";
			//						_doubleTapTimer = DOUBLE_TAP_TIME;
			//					}
			//				}
			//				if(FlxG.keys.justPressed("W")) {
			//					if(_doubleTapKey == "W" && _doubleTapTimer > 0 && _teleportTimer < 0) {
			//						teleport(0, -1);
			//					} else {
			//						_doubleTapKey = "W";
			//						_doubleTapTimer = DOUBLE_TAP_TIME;
			//					}
			//				}
			//			}
			
			// Bezier Distance Logic (only used for the special weapon)
			//			if(FlxG.keys.Q)
			//				PlayerProjSpecial._bezierDistance -= 0.1;
			//			if(FlxG.keys.E)
			//				PlayerProjSpecial._bezierDistance += 0.1;
			//			if(PlayerProjSpecial._bezierDistance < -3)
			//				PlayerProjSpecial._bezierDistance = 0;
			//			if(PlayerProjSpecial._bezierDistance > 3)
			//				PlayerProjSpecial._bezierDistance = 3;
			
			// This is where we handle acceleration
			acceleration.x = 0;
			acceleration.y = 0;
			
			_tx = 0;
			_ty = 0;
			
			if(FlxG.keys.A) {
				acceleration.x -= SHIP_ACCEL;
				_thruster_right.visible = true;
				// Teleport
				_tx--;
			} else {
				_thruster_right.visible = false;
			}

			if(FlxG.keys.D) {
				acceleration.x += SHIP_ACCEL;
				_thruster_left.visible = true;
				// Teleport
				_tx++;
			} else {
				_thruster_left.visible = false;
			}

			if(FlxG.keys.W) {
				acceleration.y -= SHIP_ACCEL;
				_thruster_bot.visible = true;
				// Teleport
				_ty--;
			} else {
				_thruster_bot.visible = false;
			}

			if(FlxG.keys.S) {
				acceleration.y += SHIP_ACCEL;
				_thruster_top.visible = true;
				// Teleport
				_ty++;
			} else {
				_thruster_top.visible = false;
			}
			
			if((_tx || _ty) && _teleportSpecial && _teleportTimer < 0 && FlxG.keys.pressed("SPACE")) {
				teleport(_tx, _ty);
			}

			//Cap Velocity
			_tmp.x = velocity.x;
			_tmp.y = velocity.y;
			if(_tmp.length > _speed) {
				_tmp.normalize(_speed);
				velocity.x = _tmp.x;
				velocity.y = _tmp.y;
			}
      		
			//The all important parent game loop update - this calls FlxGroup.update()
			super.update();
			
			// Update the thrusters
			_thruster_top.x = _thruster_bot.x = _thruster_left.x = _thruster_right.x = x - 32;
			_thruster_top.y = _thruster_bot.y = _thruster_left.y = _thruster_right.y = y - 32;
			
			// Update the shield
			_shield.frame = health - 1;
			_shield.x = x - 44;
			_shield.y = y - 44;
			
			// Update the reflector graphic
			if(_reflectSpecial) {
				_reflectField.x = x - 82;
				_reflectField.y = y - 82;
			}
			
			//Fire bullets
			_bulletSpawner.update();
		}

		override public function render():void {
			// Render the thrusters
			if(_thruster_top.visible)
				_thruster_top.render();
			if(_thruster_bot.visible)
				_thruster_bot.render();
			if(_thruster_left.visible)
				_thruster_left.render();
			if(_thruster_right.visible)
				_thruster_right.render();
			// Render myself
			super.render();
			// Render the shield
			_shield.render();
			// Render the reflector field
			if(_reflectSpecial && active)
				_reflectField.render();
			
			//Render spawner sprites
			_bulletSpawner.render();
		}

		/*
		 * Teleport Logic
		 */
		private function teleport(X:int, Y:int):void {
			x += TELEPORT_DISTANCE * X;
			y += TELEPORT_DISTANCE * Y;
			fitInBounds();
			FlxG.play(SndTeleport);
			_teleportTimer = TELEPORT_COOLDOWN;
			acceleration.x = velocity.x = 0;
			acceleration.y = velocity.y = 0;
		}

		/*
		 * Reflect Logic
		 */
		private function reflect():void {
			if(health > 1) {
//				health--;
				reflectGroup(SpaceView.enemyProjectiles);
				FlxG.play(SndReflect);
				FlxG.play(SndReflectReload);
				_reflectTimer = REFLECT_COOLDOWN;
			}
		}

		/*
		 * Start firing your weapons
		 */
		public function startFiring():void {
			// Start playing and looping the firing sound
			_fireSound = FlxG.play(SndFire, 1, true);
			_bulletSpawner.start();
			_firing = true;
		}

		/*
		 * Stop firing weapons
		 */
		public function stopFiring():void {
			_fireSound.stop();
			_bulletSpawner.stop();
			_firing = false;
		}

		/*
		 * Recursively reflect a group and any groups under it
		 */
		private function reflectGroup(grp:FlxGroup):void {
			var grpl:int = grp.members.length;
			var p:Proj;
				
			for (var i:int = 0;i < grpl;i++) {
				if(grp.members[i] is FlxGroup) {
					reflectGroup(grp.members[i]);
				} else {
					p = Proj(grp.members[i]);
					if(p.exists && !p.reflected) {
						if(p is GatesLaserProjH) {
							// Custom code for horizontal gates laser
							_tmp.x = 0;
							_tmp.y = (p.y + p.origin.y) - (y + origin.y);
						} else if (p is GatesLaserProjV) {
							// Custom code for vertical gates laser
							_tmp.x = (p.x + p.origin.x) - (x + origin.x);
							_tmp.y = 0;
						} else {
							// All other projectiles
							_tmp.x = (p.x + p.origin.x) - (x + origin.x);
							_tmp.y = (p.y + p.origin.y) - (y + origin.y);
						}
						// Calculate distance and compare with reflect radius
						if((_tmp.length - (p.width > p.height ? p.height : p.width) / 2) <= REFLECT_RADIUS)
							p.reflect();
					}
				}
			}
		}

		/*
		 * Fits the ship into the bounds of the space view
		 * used after teleporting
		 */
		public function fitInBounds():void {
			if(x < 0)
				x = 0;
			if(y < 0)
				y = 0;
			if(x + width > SpaceView.LEVEL_WIDTH)
				x = SpaceView.LEVEL_WIDTH - width - 1;
			if(y + height > SpaceView.LEVEL_HEIGHT)
				y = SpaceView.LEVEL_HEIGHT - height - 1;
		}

		override public function hurt(Damage:Number):void {
			if(!_invincible) {
				// Quake
				FlxG.quake.start(0.01, 0.5);
				FlxG.play(SndCrash, 0.5);

				// Become invincible
				_invincible = true;
				
				if(health <= 1 && !SpaceView.ejectee.ejected) {
					// Eject
					PlayState(FlxG.state).sv.ejecteeEject();
					_invincibilityTimer = INVINICBILITY_EJECT_DURATION;
				} else {
					// Death sound
					if(health <= 1) {
						FlxG.play(SndDeath);
					}
					// Hurt
					super.hurt(Damage);
					_invincibilityTimer = INVINICBILITY_DURATION;
				}
				
				// Flicker
				this.flicker(_invincibilityTimer);
			}
		}

		override public function kill():void {
			dead = true;
			solid = false;
			active = false;
			flicker(-1);
		}

		/*
		 * Damage done by bullets of the ship
		 */
		public function get damage():Number {
			return _damage;
		}

		/*
		 * Boolean for turning firing on/off
		 */
		public function get firing():Boolean {
			return _firing;
		}

		/*
		 * Boolean for checking if the ship is currently invincible
		 */
		public function get invincible():Boolean {
			return _invincible;
		}

		/*
		 * Boolean for turning firing on/off
		 */
		public function set firing(value:Boolean):void {
			_firing = value;
		}
	}
}